-- Fenestra; version 0.80(beta); 1998-09-27

-- 
-- Copyright (c) 1998 Franck Arnaud, http://www.altsoft.demon.co.uk/
-- 
-- This package is distributed under the Eiffel Forum Freeware License, 
-- version 1 (see file forum.txt for full text). The main points are:
-- 
-- "Permission is hereby granted, without written agreement and without 
--  license or royalty fees, to use, copy, modify and/or distribute this 
--  package, provided that: copyright notices are retained unchanged [...]
-- 
--  Permission is hereby also granted [...] to distribute binary programs 
--  which depend on this package, provided that: if the binary program 
--  depends on a modified version of this package, you must publicly release 
--  the modified version of this package [...].
-- 
--  THIS PACKAGE IS PROVIDED "AS IS" AND WITHOUT WARRANTY [...]"
-- 

indexing
	description: "Minimal ELKS-95 style file class for SmallEiffel"
	
class FILE

creation
	make_open_read,
	make_open_write
	
feature {NONE} -- Creation

	make_open_read (a_file: STRING) is
		require
			not_void: a_file /= Void
		do
			!! in_file.connect_to (a_file)
		end

	make_open_write (a_file: STRING) is
		require
			not_void: a_file /= Void
		do
			!! out_file.connect_to (a_file)
		end

feature -- Status

	is_closed: BOOLEAN is
			-- Is file closed?
		do
			if in_file /= Void then
				Result := not in_file.is_connected
			elseif out_file /= Void then
				Result := not out_file.is_connected
			else
				Result := true
			end
		end

	end_of_file: BOOLEAN is
			-- End of file?
		require
			not_closed: not is_closed
		do
			Result := in_file.end_of_input
		end

	last_string: STRING is
			-- Last string.
		require
			not_closed: not is_closed
		do
			Result := in_file.last_string
		end

feature -- Actions

	read_line is
			-- Read a line.
		require
			not_closed: not is_closed
		do
			in_file.read_line
		end

	put_string (s: STRING) is
			-- Put string.
		require
			not_void: s /= Void
			not_closed: not is_closed
		do
			out_file.put_string (s)
		end

	close is
			-- Close file.
		require
			not_closed: not is_closed
		do
			if in_file /= Void then
				in_file.disconnect
			elseif out_file /= Void then
				out_file.disconnect
			end
			in_file := Void
			out_file := Void
		end

feature {NONE}
	
	in_file: STD_FILE_READ
	out_file: STD_FILE_WRITE

end -- class FILE

