-- Fenestra; version 0.80(beta); 1998-09-27

-- 
-- Copyright (c) 1998 Franck Arnaud, http://www.altsoft.demon.co.uk/
-- 
-- This package is distributed under the Eiffel Forum Freeware License, 
-- version 1 (see file forum.txt for full text). The main points are:
-- 
-- "Permission is hereby granted, without written agreement and without 
--  license or royalty fees, to use, copy, modify and/or distribute this 
--  package, provided that: copyright notices are retained unchanged [...]
-- 
--  Permission is hereby also granted [...] to distribute binary programs 
--  which depend on this package, provided that: if the binary program 
--  depends on a modified version of this package, you must publicly release 
--  the modified version of this package [...].
-- 
--  THIS PACKAGE IS PROVIDED "AS IS" AND WITHOUT WARRANTY [...]"
-- 

indexing
	cluster: Fenestra, Windows
	description: "Control - button, check box, list box, combo box, etc"
	interface: inheritance, abstract

class CONTROL

inherit
	DIALOG_ELEMENT
		end
	WIN_API
		end

creation
	-- effectively deferred

feature -- Implementation
	
	self: CHILD_WINDOW;
			-- Associated child window.

feature {NONE} -- Dialog box
	
	dbox: DIALOG

feature {NONE} -- Default value for default size

	Def_height: INTEGER is 14
	Def_width: INTEGER is 50
	Def_interspace: INTEGER is 4

feature {NONE} -- Base dialog units

	dunitspace: INTEGER -- dialog unit space
	dunits: DIALOG_UNITS

feature {PUBLIC_NONE}
 
	-- Control registration
	--
	-- A CHILD_WINDOW-based control should, in its creation feature: 
	--  a) Create `self', formally an heir of CHILD_WINDOW,
	--     preferably an heir of CONTROL_WINDOW for the keyboard 
	--     interface (eventually self = Current if multiple inheritance).
	--  a.1) If self is CONTROL_WINDOW, its `set_control' procedure 
	--     must be called in its own creation procedure or in the 
	--     corresponding control creation procedure. 
	--  b) Call 'register'.
	
	register, control_register is
			-- Register.
		require
			has_self: self /= Void
		do
			-- init dbox
			dbox ?= self.parent
			
			-- acquaint with dialog box
			if has_dialog_box then 
				-- add us to parent dialog
				dialog_box.add_control (Current)
			
				-- set our font
				if dialog_box.font /= Void then
					set_font (dialog_box.font)
				end
				
				-- set our dialog units
				dunits := dialog_box.dialog_units
				dunitspace := dialog_box.default_interspace
			else
				-- default base units
				!!dunits.make (wrt.win_dialog_unitx, wrt.win_dialog_unity)
				dunitspace := Def_interspace
			end

			-- set size
			default_size
		ensure
			done: dunits /= Void
		end

	default_size is
			-- Set default size.
		local
			r: RECTANGLE
			p: POINT
		do
			!!p.make
			p.set_xy (default_interspace, default_interspace)
			
			!!r.make
			r.translate_lower (p)
			r.set_width (default_width)
			r.set_height (default_height)
			indep_set_size (r)
		end

feature -- Dialog box
	
	has_dialog_box: BOOLEAN is
			-- Check if control has associated dialog box.
		do
			Result := (dbox /= Void)
		end

	dialog_box: DIALOG is
			-- Associated dialog which holds the control.
		do
			Result := dbox
		end

feature -- Control's font

	set_font (ft: FONT) is
			-- Set control's font.
		require
			valid_font: ft /= Void
		do
			self.device.set_font (ft)
		end

feature {PUBLIC_NONE} -- Default size
		
	default_interspace: INTEGER is
			-- Blank space between controls.
		do
			Result := dunitspace
		end

	default_width: INTEGER is
			-- Default width of a control.
		do
			Result := Def_width
		end

	default_height: INTEGER is
			-- Default heigth of a control.
		do
			Result := Def_height
		end

	estimate_width (str: STRING): INTEGER is
			-- Estimate width of label based on independent coordinates.
		do
			Result := 4 * (str.count + 1)
		end

feature -- Position, independent

	indep_size: RECTANGLE is
			-- Control size using dialog coordinates.
		do
			if isize = Void then
				isize := dunits.abs_to_indep (self.size)
			end
			!!Result.make
			Result.copy (isize)
		end

	indep_set_size (sz: RECTANGLE) is
			-- Set control size using dialog coordinates.
		do
			!!isize.make
			isize.copy (sz)
			self.set_size (dunits.indep_to_abs (sz))
		end

feature {NONE} -- (Internal)

	isize: RECTANGLE
			-- Independent size cache.

feature {PUBLIC_NONE} -- Set text
 
	set_text (s: STRING) is
			-- Change control text (incl. optional hotkey 
			-- preceded with &).
		require
			valid: s /= Void
		do
			set_hotkey (s)
			self.set_text (s)
		end


feature {PUBLIC_NONE} -- Control keyboard interface features 

	set_hotkey (s: STRING) is
			-- Find hotkey from string (preceded by '&').
		require
			valid_text: s /= Void
		local
			fmt: P_FORMAT
			idx: INTEGER
		do
			-- hotkey = voidchar
			if s.count > 0 then
				from
					idx := 1
				invariant
					upper_bound: idx <= s.count 
					lower_bound: idx >= 1
				variant
					s.count - idx
				until
					idx >= s.count -- don't check last char
				loop
					if s.item (idx) = '&' then
						!!fmt
						hotkey := fmt.to_lower (s.item (idx + 1))
					end
					idx := idx + 1
				end
			end
		end

feature {PUBLIC_NONE, DIALOG} --? export status
	
	tabstop: BOOLEAN is
			-- Stop on this control when <TAB>?
			-- (Default: true unless redefined).
		do
			Result := true
		end
	
	arrowstop: BOOLEAN is
			-- Stop on up/down arrow.
			-- (Default: false).
		do
			Result := false
		end

	hotkey: CHARACTER;
			-- Control hotkey.
		

feature {PUBLIC_NONE, CHILD_WINDOW} -- Dialog keyboard interface, events

	when_hotkey (c: CHARACTER) is
			-- Alt+<c>entered.
			-- Standard behaviour: focus control which has associated hotkey if any.
		do
			if has_dialog_box then
				dialog_box.keyboard_interface_hotkey (Current, c)
			end
		end

	when_tab is
			-- TAB entered.
			-- Standard behaviour: focus next tapstop control.
		do
			if has_dialog_box then
				dialog_box.keyboard_interface_tab (Current)
			end
		end

	when_shift_tab is
			-- Shift-TAB entered.
			-- Standard behaviour: focus previous control with tabstop.
		do
			if has_dialog_box then
				dialog_box.keyboard_interface_shift_tab (Current)
			end
		end

	when_down is
			-- Down arrow has been hit.
			-- Standard behaviour: focus previous control.
		do
			if has_dialog_box then
				dialog_box.keyboard_interface_down (Current)
			end
		end

	when_up is
			-- Up arrow has been hit.
			-- Standard behaviour: focus next control
		do
			if has_dialog_box then
				dialog_box.keyboard_interface_up (Current)
			end
		end

	when_enter is
			-- Enter has been hit.
			-- Standard behaviour: execute default push button.
		do
			if has_dialog_box then
				dialog_box.keyboard_interface_enter
			end
		end

	when_escape is
			-- Escape key has been hit.
			-- Standard behaviour: close dialog.
		do
			if has_dialog_box then
				dialog_box.keyboard_interface_escape
			end
		end

end -- class CONTROL
