-- Fenestra; version 0.80(beta); 1998-09-27

-- 
-- Copyright (c) 1998 Franck Arnaud, http://www.altsoft.demon.co.uk/
-- 
-- This package is distributed under the Eiffel Forum Freeware License, 
-- version 1 (see file forum.txt for full text). The main points are:
-- 
-- "Permission is hereby granted, without written agreement and without 
--  license or royalty fees, to use, copy, modify and/or distribute this 
--  package, provided that: copyright notices are retained unchanged [...]
-- 
--  Permission is hereby also granted [...] to distribute binary programs 
--  which depend on this package, provided that: if the binary program 
--  depends on a modified version of this package, you must publicly release 
--  the modified version of this package [...].
-- 
--  THIS PACKAGE IS PROVIDED "AS IS" AND WITHOUT WARRANTY [...]"
-- 

indexing
	cluster: Fenestra, Controls
	description: "Dialog - an ensemble of controls, combined with a WINDOW"
	interface: inheritance, mixin

deferred class DIALOG

inherit
	WIN_API -- for access to handle
		end
	COMMON_NOTIFIER
		end

feature {NONE} -- Init class (use by heirs' creator)

	make_dialog is
			-- Initialise this class, must be called by creator of heir. 
		do
			!!control_list.make
			set_dialog_units (wrt.win_dialog_unitx, wrt.win_dialog_unity)
		ensure
			init_ok: control_list /= Void
		end

feature {CONTROL} 
	
	font: FONT

feature -- Default dialog font used by all controls

	set_font (ft: FONT) is
			-- Set default font.
		require
			valid: ft /= Void 
		deferred
		ensure
			keep_reference: font = ft
		end

feature -- Action(s)

	apply is
			-- Apply the dialog box settings.
		do
			when_applied
		end

	finish is
			-- End processing.
		deferred
		end

feature {PUBLIC_NONE} -- Dialog events

	when_prepared is
			-- Dialog is about to be displayed.
		deferred
		end

	when_finished is
			-- Dialog has been closed.
		deferred
		end

	when_applied is
			-- Apply settings.
		deferred
		end

feature -- Buttons & keyboard interface
	
	set_ok_push_button (pushb: PUSH_BUTTON) is
			-- Set the button which is a default push button at 
			-- initialisation or when no other button is active.
			-- <enter> will execute that button, which has to do 
			-- the actual action, such as calling 'apply' & 
			-- `finish' itself, like, for instance, OK_BUTTON.
		require
			ok: pushb /= Void
		do
			ok_push_button := pushb
			pushb.set_default (true)
		ensure
			keep_reference: ok_push_button = pushb
		end

	set_cancel_push_button (pushb: PUSH_BUTTON) is
			-- Set the button which will be executed when <ESC> is 
			-- pressed. The button is responsible for closing the 
			-- dialog, without applying the settings. CANCEL_BUTTON 
			-- provides an implementation of such a button.
		require
			ok: pushb /= Void
		do
			cancel_push_button := pushb
		ensure
			keep_reference: cancel_push_button = pushb
		end

feature {DIALOG_ELEMENT} --? (document?, improve?, export?)

	ok_push_button, cancel_push_button: PUSH_BUTTON

feature {CONTROL}
	
	reset_default is
		local
			it: P_ITERATOR[CONTROL]
			pushb: PUSH_BUTTON
		do
			check dialog_init: control_list /= Void end

			from
				it := control_list.iterator
				it.first
			variant
				it.forward_variant
			until
				it.outside
			loop
				pushb ?= it.item
				if pushb /= Void and then pushb.is_default then
					pushb.set_default (false)
				end
				it.forth
			end
		end

feature {CONTROL} -- Keyboard interface

-- ALT+key:		hotkey
-- TAB:			next tabstop control
-- right-TAB	previous tabstop control
-- up/down:		previous/next control in group
-- ENTER/ESC:	OK, Cancel (used by special objects)

	keyboard_interface_hotkey (curctl: CONTROL; key: CHARACTER) is
			-- Alt+<c>entered.
		require
			valid: curctl /= Void
		local
			fmt: P_FORMAT
			hot: CHARACTER
			it: P_ITERATOR[CONTROL]
			cur: CONTROL
			found: BOOLEAN
		do
			check dialog_init: control_list /= Void end
			
			-- search hotkey
			!!fmt
			hot := fmt.to_lower (key)

			from
				it := control_list.iterator
				it.first
			variant
				it.forward_variant
			until
				it.outside
			loop
				-- find item with associated hotkey
				if not found then
					found := (it.item.hotkey = hot)
				end
				
				-- then find 1st tabstop/arrowstop control (useful for labels)
				if found and then (it.item.tabstop or it.item.arrowstop) then
					cur := it.item
					it.stop
				else
					it.forth
				end
			end

			if cur /= Void then
				cur.self.focus
			end
		end

	keyboard_interface_tab (curctl: CONTROL) is
			-- TAB entered: focus next control with tabstop.
		require
			valid: curctl /= Void
		local
			it: P_ITERATOR[CONTROL]
			found: BOOLEAN
		do
			check dialog_init: control_list /= Void end

			control_list.equality_search (curctl)
			check found: control_list.found end
			
			from
				it := control_list.iterator
				it.synchronize
			until it.outside loop
				it.forth
				if it.outside and not found then -- cycle
					it.first
				end
				if not it.outside and then (it.item.tabstop or it.item = curctl) then
					if it.item.tabstop then
						it.item.self.focus
						found := true
					end
					it.stop
				end
			end -- loop
		end

	keyboard_interface_shift_tab (curctl: CONTROL) is
			-- shift-TAB entered: focus previous control with tabstop.
		require
			valid: curctl /= Void
		local
			it: P_ITERATOR[CONTROL]
			found: BOOLEAN
		do
			check init_dialog: control_list /= Void end

			control_list.equality_search (curctl)
			check found: control_list.found end
			
			from
				it := control_list.iterator
				it.synchronize
			until it.outside loop
				it.back
				if it.outside and not found then -- cycle
					it.last
				end
				if not it.outside and then (it.item.tabstop or it.item = curctl) then
					if it.item.tabstop then
						it.item.self.focus
						found := true
					end
					it.stop
				end
			end -- loop
		end

	keyboard_interface_down (curctl: CONTROL) is
			-- Down arrow has been hit: focus previous control.
		require
			valid: curctl /= Void
		local
			it: P_ITERATOR[CONTROL]
		do
			check init_dialog: control_list /= Void end

			control_list.equality_search (curctl)
			check found: control_list.found end
			
			from
				it := control_list.iterator
				it.synchronize
			until
				it.outside
			loop
				it.forth
				if not it.outside then
					if it.item.tabstop or it.item.arrowstop then
						if it.item.arrowstop and not it.item.tabstop then
							it.item.self.focus
						end
						it.stop
					end
				end
			end
		end

	keyboard_interface_up (curctl: CONTROL) is
			-- Up arrow has been hit: focus next control
		require
			valid: curctl /= Void
		local
			it: P_ITERATOR[CONTROL]
		do
			check init_dialog: control_list /= Void end

			control_list.equality_search (curctl)
			check found: control_list.found end
			
			from
				it := control_list.iterator
				it.synchronize
			until
				it.outside
			loop
				it.back
				if not it.outside then
					if it.item.tabstop or it.item.arrowstop then
						if it.item.arrowstop and not it.item.tabstop then
							it.item.self.focus
						end
						it.stop
					end
				end
			end
		end

	keyboard_interface_enter is
			-- Enter has been hit.
		local
			it: P_ITERATOR[CONTROL]
			pushb: PUSH_BUTTON
		do
			check init_dialog: control_list /= Void end

			from
				it := control_list.iterator
				it.first
			variant
				it.forward_variant
			until
				it.outside
			loop
				pushb ?= it.item
				if pushb /= Void and then pushb.is_default then
					pushb.execute
					it.stop
				else
					it.forth
				end
			end
		end

	keyboard_interface_escape is
			-- Escape key has been hit.
		do
			if cancel_push_button /= Void then
				cancel_push_button.execute
			end
		end
	
feature {NONE}
	
	set_dialog_units (x, y: INTEGER) is
			-- Internal function to set dialog units.
		do
			!!dialog_units.make (x, y)
		end

feature {DIALOG_ELEMENT} -- Base units
	
	dialog_units: DIALOG_UNITS;
			-- Dialog units system
		
	default_interspace: INTEGER is
			-- Return default space between controls (used with dynamic positioning)
		do
			Result := Def_interspace
		end

feature {NONE}
	
	Def_interspace: INTEGER is 4

feature {NONE}
	
	control_list: P_LINKED_LIST[CONTROL]

feature {NONE} -- Enclose all elements

	control_bounds: RECTANGLE is
			-- Returns the bounding rectangle of all elements.
			-- (Independent coordinates)
		local
			it: P_ITERATOR[CONTROL]
			crect: RECTANGLE
		do
			check init_dialog: control_list /= Void end

			!!Result.make

			from
				it := control_list.iterator
				it.first
			variant
				it.forward_variant
			until
				it.outside
			loop
				-- get current control rectangle
				crect := it.item.indep_size

				-- update bounding rectangle
				if crect.lower.x < Result.lower.x then
					Result.set_lower_x (crect.lower.x)
				end
				if crect.lower.y < Result.lower.y then
					Result.set_lower_y (crect.lower.y)
				end
				if crect.upper.x > Result.upper.x then
					Result.set_upper_x (crect.upper.x)
				end
				if crect.upper.y > Result.upper.y then
					Result.set_upper_y (crect.upper.y)
				end

				-- next
				it.forth
			end
		ensure
			valid_result: Result /= Void
		end

feature {DIALOG_ELEMENT}
	
	arranged_size: RECTANGLE is
			-- Returns the bounding rectangle suitable for setting client size.
			-- (Independent coordinates)
		do
			Result := control_bounds
			Result.set_lower_x (0)
			Result.set_lower_y (0)

			Result.set_upper_x (Result.upper.x + default_interspace)
			Result.set_upper_y (Result.upper.y + default_interspace)
		ensure
			good_rect: Result /= Void
		end

feature {CONTROL} -- Control family planning

	add_control (ctl: CONTROL) is 
			-- Add a control to this dialog
		require
			valid_ctl: ctl /= Void
		do
			check init_dialog: control_list /= Void end
			control_list.add (ctl)
		end

	first_control: CONTROL is
			-- Get first control
		local
			it: P_ITERATOR[CONTROL]
		do
			check init_dialog: control_list /= Void end

			it := control_list.iterator
			it.first
			if not it.outside then
				Result := it.item
			end
		end

end -- class DIALOG
