-- Fenestra; version 0.80(beta); 1998-09-27

-- 
-- Copyright (c) 1998 Franck Arnaud, http://www.altsoft.demon.co.uk/
-- 
-- This package is distributed under the Eiffel Forum Freeware License, 
-- version 1 (see file forum.txt for full text). The main points are:
-- 
-- "Permission is hereby granted, without written agreement and without 
--  license or royalty fees, to use, copy, modify and/or distribute this 
--  package, provided that: copyright notices are retained unchanged [...]
-- 
--  Permission is hereby also granted [...] to distribute binary programs 
--  which depend on this package, provided that: if the binary program 
--  depends on a modified version of this package, you must publicly release 
--  the modified version of this package [...].
-- 
--  THIS PACKAGE IS PROVIDED "AS IS" AND WITHOUT WARRANTY [...]"
-- 

indexing
	cluster: Fenestra, Windows
	description: "Modeless dialog box"
	comment: "creation procedure for descendants is `make' or `make_thin'"
	interface: inheritance
		
deferred class MODELESS_DIALOG

inherit
	POPUP_WINDOW
		redefine
			register, when_created,
			when_closed, when_focus_given
		end
	DIALOG
		end

feature -- Creation
	
	make (p: like parent) is
			-- Make dialog box with thick border.
		require
			valid: p /= Void
		do
			parent := p
			!!wo.make_handle (wrt.win_create_dialog (parent.handle, true))
			register
		end

	make_thin (p: like parent) is
			-- Make dialog box with thin border.
		require
			valid: p /= Void
		do
			parent := p
			!!wo.make_handle (wrt.win_create_dialog (parent.handle, false))
			register
		end

feature {NONE}
	
	register, frozen modeless_register is
		local
			config: F_CONFIG
		do
			!!config
			make_dialog
			
			set_font (config.default_dialog_font)
			
			window_register
		end
	
feature -- Action(s)

	finish is
		do
			close
		end

feature -- Other actions

	arrange is
			-- Resize nicely so that the box is at the centre of the parent window 
			-- and encloses all controls.
		do
			set_client_size (dialog_units.indep_to_abs (arranged_size))
			centre
		end

feature {NONE}

	centre is
			-- Place dialog at the centre of the parent window.
		local
			r: RECTANGLE
		do
			if parent /= Void then
				r := size
				r.translate_center (parent.size.center)
				set_size (r)
			end
		end

feature {PUBLIC_NONE}

	when_created is
		do
			when_prepared
		end

	when_closed is
		do
			when_finished
		end

	when_focus_given is
		local
			ctl: CONTROL
		do
			ctl := first_control
			if ctl /= Void then
				if ctl.tabstop then
					ctl.self.focus
				else
					keyboard_interface_tab (ctl)
				end
			end
		end

feature -- Font

	set_font (ft: FONT) is
		do
			font := ft
			device.prepare
			device.set_font (ft)
			set_dialog_units (wrt.win_gdi_textmetr (device.handle, wrt.Ftm_avecharw), 
				wrt.win_gdi_textmetr (device.handle, wrt.Ftm_height))
			device.reset
			device.release
		end

end -- class MODELESS_DIALOG
