-- Fenestra; version 0.80(beta); 1998-09-27

-- 
-- Copyright (c) 1998 Franck Arnaud, http://www.altsoft.demon.co.uk/
-- 
-- This package is distributed under the Eiffel Forum Freeware License, 
-- version 1 (see file forum.txt for full text). The main points are:
-- 
-- "Permission is hereby granted, without written agreement and without 
--  license or royalty fees, to use, copy, modify and/or distribute this 
--  package, provided that: copyright notices are retained unchanged [...]
-- 
--  Permission is hereby also granted [...] to distribute binary programs 
--  which depend on this package, provided that: if the binary program 
--  depends on a modified version of this package, you must publicly release 
--  the modified version of this package [...].
-- 
--  THIS PACKAGE IS PROVIDED "AS IS" AND WITHOUT WARRANTY [...]"
-- 

indexing
	cluster: Fenestra, Controls
	description: "Built-in listbox"
	interface: inheritance, client

class STD_LISTBOX[G -> LISTBOX_ITEM]

inherit
	STD_CONTROL
		redefine
			std_control_when_command, register
		end
	LISTBOX[G]
		end

creation
	make, make_unsorted

feature {CREATION_NONE} -- Creation

	make (p: WINDOW) is
			-- Create sorted list box.
		do
			!!self.make_handle (p, wrt.win_listbox_create (p.handle, false, true))
			register
		end

	make_unsorted (p: WINDOW) is
			-- Create unsorted list box.
		require
			valid: p /= Void
		do
			!!self.make_handle (p, wrt.win_listbox_create (p.handle, false, false))
			register
		end

feature {NONE} -- Init

	register is
			-- Register.
		do
			std_control_register -- Precursor
			!!items.make
		end

feature {NONE} -- list

	items: P_DICTIONARY[G,STRING]

feature {NONE} -- Internal item selection

	reset_selection is
			-- Reset selection.
		do
			wrt.win_msg_setint (self.handle, wrt.Lb_setcursel, 0)
		end

feature -- Items setup

	add_item (item: G) is
			-- Add an item to the list.
		do
			check list_init: items /= Void end

			wrt.win_listbox_addtext (self.handle, wrt.ew_string (item.name).to_c)
			if items.count = 0 then
				reset_selection
			end
			items.add (item, item.name)
		end

	remove_all is
			-- Remove all items.
		do
			check list_init: items /= Void end

			wrt.win_msg_void (self.handle, wrt.Lb_resetcontent)
			!!items.make
		ensure then
			empty: items.is_empty
		end

feature {NONE} -- get current item (redefined by STD_MULTIPLE_LISTBOX)

	get_curitem_idx: INTEGER is
		do
			Result := wrt.win_msg_int (self.handle, wrt.Lb_getcursel)
		end

	get_curitem (idx: INTEGER): G is
		local
			str: STRING
		do
			check list_init: items /= Void end

			if idx >= 0 then -- not LB_ERR
				wrt.win_listbox_gettext (self.handle, idx)
				str := wrt.win_laststring
				
				items.key_search (str)
				if items.found then
					Result := items.item
				end
			end -- else no item selected
		end
		
feature -- selection

	has_selection: BOOLEAN is
			-- Has selection?
		do
			Result := (get_curitem_idx >= 0)
		end

	selected_item: G is
			-- Current selected item.
		do
			Result := get_curitem (get_curitem_idx)
		end

	has_item (itm: G): BOOLEAN is
			-- Is `itm' in the list?
		local
			it: P_ITERATOR[G]
		do
			check list_init: items /= Void end

			from
				it := items.to_item_list.iterator
				it.first
			variant
				it.forward_variant
			until
				it.outside
			loop
				if it.item = itm then
					Result := true
					it.stop
				else
					it.forth
				end
			end
		end

	select_item (new: G) is
			-- Mark `new' as selected.
		do
			wrt.win_listbox_selstring (self.handle, wrt.ew_string (new.name).to_c) --? duplicates 
			check done: (selected_item.name).is_equal (new.name) end
		end

feature {NONE}
	
	std_control_when_command (not_code: INTEGER) is
		require else
			true
		do
			if not_code = wrt.Lbn_selchange then
				when_selected
			elseif not_code = wrt.Lbn_selcancel then
				-- selection canceled
			elseif not_code = wrt.Lbn_dblclk then
				check has_selection = true end
				when_executed
			elseif not_code = wrt.Lbn_errspace then
				wrt.raise_exception (wrt.Excep_listbox)
			end
		end

end -- class STD_LISTBOX
