-- Fenestra; version 0.80(beta); 1998-09-27

-- 
-- Copyright (c) 1998 Franck Arnaud, http://www.altsoft.demon.co.uk/
-- 
-- This package is distributed under the Eiffel Forum Freeware License, 
-- version 1 (see file forum.txt for full text). The main points are:
-- 
-- "Permission is hereby granted, without written agreement and without 
--  license or royalty fees, to use, copy, modify and/or distribute this 
--  package, provided that: copyright notices are retained unchanged [...]
-- 
--  Permission is hereby also granted [...] to distribute binary programs 
--  which depend on this package, provided that: if the binary program 
--  depends on a modified version of this package, you must publicly release 
--  the modified version of this package [...].
-- 
--  THIS PACKAGE IS PROVIDED "AS IS" AND WITHOUT WARRANTY [...]"
-- 

indexing
	cluster: Fenestra, Graphics
	description: "Bitmap device"
	interface: client

class BITMAP_DEVICE

inherit
	GRAPHIC_DEVICE

	RESOURCE_OBJECT
		rename
			make_resource as load_resource
		end
	DRAWABLE
		rename
			draw as drawable_draw
		end

creation
	make

creation {CLIPBOARD}
	make_handle

feature {CREATION_NONE} -- Creation

	make (model: GRAPHIC_DEVICE) is
			-- Create bitmap/memory device.
		require
			valid: model /= Void and model.is_ready
			supported: model.capabilities.has_raster_bitmap
		do
			initialise
			set_handle (wrt.win_gdi_bmpdc (model.handle))
			check ok: is_ready end
			!!target.make
			set_size (1, 1)
			reset_mode
			initialise_attributes
		end

feature {CLIPBOARD} -- clipboard creation

	make_handle (bmp_handle: INTEGER) is
		do
			initialise --? FIXME: device handle!!!
			!!bitmap.make_handle (bmp_handle)
			select_object (bitmap)
			reset_mode
			initialise_attributes
		end

feature {CLIPBOARD}
	
	bitmap: WIN_BITMAP

feature {NONE}
	
	target: POINT
	twidth, theight: INTEGER
	stretch: BOOLEAN

feature -- Status

	width: INTEGER is
			-- Width of the bitmap, in pixels.
		require
			ready: is_ready
		do
			Result := wrt.win_gdi_bmpw (handle, bitmap.handle)
		ensure
			ok: Result >= 0
		end

	height: INTEGER is
			-- Height of the bitmap, in pixels.
		require
			ready: is_ready
		do
			Result := wrt.win_gdi_bmph (handle, bitmap.handle)
		ensure
			ok: Result >= 0
		end

feature -- Status
	
	is_ready: BOOLEAN is
			-- Is the device ready?
		do
			Result := is_valid_handle
		end

	prepare is
			-- Device is ready at creation.
		do
		ensure then
			ready_at_creation: false
		end

	release is
			-- Terminate processing with this device.
		do
			dispose
		end

feature {NONE}
	
	dispose is
			-- Finalisation.
		do
			if handle /= Invalid_handle then
				win_gdi_delete (handle)
				handle := Invalid_handle
			end
		end
	
feature -- Operations
	
	set_size (w, h: INTEGER) is
			-- Set the size of this bitmap device (the previous image is lost) 
			-- to a width of `w' and height `h', in pixels.
		require
			ready: is_ready
		do
			if bitmap /= Void then
				bitmap.force_dispose
			end
			!!bitmap.make_handle (wrt.win_gdi_bmpsize (handle, w, h))
			select_object (bitmap)
		ensure
			ok: w = width and h = height
		end

	copy_from_device (other: GRAPHIC_DEVICE) is
			-- Copy bitmap from `other' device at the position defined 
			-- by `set_position'.
		require
			ready: is_ready
			valid_device: other /= Void and then other.is_ready
			supported: other.capabilities.has_raster_bitmap
		do
			wrt.win_gdi_bitblt (handle, 0, 0, width, height, other.handle, target.x, target.y);
		end

	load_resource (name: STRING) is
			-- Load bitmap from resource.
		do
			!!bitmap.make_handle (wrt.win_gdi_loadbmp (wrt.ew_string (name).to_c))
			select_object (bitmap)
		end

	load (fname: STRING) is
			-- Load bitmap from file.
		require
			valid: fname /= Void
			ready: is_ready
		local
			hdle: INTEGER
		do
			hdle := wrt.win_gdi_bmpload (handle, wrt.ew_string (fname).to_c)
			if hdle /= 0 then
				if bitmap /= Void then
					bitmap.force_dispose
				end
				!!bitmap.make_handle (hdle)
				select_object (bitmap)
			end
		end

	save (fname: STRING) is
			-- Save bitmap to file.
		require
			valid: fname /= Void
			ready: is_ready
		local
			ret: BOOLEAN
		do
			ret := wrt.win_gdi_bmpsave (handle, bitmap.handle, wrt.ew_string (fname).to_c)
		end
			
feature -- Status

	set_position (tpos: POINT) is
			-- Set target position for drawing.
		require
			valid: tpos /= Void
		do
			target.copy (tpos)
		end

	set_mode_stretch is
			-- Set drawing mode to stretch (adapt size when drawing).
		do
			stretch := true
		end

	reset_mode, set_mode_copy is
			-- Set drawing mode to copy (size in destination device 
			-- is the same).
		do
			stretch := false
		end

	set_target (rect: RECTANGLE) is
			-- Set target rectangle for stretching.
		require
			valid: rect /= Void
		do
			target.copy (rect.lower)
			twidth := rect.width
			theight := rect.height
		end

feature {GRAPHIC_DEVICE} -- from DRAWABLE

	drawable_draw (gd: GRAPHIC_DEVICE) is
		do
			if stretch then
				wrt.win_gdi_strblt (gd.handle, target.x, target.y, twidth, theight,
								handle, 0, 0, width, height)
								
			else
				wrt.win_gdi_bitblt (gd.handle, target.x, target.y, width, height,-- target 
								handle, 0, 0) -- source
			end
		end

invariant
	bmp: bitmap /= Void

end -- class BITMAP_DEVICE
