-- Fenestra; version 0.80(beta); 1998-09-27

-- 
-- Copyright (c) 1998 Franck Arnaud, http://www.altsoft.demon.co.uk/
-- 
-- This package is distributed under the Eiffel Forum Freeware License, 
-- version 1 (see file forum.txt for full text). The main points are:
-- 
-- "Permission is hereby granted, without written agreement and without 
--  license or royalty fees, to use, copy, modify and/or distribute this 
--  package, provided that: copyright notices are retained unchanged [...]
-- 
--  Permission is hereby also granted [...] to distribute binary programs 
--  which depend on this package, provided that: if the binary program 
--  depends on a modified version of this package, you must publicly release 
--  the modified version of this package [...].
-- 
--  THIS PACKAGE IS PROVIDED "AS IS" AND WITHOUT WARRANTY [...]"
-- 

indexing
	cluster: Fenestra, Graphics
	description: "Custom mapping"
	interface: client

class CUSTOM_MAPPING

inherit
	GRAPHIC_MAPPING
		redefine
			copy, is_equal
		end

creation
	make

feature {CREATION_NONE}
	
	make is
		do
			!!origin.make
			isotropic
			x_factor := 1.0
			y_factor := 1.0
		end

feature {CUSTOM_MAPPING}
	
	origin: POINT
	mode: INTEGER
	x_factor, y_factor: DOUBLE

	Base: INTEGER is 100

feature {WIN_API} 
	
	duplicate: GRAPHIC_MAPPING is
			-- Duplicate current.
		do
			!CUSTOM_MAPPING!Result.make
			Result.copy (Current)
		end

feature -- from GENERAL

	copy (other: like Current) is
		do
			origin.copy (other.origin)
			mode := other.mode
			x_factor := other.x_factor
			y_factor := other.y_factor
		end

	is_equal (other: like Current): BOOLEAN is
		do
			Result := (origin.is_equal (other.origin)) and mode = other.mode and x_factor = other.x_factor and y_factor = other.y_factor
		end
		
feature -- Origin

	set_origin (pt: POINT) is
			-- Set where the origin (0, 0) is relatively to the default origin 
			-- of the device, in logical coordinates.
		require
			valid_point: pt /= Void
		do
			origin.copy (pt)
		end

feature -- Mode
	
	is_isotropic: BOOLEAN is
			-- Isotropic mode?
		do
			Result := (mode = wrt.Mm_isotropic)
		end

	isotropic is
			-- Isotropic mode (both axis scale proportionally, that is 
			-- squares remain square).
			-- This is the default.
		do
			mode := wrt.Mm_isotropic
		end

	anisotropic is
			-- Axis may be scaled independently.
		do
			mode := wrt.Mm_anisotropic
		end

feature -- Scale

	set_scale, set_x_scale (nx: DOUBLE) is
			-- Change x axis scale factor (1.0 = unchanged).
			-- If isotropic, changes both axis.
			-- Default: 1.0 (no change)
		require
			not_zero: nx /= 0.0
		do
			x_factor := nx
			if is_isotropic then
				y_factor := x_factor
			end
		ensure
			done: x_factor = nx
			iso: is_isotropic implies y_factor = x_factor
		end

	set_y_scale (ny: DOUBLE) is
			-- Change y_axis scale factor.
			-- Default: 1.0 (no change)
		require
			not_iso: not is_isotropic
			not_zero: ny /= 0.0
		do
			y_factor := ny
		ensure
			done: y_factor = ny
		end

feature {GRAPHIC_DEVICE}
	
	set_mode (device: GRAPHIC_DEVICE) is
		local
			sys: P_SYSTEM
		do
			!!sys

			-- use default Gm_compatible mode
			wrt.win_gdi_mapmode (device.handle, mode)
			wrt.win_gdi_origin (device.handle, origin.x, origin.y)
			wrt.win_gdi_vportext (device.handle, sys.floor (x_factor*Base), sys.floor (y_factor*Base))
			wrt.win_gdi_winext (device.handle, sys.floor (Base), sys.floor (Base))
		end

end -- class CUSTOM_MAPPING
