-- Fenestra; version 0.80(beta); 1998-09-27

-- 
-- Copyright (c) 1998 Franck Arnaud, http://www.altsoft.demon.co.uk/
-- 
-- This package is distributed under the Eiffel Forum Freeware License, 
-- version 1 (see file forum.txt for full text). The main points are:
-- 
-- "Permission is hereby granted, without written agreement and without 
--  license or royalty fees, to use, copy, modify and/or distribute this 
--  package, provided that: copyright notices are retained unchanged [...]
-- 
--  Permission is hereby also granted [...] to distribute binary programs 
--  which depend on this package, provided that: if the binary program 
--  depends on a modified version of this package, you must publicly release 
--  the modified version of this package [...].
-- 
--  THIS PACKAGE IS PROVIDED "AS IS" AND WITHOUT WARRANTY [...]"
-- 

indexing
	cluster: Fenestra, Graphics
	description: "Graphic arc (of an ellipse)"
	interface: client

class DRAWABLE_ARC

inherit
	DRAWABLE_ELLIPSE
		redefine
			make, draw
		select
			make
		end
	DRAWABLE_ELLIPSE
		rename
			make as old_make
		export
			{NONE} all
		undefine
			draw
		end

creation
	make

feature {CREATION_NONE} -- Creation
	
	make is
		do
			old_make
			!!start.make
			!!final.make
		end


feature {NONE} -- Internal position
	
	start: POINT
			-- Point defining starting radial (line crossing 
			-- this point and the centre of the ellipse).

	final: POINT
			-- Point defining ending radial.

	clockwise: BOOLEAN
			-- Direction used when the arc is drawn.

feature -- Position

	set_starting_radial (pt: POINT) is
			-- Set the point defining the starting radial from which 
			-- the arc will be drawn. The radial is the line containing 
			-- this point and the centre of the ellipse.
		require
			valid_point: pt /= Void
		do
			start.copy (pt)
		end

	set_ending_radial (pt: POINT) is
			-- Set the point defining the ending radial to which 
			-- the arc will be drawn.
		require
			valid_point: pt /= Void
		do
			final.copy (pt)
		end

	set_direction_anticlockwise is
			-- Draw ellipse anticlockwise (default).
		do
			clockwise := false
		ensure
			done: clockwise = false
		end

	set_direction_clockwise is
			-- Draw ellipse clockwise.
		do
			clockwise := true
		ensure
			done: clockwise = true
		end

feature {GRAPHIC_DEVICE} -- Drawing
	
	draw (dev: GRAPHIC_DEVICE) is
			-- Actual draw
		do
			wrt.win_gdi_arcdir (dev.handle, clockwise)
			wrt.win_gdi_arc (dev.handle, bounding_rectangle.lower.x, bounding_rectangle.lower.y,
							bounding_rectangle.upper.x, bounding_rectangle.upper.y,
							start.x, start.y, final.x, final.y)
		end

invariant
	valid_arc: start /= Void and final /= Void

end -- class DRAWABLE_ARC
