-- Fenestra; version 0.80(beta); 1998-09-27

-- 
-- Copyright (c) 1998 Franck Arnaud, http://www.altsoft.demon.co.uk/
-- 
-- This package is distributed under the Eiffel Forum Freeware License, 
-- version 1 (see file forum.txt for full text). The main points are:
-- 
-- "Permission is hereby granted, without written agreement and without 
--  license or royalty fees, to use, copy, modify and/or distribute this 
--  package, provided that: copyright notices are retained unchanged [...]
-- 
--  Permission is hereby also granted [...] to distribute binary programs 
--  which depend on this package, provided that: if the binary program 
--  depends on a modified version of this package, you must publicly release 
--  the modified version of this package [...].
-- 
--  THIS PACKAGE IS PROVIDED "AS IS" AND WITHOUT WARRANTY [...]"
-- 

indexing
	cluster: Fenestra, Graphics
	description: "Geometric pen"
	interface: client

class GEOMETRIC_PEN

inherit
	BASIC_PEN
		redefine
			set_color
		end

creation
	make
	
feature {CREATION_NONE} -- Creation

	make is
		local
			white: RGB_COLOR
		do
			endstyle := 0 -- OS default
			joinstyle := 0
			set_width (1)
			!!white.white
			set_color (white) -- implies default brush
			make_basic
		end
		

feature {NONE}
	
	joinstyle: INTEGER
	endstyle: INTEGER
	brush: BRUSH
	width: INTEGER

feature -- Color

	set_color (cl: COLOR) is
			-- Set brush to solid brush with colour `cl'. 
		local
			cbrush: COLOR_BRUSH
		do
			!COLOR_BRUSH!cbrush.make
			cbrush.set_color (cl)
			set_brush (cbrush)
			color := cl -- invariant
		end

feature -- Brush

	set_brush (br: BRUSH) is
			-- Set brush associated with pen, overrides set_color.
			-- Default: solid brush.
		require
			valid_brush: br /= Void
		do
			brush := br
		ensure
			keep_reference: br = brush
		end

feature -- Width

	set_width (wi: INTEGER) is
			-- Set pen width in logical device units.
		require
			valid_width: wi >= 0
		do
			width := wi
		end

feature -- End cap style

	set_end_round is
			-- Set endcap style: round.
		do
			endstyle := wrt.Ps_endcap_round
		end

	set_end_square is
			-- Set endcap style: square.
		do
			endstyle := wrt.Ps_endcap_square
		end

	set_end_flat is
			-- Set endcap stylr: flat.
		do
			endstyle := wrt.Ps_endcap_flat
		end

feature -- Join style

	set_join_round is
			-- Line joins are round.
		do
			joinstyle := wrt.Ps_join_round
		end

	set_join_bevel is
			-- Line joins are beveled.
		do
			joinstyle := wrt.Ps_join_bevel
		end

	set_join_miter is
			-- Line joins are mitered.
		do
			joinstyle := wrt.Ps_join_miter
		end

feature {WIN_API}
	
	to_windows: WIN_PEN is
		do
			!!Result.make_geometric (width, style, endstyle, joinstyle, brush)
		end

invariant
	valid_brush: brush /= Void

end -- class GEOMETRIC_PEN
