-- Fenestra; version 0.80(beta); 1998-09-27

-- 
-- Copyright (c) 1998 Franck Arnaud, http://www.altsoft.demon.co.uk/
-- 
-- This package is distributed under the Eiffel Forum Freeware License, 
-- version 1 (see file forum.txt for full text). The main points are:
-- 
-- "Permission is hereby granted, without written agreement and without 
--  license or royalty fees, to use, copy, modify and/or distribute this 
--  package, provided that: copyright notices are retained unchanged [...]
-- 
--  Permission is hereby also granted [...] to distribute binary programs 
--  which depend on this package, provided that: if the binary program 
--  depends on a modified version of this package, you must publicly release 
--  the modified version of this package [...].
-- 
--  THIS PACKAGE IS PROVIDED "AS IS" AND WITHOUT WARRANTY [...]"
-- 

indexing
	cluster: Fenestra, Graphics
	description: "Extra graphics routines"
	interface: mixin, client

class GRAPHICS_ROUTINES

feature -- Stock brush

	void_pen: PEN is
			-- Invisible pen.
		do
			!VOID_PEN!Result
		end

	void_brush: BRUSH is
			-- Invisible brush.
		do
			!VOID_BRUSH!Result
		end

feature -- Simple settings
	
	color_pen (col: COLOR): PEN is
			-- Create a basic colour pen.
		require
			col_ok: col /= Void
		local
			spen: SIMPLE_PEN
		do
			!!spen.make
			spen.set_color (col)
			Result := spen
		ensure
			is_copy: Result /= Void
		end

	color_brush (col: COLOR): BRUSH is
			-- Create a solid colour brush.
		require
			col_ok: col /= Void
		local
			cbrush: COLOR_BRUSH
		do
			!!cbrush.make
			cbrush.set_color (col)
			Result := cbrush
		ensure
			is_copy: Result /= Void
		end

feature -- Device setup

	set_pen (device: GRAPHIC_DEVICE; col: COLOR) is
			-- Set the pen of the device to a color_pen (col). 
		require
			dev_ok: device /= Void
			col_ok: col /= Void
		do
			device.set_pen (color_pen (col))
		end

	set_brush (device: GRAPHIC_DEVICE; col: COLOR) is
			-- Set the brush of the device to a color_brush (col).
		require
			dev_ok: device /= Void
			col_ok: col /= Void
		do
			device.set_brush (color_brush (col))
		end

	set_pen_and_brush (device: GRAPHIC_DEVICE; col: COLOR) is
			-- Set the pen and brush of the device.
		require
			dev_ok: device /= Void
			col_ok: col /= Void
		do
			set_pen (device, col)
			set_brush (device, col)
		end

feature -- Vector conversion

	device_to_logical_size (device: GRAPHIC_DEVICE; pt: POINT): POINT is
			-- Convert width/height (instead of actual point).
		require
			dev_ok: device /= Void
			dev_ready: device.is_ready
			point_ok: pt /= Void
		local
			pt1, pt2: POINT
		do
			!!pt1.make
			pt1 := device.device_to_logical (pt1)
			pt2 := device.device_to_logical (pt)
			!!Result.make
			Result.set_x (pt2.x - pt1.x)
			Result.set_y (pt2.y - pt1.y)
		end

	logical_to_device_size (device: GRAPHIC_DEVICE; pt: POINT): POINT is
			-- Convert width/height (instead of actual point).
		require
			dev_ok: device /= Void
			dev_ready: device.is_ready
			point_ok: pt /= Void
		local
			pt1, pt2: POINT
		do
			!!pt1.make
			pt1 := device.logical_to_device (pt1)
			pt2 := device.logical_to_device (pt)
			!!Result.make
			Result.set_x (pt2.x - pt1.x)
			Result.set_y (pt2.y - pt1.y)
		end

end -- class GRAPHICS_ROUTINES
