-- Fenestra; version 0.80(beta); 1998-09-27

-- 
-- Copyright (c) 1998 Franck Arnaud, http://www.altsoft.demon.co.uk/
-- 
-- This package is distributed under the Eiffel Forum Freeware License, 
-- version 1 (see file forum.txt for full text). The main points are:
-- 
-- "Permission is hereby granted, without written agreement and without 
--  license or royalty fees, to use, copy, modify and/or distribute this 
--  package, provided that: copyright notices are retained unchanged [...]
-- 
--  Permission is hereby also granted [...] to distribute binary programs 
--  which depend on this package, provided that: if the binary program 
--  depends on a modified version of this package, you must publicly release 
--  the modified version of this package [...].
-- 
--  THIS PACKAGE IS PROVIDED "AS IS" AND WITHOUT WARRANTY [...]"
-- 

indexing
	cluster: Fenestra, Graphics
	description: "Description of a font (typeface)"
	interface: client

class LOGICAL_FONT

inherit
	FONT
		redefine
			copy, is_equal
		end
	WIN_LOGFONT
		undefine copy, is_equal end
	P_TEXT_OBJECT
		undefine copy, is_equal end
	COMMON_NOTIFIER
		undefine copy, is_equal end

creation
	make, from_string

creation {WIN_API}
	make_device,
	make_logfont

feature {CREATION_NONE} -- Creation

	make is
		do
			init_logfont
			reset_style
			is_pt := false
		end

feature {NONE}
	
	make_device (gd: GRAPHIC_DEVICE) is
		do
			make

			wrt.win_gdi_textface (gd.handle)
			lf_set_iface (wrt.win_laststring)
			
			lf_set_iheight (wrt.win_gdi_textmetr (gd.handle, wrt.Ftm_height) - wrt.win_gdi_textmetr (gd.handle, wrt.Ftm_intleading))
			lf_set_iwidth (wrt.win_gdi_textmetr (gd.handle, wrt.Ftm_avecharw))
			lf_set_iweight (wrt.win_gdi_textmetr (gd.handle, wrt.Ftm_weight))
			lf_set_charset (wrt.win_gdi_textmetr (gd.handle, wrt.Ftm_charset))
			
			lf_set_escapement (0) -- These values are not requestable. 
			lf_set_orientation (0) 
			
			lf_set_italic (wrt.int_to_bool (wrt.win_gdi_textmetr (gd.handle, wrt.Ftm_italic)))
			lf_set_underline (wrt.int_to_bool (wrt.win_gdi_textmetr (gd.handle, wrt.Ftm_underline)))
			lf_set_strike_out (wrt.int_to_bool (wrt.win_gdi_textmetr (gd.handle, wrt.Ftm_strikeout)))

			-- Note: the pitch value return by GetTextMetrics is not the same as 
			-- the one in LOGFONT. In LOGFONT the lower 4 bits of pitch & family only 
			-- indicates the fixed size (1) or variable size (2), while the GetTextMetrics 
			-- returns a bit field to indicate if the font is fixed pitch (bit 0 (LSB)), 
			-- vector (bit 1), true type (bit 2), or a device font (bit 3). The 4 higher bits 
			-- indicate the family in both cases.
			
			if wrt.win_gdi_textmetr (gd.handle, wrt.Ftm_fixedpitch) = 0 then
				lf_set_pitch_family (wrt.Fixed_pitch, wrt.win_gdi_textmetr (gd.handle, wrt.Ftm_family))
			else
				lf_set_pitch_family (wrt.Variable_pitch, wrt.win_gdi_textmetr (gd.handle, wrt.Ftm_family))
			end -- reversed microsoft logic.
		end

	make_logfont (longfont_ptr: POINTER) is
		do
			make
			
			struct.from_windows (longfont_ptr)
		end

feature -- Copy/Equality
	
	copy (other: like Current) is
		do
			struct.copy (other.struct)
			is_pt := other.is_point
		end

	is_equal (other: like Current): BOOLEAN is
		do
			Result := struct.is_equal (other.struct) and is_point = other.is_point
		end

feature {WIN_API}
	
	duplicate: FONT is
		do
			!LOGICAL_FONT!Result.make
			Result.copy (Current)
		end

feature {NONE} -- Effect P_TEXT_OBJECT

	Font_format_portable: INTEGER is 1
	Font_format_device: INTEGER is 2
	
	object_to_string (tm: P_TEXT_OBJECT_MANAGER) is
		do
			tm.put_integer (Font_format_device)
			tm.put_boolean (is_pt)
			tm.put_string (lf_iface)
			tm.put_integer (lf_iheight)
			tm.put_integer (lf_iwidth)
			tm.put_integer (lf_escapement)
			tm.put_integer (lf_orientation)
			tm.put_integer (lf_iweight)
			tm.put_boolean (lf_italic)
			tm.put_boolean (lf_underline)
			tm.put_boolean (lf_strike_out)
			tm.put_integer (lf_charset)
			tm.put_integer (lf_pitch)
			tm.put_integer (lf_family)
		end

	string_to_object (tm: P_TEXT_OBJECT_MANAGER) is
		local
			version: INTEGER
		do
			make

			tm.get_integer
			version := tm.last_integer

			if version = Font_format_device then
				tm.get_boolean  is_pt := tm.last_boolean
				tm.get_string   lf_set_iface (tm.last_string)
				tm.get_integer  lf_set_iheight (tm.last_integer)
				tm.get_integer  lf_set_iwidth (tm.last_integer)
				tm.get_integer  lf_set_escapement (tm.last_integer)
				tm.get_integer  lf_set_orientation (tm.last_integer)
				tm.get_integer  lf_set_iweight (tm.last_integer)
				tm.get_boolean  lf_set_italic (tm.last_boolean)
				tm.get_boolean  lf_set_underline (tm.last_boolean)
				tm.get_boolean  lf_set_strike_out (tm.last_boolean)
				tm.get_integer  lf_set_charset (tm.last_integer)
				tm.get_integer  lf_set_pitch (tm.last_integer)
				tm.get_integer  lf_set_family (tm.last_integer)

				if not tm.in_success then
					make
				end
			end
		end

feature {WIN_API}

	to_windows (dc: GRAPHIC_DEVICE): WIN_FONT is
		local
			h: INTEGER
		do
			h := lf_iheight
			
			update_height (dc)
			!!Result.make_font (struct)
			
			lf_set_iheight (h)
		end

feature {NONE}
	
	update_height (dc: GRAPHIC_DEVICE) is
		local
			pt: POINT
			dpi: INTEGER
		do
			if is_point then
				if dc /= Void then --? use special function
					!!pt.make
					-- convert (0, height) vector
					pt.set_y (wrt.win_gdi_capint (dc.handle, wrt.Logpixelsy))
					pt := dc.device_to_logical (pt)
					dpi := pt.y
					-- error correction if origin translated
					pt.set_y (0)
					pt := dc.device_to_logical (pt)
					dpi := dpi - pt.y
				else
					dpi := notifier.screen_dpi
				end

				lf_set_iheight (- wrt.win_muldiv (lf_iheight, dpi, 72))
			end
		end

feature {FONT_DIALOG}
	
	prepare_dialog is
			-- Prepare for font dialog.
		do
			if is_point then
				update_height (Void)
				is_pt := false
			end
		end

feature {NONE} -- Attributes
	
	is_pt: BOOLEAN

feature -- Typeface name

	is_default_face: BOOLEAN is
			-- Is the current typeface the default?
		do
			Result := (lf_iface = Void)
		end

	face_name: STRING is
			-- Get the current face name.
		require
			exist: not is_default_face
		do
			Result := lf_iface
		ensure
			is_copy: Result /= Void
		end

	set_face_name (str: STRING) is
			-- Set required typeface name (may be an empty string).
		require
			valid_string: str /= Void
		do
			lf_set_iface (str)
		ensure
			done: not is_default_face
		end

	reset_face_name is
			-- Use default typeface.
		do
			lf_set_iface ("")
		ensure
			done: is_default_face
		end

feature -- Height (Points)

	is_point: BOOLEAN is
			-- Is the current height the point value?
		do
			Result := is_pt
		end

	is_cell_height: BOOLEAN is
			-- Is the current height the character cell height?
		do
			Result := not is_pt and then (lf_iheight > 0)
		end

	is_character_height: BOOLEAN is
			-- Is the current height the character height?
		do
			Result := not is_pt and then (lf_iheight < 0)
		end

	is_default_height: BOOLEAN is
			-- Is the current height the default height?
		do
			Result := not is_pt and then (lf_iheight = 0)
		end

	height_value: INTEGER is
			-- Height (semantics depending on `is_point', etc).
		require
			meaningful: not is_default_height
		do
			if lf_iheight < 0 then
				Result := - lf_iheight
			else
				Result := lf_iheight
			end
		ensure
			valid: Result >= 0
		end

	set_point (h: INTEGER) is
			-- Set the character height in points.
		require
			valid_height: h > 0
			no_width: is_default_width
		do
			lf_set_iheight (h)
			is_pt := true
		ensure
			done: is_point and (not is_default_height) and 
				(not is_character_height) and (not is_cell_height)
		end

	set_cell_height (h: INTEGER) is
			-- Set the required character cell height (in logical units).
		require
			valid_height: h > 0
		do
			lf_set_iheight (h)
			is_pt := false
		ensure
			done: is_cell_height and (not is_default_height) 
				and (not is_character_height) and (not is_point)
		end

	set_character_height (h: INTEGER) is
			-- Set the required character height (in logical units).
		require
			valid_height: h > 0
		do
			lf_set_iheight (- h)
			is_pt := false
		ensure
			done: is_character_height and (not is_cell_height) 
				and (not is_default_height) and (not is_point)
		end

	reset_height is
			-- Use default height.
		do
			lf_set_iheight (0)
			is_pt := false
		ensure
			done: is_default_height and (not is_cell_height) 
				and (not is_point) and (not is_character_height)
		end

feature -- Style

	reset_style is
			-- Reset style to normal.
		do
			set_weight (Normal)
			reset_italic
			reset_strike_out
			reset_underline
		end

	is_italic: BOOLEAN is
			-- Is italic?
		do
			Result := (lf_italic = true)
		end
		
	set_italic is
			-- Use italics.
		do
			lf_set_italic (true)
		ensure
			done: is_italic
		end

	reset_italic is
			-- Do not use italics.
		do
			lf_set_italic (false)
		ensure
			done: not is_italic
		end

	is_underline: BOOLEAN is
			-- Is underlined?
		do
			Result := (lf_underline = true)
		end

	set_underline is
			-- Underline.
		do
			lf_set_underline (true)
		ensure
			done: is_underline
		end

	reset_underline is
			-- Do not underline.
		do
			lf_set_underline (false)
		ensure
			done: not is_underline
		end

feature -- Standard weight constants.

	Thin: INTEGER is 100
	Extra_light: INTEGER is 200
	Light: INTEGER is 300
	Normal: INTEGER is 400
	Medium: INTEGER is 500
	Semi_bold: INTEGER is 600
	Bold: INTEGER is 700
	Extra_bold: INTEGER is 800
	Heavy: INTEGER is 900

feature -- Weight (point)
	
	weight: INTEGER is
			-- Current weight.
		do
			Result := lf_iweight
		end

	is_default_weight: BOOLEAN is
			-- Is current weight the default?
		do
			Result := (lf_iweight = 0)
		end


	set_weight (value: INTEGER) is
			-- Set weight (1-1000).
		require
			valid_weight: value > 0 and value <= 1000
		do
			lf_set_iweight (value)
		end

	reset_weight is
			-- Use default weight.
		do
			lf_set_iweight (0)
		end
	
feature -- Line orientation

	line_orientation: INTEGER is
			-- Get line orientation angle.
		do
			Result := lf_escapement
		end

	set_line_orientation (angle: INTEGER) is
			-- Line orientation, 10th of degree, x-axis, anticlockwise.
		do
			lf_set_escapement (angle)
		end

feature -- (Extended) Character orientation

	character_orientation: INTEGER is
			-- Get character orientation.
		do
			Result := lf_orientation
		end

	set_character_orientation (angle: INTEGER) is
			-- Individual character orientation, 10th of degree, x-axis, 
			-- anticlockwise.
		do
			lf_set_orientation (angle)
		end

feature -- (Extended) Width

	width: INTEGER is
			-- Get current width.
		do
			Result := lf_iwidth
		end

	is_default_width: BOOLEAN is
			-- Is default width?
		do
			Result := (lf_iwidth = 0)
		end

	reset_width is
			-- Font width optimised from optimal aspect ratio.
		do
			lf_set_iwidth (0)
		ensure
			done: is_default_width
		end

	set_width (w: INTEGER) is
			-- Set required width of font.
		require
			valid_width: w > 0
		do
			lf_set_iwidth (w)
		ensure
			done: not is_default_width
		end

feature -- (Extended) strike out

	is_strike_out: BOOLEAN is
			-- Is text struck out?
		do
			Result := (lf_strike_out = true)
		end

	set_strike_out is
			-- Enable strike out.
		do
			lf_set_strike_out (true)
		ensure
			done: is_strike_out
		end

	reset_strike_out is
			-- Disable strike out.
		do
			lf_set_strike_out (false)
		ensure
			done: not is_strike_out
		end

feature -- (Extended) Pitch

	is_pitch_fixed: BOOLEAN is
			-- Fixed-size font?
		do
			Result := (lf_pitch = wrt.Fixed_pitch)
		end

	is_pitch_unknown: BOOLEAN is
			-- Unknown pitch?
		do
			Result := (lf_pitch = 0)
		end

	is_pitch_variable: BOOLEAN is
			-- Proportional font?
		do
			Result := (lf_pitch = wrt.Variable_pitch)
		end

	reset_pitch is
		do
			lf_set_pitch (0)
		ensure
			done: is_pitch_unknown
		end

	set_pitch_fixed is
		do
			lf_set_pitch (wrt.Fixed_pitch)
		ensure
			done: is_pitch_fixed
		end

	set_pitch_variable is
		do
			lf_set_pitch (wrt.Variable_pitch)
		ensure
			done: is_pitch_variable
		end

feature -- (Extended) Font family

	is_family_unknown: BOOLEAN is
			-- Font family is not known.
		do
			Result := (lf_family = 0)
		end

	is_family_decorative: BOOLEAN is
			-- Decorative: novelty font such as old english.
		do
			Result := (lf_family = wrt.Ff_decorative)
		end

	is_family_swiss: BOOLEAN is
			-- Swiss: proportional and sans-serif.
		do
			Result := (lf_family = wrt.Ff_swiss)
		end

	is_family_roman: BOOLEAN is
			-- Roman: proportional and serif (classic font such as Times).
		do
			Result := (lf_family = wrt.Ff_roman)
		end

	is_family_modern: BOOLEAN is
			-- Modern: fixed-pitch (such as Courier).
		do
			Result := (lf_family = wrt.Ff_modern)
		end

	is_family_script: BOOLEAN is
			-- Script: handwriting like.
		do
			Result := (lf_family = wrt.Ff_script)
		end
	
	reset_family is
			-- No family preference.
		do
			lf_set_family (0)
		ensure
			done: is_family_unknown
		end

	set_family_decorative is
		do
			lf_set_family (wrt.Ff_decorative)
		ensure
			done: is_family_decorative
		end

	set_family_swiss is
		do
			lf_set_family (wrt.Ff_swiss)
		ensure
			done: is_family_swiss
		end

	set_family_roman is
		do
			lf_set_family (wrt.Ff_roman)
		ensure
			done: is_family_roman
		end

	set_family_modern is
		do
			lf_set_family (wrt.Ff_modern)
		ensure
			done: is_family_modern
		end

	set_family_script is
		do
			lf_set_family (wrt.Ff_script)
		ensure
			done: is_family_script
		end
		
feature -- (Extended) charset

	is_charset_ansi: BOOLEAN is
			-- Is font using ANSI (ISO Latin-1) character set?
		do
			Result := (lf_charset = wrt.Ansi_charset)
		end

	is_charset_default: BOOLEAN is
			-- No specified character set.
		do
			Result := (lf_charset = wrt.Default_charset)
		end

	is_charset_symbol: BOOLEAN is
			-- Symbol font?
		do
			Result := (lf_charset = wrt.Symbol_charset)
		end

	is_charset_oem: BOOLEAN is
			-- Is OEM (usually PC437) character set?
		do
			Result := (lf_charset = wrt.Oem_charset)
		end

	reset_charset, set_charset_ansi is
			-- Use Windows' ANSI (Latin-1) or Unicode charset.
		do
			lf_set_charset (wrt.Ansi_charset)
		ensure
			done: is_charset_ansi
		end

	set_charset_default is
			-- Use default charset.
		do
			lf_set_charset (wrt.Default_charset)
		ensure
			done: is_charset_default
		end

	set_charset_symbol is
			-- Use symbol charset.
		do
			lf_set_charset (wrt.Symbol_charset)
		ensure
			done: is_charset_symbol
		end

	set_charset_oem is
			-- Use OEM (device dependent) charset
		do
			lf_set_charset (wrt.Oem_charset)
		ensure
			done: is_charset_oem
		end

end -- class LOGICAL_FONT
