-- Fenestra; version 0.80(beta); 1998-09-27

-- 
-- Copyright (c) 1998 Franck Arnaud, http://www.altsoft.demon.co.uk/
-- 
-- This package is distributed under the Eiffel Forum Freeware License, 
-- version 1 (see file forum.txt for full text). The main points are:
-- 
-- "Permission is hereby granted, without written agreement and without 
--  license or royalty fees, to use, copy, modify and/or distribute this 
--  package, provided that: copyright notices are retained unchanged [...]
-- 
--  Permission is hereby also granted [...] to distribute binary programs 
--  which depend on this package, provided that: if the binary program 
--  depends on a modified version of this package, you must publicly release 
--  the modified version of this package [...].
-- 
--  THIS PACKAGE IS PROVIDED "AS IS" AND WITHOUT WARRANTY [...]"
-- 

indexing
	cluster: Fenestra, Graphics
	description: "Graphic device associated with a printer"
	interface: client

class PRINTER_DEVICE

inherit
	GRAPHIC_DEVICE
		end

creation
	make_default

creation {PRINT_DIALOG}
	make_handle

feature {CREATION_NONE} -- Creation

	make_default is
			-- Create a printer device for the default printer, 
			-- if it exists.
		do
			initialise
			set_handle (wrt.win_std_print (0, true, true))
			!!name.make (0)
			if is_ready then
				initialise_attributes
			end
		ensure
			can_fail: -- is_ready = true or is_ready = false
		end

feature {NONE} -- Creation

	make_handle (h: INTEGER) is
		do
			initialise
			
			set_handle (h)
			check ok: is_ready end

			!!name.make (0)
			initialise_attributes
		end

feature {NONE}
	
	name: STRING

feature -- Access

	is_ready: BOOLEAN is
			-- Is the device ready?
		do
			Result := is_valid_handle
		end

	prepare is
		do
		ensure then
			ready_at_creation: false
		end

	release is
		do
			dispose
		end

feature {NONE}
	
	dispose is
			-- Finalisation.
		do
			if handle = Invalid_handle then
				win_gdi_delete (handle)
				handle := Invalid_handle
			end
		end

feature -- Printing

	set_document_name (str: STRING) is
			-- Document name for display in Print Manager.
		require
			valid: str /= Void
		do
			name.copy (str)
		end

	start_document is
			-- Start a document.
		do
			wrt.win_gdi_startdoc (handle, wrt.ew_string (name).to_c)
		end

	end_document is
			-- End the current document.
		do
			wrt.win_gdi_enddoc (handle)
		end

	start_page is
			-- Start a page.
		do
			wrt.win_gdi_startpage (handle)
		end

	end_page is
			-- End the current page.
		do
			wrt.win_gdi_endpage (handle)
		end

invariant
	valid_name: name /= Void

end -- class PRINTER_DEVICE
