-- Fenestra; version 0.80(beta); 1998-09-27

-- 
-- Copyright (c) 1998 Franck Arnaud, http://www.altsoft.demon.co.uk/
-- 
-- This package is distributed under the Eiffel Forum Freeware License, 
-- version 1 (see file forum.txt for full text). The main points are:
-- 
-- "Permission is hereby granted, without written agreement and without 
--  license or royalty fees, to use, copy, modify and/or distribute this 
--  package, provided that: copyright notices are retained unchanged [...]
-- 
--  Permission is hereby also granted [...] to distribute binary programs 
--  which depend on this package, provided that: if the binary program 
--  depends on a modified version of this package, you must publicly release 
--  the modified version of this package [...].
-- 
--  THIS PACKAGE IS PROVIDED "AS IS" AND WITHOUT WARRANTY [...]"
-- 

indexing
	cluster: Fenestra, Graphics
	description: "Holds the coordinates of a rectangle in a 2D graphic space"
	interface: client
	
class RECTANGLE

inherit
	P_TEXT_OBJECT
		redefine
			out
		end

creation
	make,
	make_points,
	from_string

feature {CREATION_NONE} -- Creation

	make is
			-- Create default.
		do
			x := 0
			y := 0
			width := 1
			height := 1
		end

feature -- General
	
	out: STRING is
		do
			!!Result.make (0)
			Result.append_string (lower.out)
			Result.append_character ('-')
			Result.append_string (upper.out)
		end

feature {NONE} -- Effect P_TEXT_OBJECT
	
	object_to_string (tm: P_TEXT_OBJECT_MANAGER) is
		do
			tm.put_integer (1)
			tm.put_integer (lower.x)
			tm.put_integer (lower.y)
			tm.put_integer (upper.x)
			tm.put_integer (upper.y)
		end

	string_to_object (tm: P_TEXT_OBJECT_MANAGER) is
		local
			version: INTEGER
			low, up: POINT
		do
			make

			tm.get_integer  version := tm.last_integer
			!!low.make
			tm.get_integer  low.set_x (tm.last_integer)
			tm.get_integer  low.set_y (tm.last_integer)
			!!up.make
			tm.get_integer  up.set_x (tm.last_integer)
			tm.get_integer  up.set_y (tm.last_integer)

			if tm.in_success and then version = 1 then
				make_points (low, up)
			end
		end

feature {NONE} -- actual coordinates, private component
	
	x, y: INTEGER

feature -- Public rectangle coordinates
	
	width: INTEGER
			-- Rectangle width
	height: INTEGER
			-- Rectangle height

feature -- Points making the rectangle

	lower: POINT is 
			-- Lower point (coordinates < upper).
		do
			!!Result.make_coord (x, y)
		ensure
			is_copy: -- can be modified
		end


	upper: POINT is
			-- Upper point.
		do
			!!Result.make_coord (x + width - 1, y + height - 1)
		ensure
			is_copy: -- can be modified
		end

feature -- Changing coordinates

	set_lower (newl: POINT) is
			-- Change lower point.
		require
			valid_point: newl /= Void
			lower: newl.x <= upper.x and newl.y <= upper.y
		do
			width := upper.x - newl.x + 1
			height := upper.y - newl.y + 1
			x := newl.x
			y := newl.y
		end
	
	set_upper (newu: POINT) is
			-- Change upper point.
		require
			valid_point: newu /= Void
			upper: newu.x >= lower.x and newu.y >= lower.y
		do
			width := newu.x - lower.x + 1
			height := newu.y - lower.y + 1
		end

	set_width (nwidth: INTEGER) is
			-- Change rectangle width.
		require
			positive_size: nwidth >= 0
		do
			width := nwidth
		ensure
			done: width = nwidth
			lower: -- lower constant
		end

	set_height (nheight: INTEGER) is
			-- Change rectangle height.
		require
			positive_size: nheight >= 0
		do
			height := nheight
		ensure
			done: height = nheight
			lower: -- lower constant 
		end

	set_points, make_points (one: POINT; two: POINT) is
			-- Initialise rectangle from two points.
		require
			valid_points: one /= Void and then two /= Void
		do
			-- x
			if one.x < two.x then
				x := one.x
				width := two.x - one.x + 1
			else
				x := two.x
				width := one.x - two.x + 1
			end

			-- y
			if one.y < two.y then
				y := one.y
				height := two.y - one.y + 1
			else
				y := two.y
				height := one.y - two.y + 1
			end
		end

feature -- Helper functions

	set_lower_x (xc: INTEGER) is
		require
			ok: xc <= upper.x
		local
			p: POINT
		do
			!!p.make_coord (xc, y)
			set_lower (p)
		end

	set_lower_y (yc: INTEGER) is
		require
			ok: yc <= upper.y
		local
			p: POINT
		do
			!!p.make_coord (x, yc)
			set_lower (p)
		end   
		
	set_upper_x (xc: INTEGER) is
		require
			ok: xc >= lower.x
		local
			p: POINT
		do
			!!p.make_coord (xc, upper.y)
			set_upper (p)
		end

	set_upper_y (yc: INTEGER) is
		require
			ok: yc >= lower.y
		local
			p: POINT
		do
			!!p.make_coord (upper.x, yc)
			set_upper (p)
		end

feature -- Actions 

	translate_lower (pt: POINT) is
			-- Translate rectangle so that lower point is equal to `pt'
			-- Rectangle size is unchanged. 
		require
			valid_point: pt /= Void
		do
			x := pt.x
			y := pt.y
		ensure
			done: lower.is_equal (pt)
			size_unchanged: (width = old width) and (height = old height)
		end

	translate_center (pt: POINT) is
			-- Translate rectangle so that its center is equal to `pt'.
		require
			valid_point: pt /= Void
		do
			x := pt.x - width // 2
			y := pt.y - height // 2
		ensure
			done: center.is_equal (pt)
			size_unchanged: (width = old width) and (height = old height)
		end

	translate_upper (pt: POINT) is
			-- Translate rectangle so that upper point it `pt'.
		require
			valid: pt /= Void
		do
			x := pt.x - width + 1
			y := pt.y - height + 1
		ensure
			done: upper.is_equal (pt)
			size_unchanged: (width = old width) and (height = old height)
		end

feature -- Update

	add (other: RECTANGLE) is
			-- Union of rectangles.
			-- Current := (Current) U (other)
		require
			ok: other /= Void
		do
			if other.lower.x < lower.x then
				set_lower_x (other.lower.x)
			end
			if other.lower.y < lower.y then
				set_lower_y (other.lower.y)
			end
			if other.upper.x > upper.x then
				set_upper_x (other.upper.x)
			end
			if other.upper.y > upper.y then
				set_upper_y (other.upper.y)
			end
		end

feature -- Queries

	center: POINT is
			-- Return the rectangle centre point.
		do
			!!Result.make_coord (x + (width//2), y + (height//2))
		ensure
			is_copy: -- can be modified
		end

	is_point: BOOLEAN is
			-- Is the rectangle reduced to a point?
		do
			Result := ( width = 1 and height = 1)
		end

	is_line: BOOLEAN is
			-- Is the rectangle reduced to a line (but not a point)?
		do
			Result := ( width = 1 xor height = 1 )
		ensure
			line_not_point: Result implies (not is_point)
		end
	
	is_thin: BOOLEAN is
			-- Is the rectangle a line or a point?
		do
			Result := width = 1 or height = 1
		ensure
			done: Result = (is_line or is_point)
		end

	has_point (other: POINT): BOOLEAN is
			-- Is point inside current? (not strict)
		require
			valid_point: other /= Void
		do
			Result := other.x >= x and other.y >= y and other.x <= upper.x and other.y <= upper.y
		end

	is_inside (other: RECTANGLE): BOOLEAN is
			-- Is current inside other? (not strict)
		require
			valid_rectangle: other /= Void
		do
			Result := lower.x >= other.lower.x and lower.y >= other.lower.y 
					and upper.x <= other.upper.x and upper.y <= other.upper.y 
		end

invariant

	positive_offset: width > 0 and height > 0
	coherent_coord: upper.x = lower.x + width - 1 and upper.y = lower.y + height - 1

end -- class RECTANGLE

