-- Fenestra; version 0.80(beta); 1998-09-27

-- 
-- Copyright (c) 1998 Franck Arnaud, http://www.altsoft.demon.co.uk/
-- 
-- This package is distributed under the Eiffel Forum Freeware License, 
-- version 1 (see file forum.txt for full text). The main points are:
-- 
-- "Permission is hereby granted, without written agreement and without 
--  license or royalty fees, to use, copy, modify and/or distribute this 
--  package, provided that: copyright notices are retained unchanged [...]
-- 
--  Permission is hereby also granted [...] to distribute binary programs 
--  which depend on this package, provided that: if the binary program 
--  depends on a modified version of this package, you must publicly release 
--  the modified version of this package [...].
-- 
--  THIS PACKAGE IS PROVIDED "AS IS" AND WITHOUT WARRANTY [...]"
-- 

indexing
	cluster: Fenestra, Graphics
	description: "Graphic device associated with a screen window"
	interface: client

class WINDOW_DEVICE

inherit
	GRAPHIC_DEVICE
		end

creation {WINDOW} -- Window devices are created only by WINDOWS
	make

feature {NONE} -- Creation

	make (par: WINDOW) is
		do
			parent := par
			initialise
		end

feature {NONE}

	parent: WINDOW

	internal_ready: BOOLEAN

feature -- Access

	is_ready: BOOLEAN is
			-- Is the device ready to accept graphic command?
		do
			Result := internal_ready and parent.is_valid
		end

	prepare is
			-- Get the device ready for graphic commands.
		do
			internal_ready := true
			set_handle (wrt.win_gdi_get (parent.handle))
			initialise_attributes
		end

	release is
			-- Invalidate graphic device.
		do
			wrt.win_gdi_release (handle, parent.handle)
			reset_handle
			internal_ready := false
		end

feature {NONE}
	
	dispose is
			-- Finalisation.
		do
			check
				already_released: handle = Invalid_handle
			end
		end

feature {WINDOW} -- Window access

	prepare_paint (newh: INTEGER) is
			-- Prepare the window for paint (reply to WM_PAINT).
		require
			not_ready: not is_ready 
		do
			internal_ready := true
			set_handle (newh)
			initialise_attributes
		ensure
			ready: is_ready
		end

	release_paint is
			-- Release.
		require
			ready: is_ready
		do
			reset_handle
			internal_ready := false
		ensure
			not_ready: not is_ready
		end

invariant
	good_parent: parent /= Void

end -- class WINDOW_DEVICE
