-- Fenestra; version 0.80(beta); 1998-09-27

-- 
-- Copyright (c) 1998 Franck Arnaud, http://www.altsoft.demon.co.uk/
-- 
-- This package is distributed under the Eiffel Forum Freeware License, 
-- version 1 (see file forum.txt for full text). The main points are:
-- 
-- "Permission is hereby granted, without written agreement and without 
--  license or royalty fees, to use, copy, modify and/or distribute this 
--  package, provided that: copyright notices are retained unchanged [...]
-- 
--  Permission is hereby also granted [...] to distribute binary programs 
--  which depend on this package, provided that: if the binary program 
--  depends on a modified version of this package, you must publicly release 
--  the modified version of this package [...].
-- 
--  THIS PACKAGE IS PROVIDED "AS IS" AND WITHOUT WARRANTY [...]"
-- 

indexing
	cluster: Controls, Internal
	description: "Common functionality for built-in scrollbars"
	interface: internal

deferred class BASE_SCROLLBAR

inherit
	SCROLLBAR
		end
	WIN_API
		end

feature {NONE} -- Internal status

	type: INTEGER
			-- Scrollbar type (OS).

	sb_handle: INTEGER is
		deferred
		end

feature {NONE} -- Initialisation (for creation)

	init is
			-- Initialise creation.
		do
			tracked := false
			set_range (Default_range)
			set_line_step (1)
			set_page_step (Default_range // 10) -- 20%
			set_slider_size (Default_range // 10)
			set_slider_position (Default_range // 2)
		end

feature -- Range & conversion from/to ratios

	range: INTEGER;
			-- Slider position range.

	set_range (rg: INTEGER) is
			-- Set slider position range.
		do
			range := rg
			wrt.win_scroll_range (sb_handle, type, range)
		end

feature -- Slider position and size

	slider_size: INTEGER;
			-- Size of the slider. 

	set_slider_size (s: INTEGER) is
			-- Set the size of the slider.
		do
			slider_size := s -- Current WIN32 API has only square sliders
		end

	slider_position: INTEGER is
			-- Slider position range.
		do
			Result := wrt.win_scroll_get (sb_handle, type)
			if Result = 0 then -- disabled scrollbar
				Result := 1
			end
		end

	set_slider_position (pos: INTEGER) is
			-- Set slider position range.
		do
			wrt.win_scroll_set (sb_handle, type, pos)
		end

feature -- Status
	
	tracked: BOOLEAN;
			-- Is the mouse cursor currently moving slider

feature {NONE}
	
	scrollbar_event (code: INTEGER; position: INTEGER) is
			-- Event implementation.
		local
			req_position: INTEGER
		do
			if code = wrt.Sb_pagedown then
				req_position := (range).min (slider_position + page_step)
			elseif code = wrt.Sb_pageup then
				req_position := (1).max (slider_position - page_step)
			elseif code = wrt.Sb_linedown then
				req_position := (range).min (slider_position + line_step)
			elseif code = wrt.Sb_lineup then
				req_position := (1).max (slider_position - line_step)
			elseif code = wrt.Sb_thumbtrack then
				tracked := true
				req_position := position
			elseif code = wrt.Sb_thumbposition then
				tracked := false
				req_position := position
			else
				req_position := slider_position
			end

			if req_position /= slider_position then -- actually moved.
				when_slider_moved (req_position)
			end
		end

end -- class BASE_SCROLLBAR
