-- Fenestra; version 0.80(beta); 1998-09-27

-- 
-- Copyright (c) 1998 Franck Arnaud, http://www.altsoft.demon.co.uk/
-- 
-- This package is distributed under the Eiffel Forum Freeware License, 
-- version 1 (see file forum.txt for full text). The main points are:
-- 
-- "Permission is hereby granted, without written agreement and without 
--  license or royalty fees, to use, copy, modify and/or distribute this 
--  package, provided that: copyright notices are retained unchanged [...]
-- 
--  Permission is hereby also granted [...] to distribute binary programs 
--  which depend on this package, provided that: if the binary program 
--  depends on a modified version of this package, you must publicly release 
--  the modified version of this package [...].
-- 
--  THIS PACKAGE IS PROVIDED "AS IS" AND WITHOUT WARRANTY [...]"
-- 

indexing
	cluster: Fenestra, Internal, Controls
	description: "Utility features for drawing 3D controls"
	interface: client, mixin

class DRAW_3D_CONTROLS

feature
	
	frame_down (device: GRAPHIC_DEVICE; rect: RECTANGLE) is
			-- Draw `punched' frame.
		require
			device_ok: device /= Void
			device_ready: device.is_ready
			rect_ok: rect /= Void
		do
			draw_background (device, rect)
			draw_angle (device, dark, rect, 0, true)
			draw_angle (device, light, rect, 0, false)
		end
			

	draw_button (device: GRAPHIC_DEVICE; rect: RECTANGLE; is_down: BOOLEAN) is
			-- Draw button shell.
		require
			device_ok: device /= Void
			device_ready: device.is_ready
			rect_ok: rect /= Void
		do
			draw_background (device, rect)
			-- draw border
			draw_angle (device, light, rect, 0, not is_down)
			draw_angle (device, very_dark, rect, 0, is_down)
			draw_angle (device, dark, rect, 1, is_down)
		end

feature {NONE}
	
	draw_background (device: GRAPHIC_DEVICE; rect: RECTANGLE) is
		require
			ok_device: device /= Void and device.is_ready
			ok_rect: rect /= Void
		local
			gr: GRAPHICS_ROUTINES
			dr: DRAWABLE_RECTANGLE
		do
			!!gr

			-- draw background
			gr.set_pen_and_brush (device, background)
			
			!!dr.make
			dr.set_position (rect)
			device.draw (dr)
		end

	draw_angle (device: GRAPHIC_DEVICE; color: COLOR; rect: RECTANGLE; pos: INTEGER; sunny: BOOLEAN) is
			-- Drawing the angle of a rectangle.
		require
			ok_device: device /= Void and device.is_ready
			ok_color: color /= Void
			ok_rect: rect /= Void
			width: pos >= 0
			device: device.is_ready
		local
			dp: DRAWABLE_POINT
			line: DRAWABLE_LINE
			pt: POINT
			gr: GRAPHICS_ROUTINES
		do
			!!gr
			!!pt.make
			!!line.make

			device.set_pen (gr.color_pen (color))
			
			pt.set_xy (rect.lower.x+pos, rect.upper.y-pos)
			line.set_start (pt)
			
			if sunny then
				pt := rect.lower
				pt.set_xy (pt.x+pos, pt.y+pos)
			else
				pt := rect.upper
				pt.set_xy (pt.x-pos, pt.y-pos)
			end
			line.set_final (pt)
			device.draw (line)
			
			!!dp.make
			dp.set_position (pt)
			dp.set_color (color)
			device.draw (dp)
			
			pt.set_xy (rect.upper.x-pos, rect.lower.y+pos)
			line.set_start (pt)
			device.draw (line)
		end

feature {NONE} -- Colours

	background: COLOR is
		do
			!RGB_COLOR!Result.light_gray
		end

	light: COLOR is
		do
			!RGB_COLOR!Result.white
		end
	
	very_dark: COLOR is
		do
			!RGB_COLOR!Result.black
		end
	
	dark: COLOR is
		do
			!RGB_COLOR!Result.dark_gray
		end

end -- class DRAW_3D_CONTROLS
