-- Fenestra; version 0.80(beta); 1998-09-27

-- 
-- Copyright (c) 1998 Franck Arnaud, http://www.altsoft.demon.co.uk/
-- 
-- This package is distributed under the Eiffel Forum Freeware License, 
-- version 1 (see file forum.txt for full text). The main points are:
-- 
-- "Permission is hereby granted, without written agreement and without 
--  license or royalty fees, to use, copy, modify and/or distribute this 
--  package, provided that: copyright notices are retained unchanged [...]
-- 
--  Permission is hereby also granted [...] to distribute binary programs 
--  which depend on this package, provided that: if the binary program 
--  depends on a modified version of this package, you must publicly release 
--  the modified version of this package [...].
-- 
--  THIS PACKAGE IS PROVIDED "AS IS" AND WITHOUT WARRANTY [...]"
-- 

indexing
	cluster: Fenestra, System, Internal
	description: "Overlapped structure for asynchronous I/O"
	interface: client, internal

class WIN_OVERLAPPED

inherit
	WIN_OVERLAPPED_CONST
		end
	WIN_API
		end
	GUI_COMMAND
		end

creation {F_ASYNCH_IO}
	make_read, make_write

feature {NONE} -- Creation

	make_read (par: F_ASYNCH_IO) is
			-- Create OVERLAPPED structure for file reading.
		require
			ok: par /= Void
		do
			parent := par

			-- create structure
			!!overlapped.make (Overlapped_size)

			-- create event
			!!event.make_anonymous
			event.reset
			event.set_command (Current)
		ensure
			keep_reference: parent = par
		end

	make_write (par: F_ASYNCH_IO) is
			-- Create OVERLAPPED structure for file writing.
		require
			ok: par /= Void
		do
			make_read (par)
			write := true
		end

feature {NONE}
	
	parent: F_ASYNCH_IO
	write: BOOLEAN

	overlapped: F_MEMORY
	event: F_EVENT

feature {F_ASYNCH_IO}
	
	prepare (position: INTEGER) is
			-- Prepare overlapped structure.
		require
			position_ok: position >= 0
			one_at_a_time: -- previous finished
		do
			event.reset
			overlapped.set_double_word (Offset_offset, position)
			overlapped.set_double_word (Offsethigh_offset, 0)
			overlapped.set_double_word (Event_offset, event.handle)
		end

	to_windows: POINTER is
			-- Pointer to OVERLAPPED structure.
		do
			Result := overlapped.to_windows
		end
	
	wait is
			-- Start waiting for result (non-blocking).
		do
			event.wait
		end

feature -- Event(s)
	
	execute is
			-- Finished waiting.
		do
			event.reset
			if write then
				parent.internal_when_written
			else
				parent.internal_when_read
			end
		end

end -- class WIN_OVERLAPPED
