-- Fenestra; version 0.80(beta); 1998-09-27

-- 
-- Copyright (c) 1998 Franck Arnaud, http://www.altsoft.demon.co.uk/
-- 
-- This package is distributed under the Eiffel Forum Freeware License, 
-- version 1 (see file forum.txt for full text). The main points are:
-- 
-- "Permission is hereby granted, without written agreement and without 
--  license or royalty fees, to use, copy, modify and/or distribute this 
--  package, provided that: copyright notices are retained unchanged [...]
-- 
--  Permission is hereby also granted [...] to distribute binary programs 
--  which depend on this package, provided that: if the binary program 
--  depends on a modified version of this package, you must publicly release 
--  the modified version of this package [...].
-- 
--  THIS PACKAGE IS PROVIDED "AS IS" AND WITHOUT WARRANTY [...]"
-- 

indexing
	cluster: Fenestra, Internal
	description: "Access to the low-level runtime system"
	interface: internal, client

class WIN_RUNTIME

inherit
	WIN_CONST

feature -- Exception

	Excep_listbox: STRING is "Win32 listbox memory error"
	Excep_combo: STRING is "Win32 combobox memory error"
	Excep_editbox: STRING is "Win32 editbox memory error"
	Excep_file: STRING is "File open failed" 

	raise_exception (name: STRING) is
			-- Portable raise function.
		local
			sys: P_SYSTEM
		do
			sys.raise_exception (name) --? use P_SYSTEM directly
		end

feature -- Convert types

	int_to_char (int: INTEGER): CHARACTER is
			-- Character from character code (missing feature from kernel).
		do
			Result := win_int2char (int)
			--? use P_SYSTEM?
		end

	int_to_bool (i: INTEGER): BOOLEAN is 
			-- Convert window integer representing a boolean value.
		do 
			Result := (i /= 0) 
		end

feature -- String (convert to low level value)

	ew_string (in: STRING): F_MEMORY is
			-- Convert Eiffel string to memory block (then LPTSTR).
		require
			ok: in /= Void
		do
			!!Result.make (in.count + 1)
			Result.set_cstring (1, in.count+1, in)
		end
	
feature -- String (return value)

	win_laststring: STRING is
			-- Make from string.
		local
			code, index: INTEGER
			str_pointer: POINTER
		do
			-- get pointer
			str_pointer := win_str_get

			-- copy string
			--? unicode
			!!Result.make (0)
			from
				index := 0
				code := 1
			until
				code = 0
			loop
				code := win_getbyte (str_pointer, index)
				index := index + 1
				if code /= 0 then
					Result.append_character (int_to_char (code))
				end
			end

			-- free
			win_str_free
		ensure
			done: Result /= Void
		end

feature {NONE}

	win_str_get: POINTER is external "C" end

	win_str_free is external "C" end

feature -- Misc return values
	
	win_lastbool1: BOOLEAN is external "C" end

	win_lastint1: INTEGER is external "C" end

	win_lastint2: INTEGER is external "C" end

	win_lastint3: INTEGER is external "C" end

	win_lastint4: INTEGER is external "C" end

feature -- Notifier

	win_init_notifier (notifier: NOTIFIER) is external "C" end

	win_init_notifier_ex (notifier: NOTIFIER; callback1, callback2, callback3: POINTER) is external "C" end

feature -- Memory for F_MEMORY

	win_mem_alloc (size: INTEGER): POINTER is external "C" end

	win_mem_free (ptr: POINTER) is external "C" end

feature -- Basic types pointer access

	win_getbyte (ptr: POINTER; offset: INTEGER): INTEGER is external "C" end

	win_setbyte (ptr: POINTER; offset: INTEGER; value: INTEGER) is external "C" end

	win_getword (byteptr: POINTER): INTEGER is external "C" end

	win_getlong (byteptr: POINTER): INTEGER is external "C" end

	win_setword (byteptr: POINTER; offset: INTEGER; value: INTEGER) is external "C" end
	
	win_setlong (byteptr: POINTER; offset: INTEGER; value: INTEGER) is external "C" end
	
	win_setlong_l (byteptr: POINTER; offset: INTEGER; value: INTEGER) is external "C" end
	
	win_setlong_b (byteptr: POINTER; offset: INTEGER; value: INTEGER) is external "C" end

	win_int2char (in: INTEGER): CHARACTER is external "C" end

	win_bitand (a, b: INTEGER): INTEGER is external "C" end

	win_bitor (a, b: INTEGER): INTEGER is external "C" end

	win_bitxor (a, b: INTEGER): INTEGER is external "C" end

	win_muldiv (m1, m2, d1: INTEGER): INTEGER is external "C" end

feature -- Windows

	win_show (winid: INTEGER; show_how: INTEGER) is external "C" end

	win_subclass_control (winid: INTEGER) is external "C" end

	win_event: BOOLEAN is external "C" end

	win_initwin is external "C" end

	win_create: INTEGER is external "C" end

	win_create_popup (winid: INTEGER): INTEGER is external "C" end

	win_create_dialog (winid: INTEGER; thick: BOOLEAN): INTEGER is external "C" end

	win_create_appdialog (thick: BOOLEAN): INTEGER is external "C" end
	
	win_create_child (winid: INTEGER; cursor: BOOLEAN; horscroll: BOOLEAN; verscroll: BOOLEAN): INTEGER is external "C" end

	win_create_clrchild (winid: INTEGER; back_color: INTEGER; cursor: BOOLEAN; verscroll, horscroll: BOOLEAN): INTEGER is external "C" end

	win_close (winid: INTEGER) is external "C" end

	win_destroy (winid: INTEGER) is external "C" end

	win_quit is external "C" end

	win_border (winid: INTEGER; newb: BOOLEAN) is external "C" end

	win_enable (winid: INTEGER) is external "C" end
		
	win_disable (winid: INTEGER) is external "C" end

	win_update (winid: INTEGER) is external "C" end

	win_update_rect (winid: INTEGER; x, y, xp, yp: INTEGER) is external "C" end

	win_scroll (winid: INTEGER; dx, dy: INTEGER) is external "C" end

	win_title (winid: INTEGER; str: POINTER) is external "C" end

	win_gettitle (winid: INTEGER) is external "C" end

	win_size (winid, parid: INTEGER) is external "C" end

	win_clientsize (winid: INTEGER) is external "C" end

	win_capture (windid: INTEGER) is external "C" end

	win_endcapture is external "C" end

	win_resize (winid: INTEGER;x, y, xp, yp: INTEGER) is external "C" end

	win_resizeclient (winid: INTEGER;x, y, xp, yp: INTEGER) is external "C" end

	win_keystate (vkey: INTEGER): BOOLEAN is external "C" end

	win_isicon (winid: INTEGER): BOOLEAN is external "C" end

	win_isenable (winid: INTEGER): BOOLEAN is external "C" end

	win_isvisible (winid: INTEGER): BOOLEAN is external "C" end

	win_activate (winid: INTEGER) is external "C" end

	win_focus (winid: INTEGER) is external "C" end

	win_hasfocus (winid: INTEGER): BOOLEAN is external "C" end

	win_isactive (winid: INTEGER): BOOLEAN is external "C" end

	win_setfont (winid, fontid: INTEGER) is external "C" end

	win_getfont (winid: INTEGER): INTEGER is external "C" end

	win_back_color (winid: INTEGER; clr: INTEGER) is external "C" end

feature -- Clipboard

	win_clip_open (winid: INTEGER): BOOLEAN is external "C" end

	win_clip_close is external "C" end

	win_clip_empty is external "C" end

	win_clip_has (type: INTEGER): BOOLEAN is external "C" end

	win_clip_hastext: BOOLEAN is external "C" end

	win_clip_copytext (text_str: POINTER) is external "C" end

	win_clip_pastetext is external "C" end

	win_clip_copy (type: INTEGER;hdle: INTEGER) is external "C" end

	win_clip_paste (type: INTEGER): INTEGER is external "C" end

feature -- Drag & Drop

	win_drag_accept (dropid: INTEGER; stat: BOOLEAN) is external "C" end

	win_drag_files (dropid: INTEGER): INTEGER is external "C" end

	win_drag_getfile (dropid: INTEGER; index: INTEGER) is external "C" end

	win_drag_getpoint (dropip: INTEGER): BOOLEAN is external "C" end

	win_drag_finish (dropid: INTEGER) is external "C" end

feature -- Common dialogs

	win_message (winid: INTEGER; title_str, text_str: POINTER; icon, buttons: INTEGER): INTEGER is external "C" end

	win_std_color (winid: INTEGER; oldcolor: INTEGER): INTEGER is external "C" end

	win_std_font (winid: INTEGER; oldcolor: INTEGER; getcolor, hasfont: BOOLEAN; logfontptr: POINTER): BOOLEAN is external "C" end

	win_std_print (winid: INTEGER; getdc, getdefault: BOOLEAN): INTEGER is external "C" end

	win_std_file (open: BOOLEAN; winid: INTEGER; title_str, initdir_str, initfile_str, filters_str: POINTER) is external "C" end

feature -- Menus

	win_drawmenu (winid: INTEGER) is external "C" end

	win_menu_addpopup (menuid: INTEGER; popupid: INTEGER; name_str: POINTER) is external "C" end

	win_menu_additem (menuid: INTEGER; id: INTEGER ; name_str: POINTER) is external "C" end

	win_menu_addsep (menuid: INTEGER) is external "C" end

	win_menu_cbar: INTEGER is external "C" end

	win_menu_cpopup: INTEGER is external "C" end

	win_menu_destroy (menuid: INTEGER) is external "C" end

	win_menu_delete (menuid, itemid: INTEGER) is external "C" end

	win_menu_track (menuid, winid, x, y: INTEGER) is external "C" end

	win_menu_set (menuid, parentid: INTEGER) is external "C" end

	win_menu_chtext (menuid, itemid: INTEGER; newtext_str: POINTER) is external "C" end

	win_menu_enable (menuid, itemid: INTEGER; bool: BOOLEAN) is external "C" end
	
	win_menu_isenabled (menuid, itemid: INTEGER): BOOLEAN is external "C" end

	win_menu_checked (menuid, itemid: INTEGER; bool: BOOLEAN) is external "C" end

	win_menu_ischecked (menuid, itemid: INTEGER): BOOLEAN is external "C" end

feature -- Keyboard accelerators

	win_accel_create (acc_table: POINTER; size: INTEGER): INTEGER is external "C" end

	win_accel_enable (win_handle, acc_handle: INTEGER) is external "C" end

	win_accel_destroy (acc_handle: INTEGER) is external "C" end

feature -- Messages

	win_msg_void (winid: INTEGER;msgid: INTEGER) is external "C" end

	win_msg_bool (winid: INTEGER;msgid: INTEGER): BOOLEAN is external "C" end

	win_msg_setbool (winid: INTEGER;msgid: INTEGER;stat: BOOLEAN) is external "C" end

	win_msg_int (winid: INTEGER;msgid: INTEGER): INTEGER is external "C" end

	win_msg_setint (winid: INTEGER;msgid: INTEGER;value: INTEGER) is external "C" end

	win_postmsg_void (winid: INTEGER;msgid: INTEGER) is external "C" end

feature -- Controls

	win_but_isdown (winid: INTEGER): BOOLEAN is external "C" end

	win_but_default (winid: INTEGER; def: BOOLEAN) is external "C" end

	win_but_cpush (parent: INTEGER): INTEGER is external "C" end

	win_but_cdefpush (parent: INTEGER): INTEGER is external "C" end
	
	win_but_cradio (parent: INTEGER): INTEGER is external "C" end

	win_but_ccheck (parent: INTEGER): INTEGER is external "C" end

	win_but_ctristate (parent: INTEGER): INTEGER is external "C" end

	win_but_cgroupbox (parent: INTEGER): INTEGER is external "C" end

	----------------------------------------------------------------------

	win_scroll_create (winid: INTEGER ; is_horiz: BOOLEAN): INTEGER is external "C" end

	win_scroll_set (winid: INTEGER ; dir: INTEGER ; pos: INTEGER) is external "C" end

	win_scroll_get (winid: INTEGER ; dir: INTEGER): INTEGER is external "C" end
	
	win_scroll_range (winid: INTEGER ; dir: INTEGER ;range: INTEGER) is external "C" end

	----------------------------------------------------------------------

	win_listbox_create (winid: INTEGER; multi, sorted: BOOLEAN): INTEGER is external "C" end

	win_listbox_addtext (winid: INTEGER ; text_str: POINTER) is external "C" end

	win_listbox_selstring (winid: INTEGER;sel_str: POINTER) is external "C" end

	win_listbox_mitems (winid: INTEGER; dwordptr: POINTER; dwordptr_size: INTEGER): INTEGER is external "C" end

	win_listbox_gettext (winid: INTEGER ; idx: INTEGER) is external "C" end

	----------------------------------------------------------------------

	win_editbox_create (winid: INTEGER;multi, wrap: BOOLEAN): INTEGER is external "C" end

	----------------------------------------------------------------------

	win_combo_create (winid: INTEGER; editable, sorted: BOOLEAN): INTEGER is external "C" end

	win_combo_editctl (winid: INTEGER): INTEGER is external "C" end

	win_combo_laddtext (winid: INTEGER; text_str: POINTER) is external "C" end
	
	win_combo_selstring (winid: INTEGER;sel_str: POINTER) is external "C" end
	
	win_combo_lgettext (winid: INTEGER;idx: INTEGER) is external "C" end

	----------------------------------------------------------------------

	win_static_create (winid, style: INTEGER): INTEGER is external "C" end

	win_static_icon (winid, icon: INTEGER) is external "C" end

	----------------------------------------------------------------------
	
	win_dialog_unitx: INTEGER is external "C" end

	win_dialog_unity: INTEGER is external "C" end

	----------------------------------------------------------------------

	win_cursor_std (code: INTEGER): INTEGER is external "C" end

	win_cursor_rsc (name_str: POINTER): INTEGER is external "C" end

	win_cursor_set (cursid: INTEGER) is external "C" end

	win_cursor_show (value: BOOLEAN) is external "C" end

	----------------------------------------------------------------------

	win_caret_create (winid: INTEGER;w, h: INTEGER) is external "C" end

	win_caret_destroy is external "C" end

	win_caret_hide (winid: INTEGER) is external "C" end

	win_caret_show (winid: INTEGER) is external "C" end

	win_caret_position (x, y: INTEGER) is external "C" end

	----------------------------------------------------------------------

	win_icon_rsc (name_str: POINTER): INTEGER is external "C" end

	win_icon_set (winid, iconid: INTEGER) is external "C" end

	----------------------------------------------------------------------

	win_help_int (winid: INTEGER;helpfile_str: POINTER;command, value: INTEGER): BOOLEAN is external "C" end

	win_help_str (winid: INTEGER;helpfile_str: POINTER;command: INTEGER;value_str: POINTER): BOOLEAN is external "C" end

feature -- Graphics device interface: Context
	
	win_beginpaint (winid: INTEGER; paintstruct: POINTER): INTEGER is external "C" end

	win_endpaint (winid: INTEGER; paintstruct: POINTER) is external "C" end

	win_gdi_get (winid: INTEGER): INTEGER is external "C" end

	win_gdi_delete (dcid: INTEGER) is external "C" end
	
	win_gdi_save (dcid: INTEGER) is external "C" end

	win_gdi_restore (dcid: INTEGER) is external "C" end

	win_gdi_release (dcid: INTEGER; winid: INTEGER) is external "C" end

	win_gdi_startdoc (dcid: INTEGER; name_str: POINTER) is external "C" end
	
	win_gdi_enddoc (dcid: INTEGER) is external "C" end

	win_gdi_startpage (dcid: INTEGER) is external "C" end

	win_gdi_endpage (dcid: INTEGER) is external "C" end
	
	win_gdi_emf (fname_str: POINTER;dcid: INTEGER;w, h: INTEGER): INTEGER is external "C" end

	win_gdi_closeemf (dcid: INTEGER): INTEGER is external "C" end

	win_gdi_delemf (emfid: INTEGER) is external "C" end

	win_gdi_getemf (fname_str: POINTER): INTEGER is external "C" end

	win_gdi_drawemf (dcid: INTEGER; emfid: INTEGER; x, y, xp, yp: INTEGER) is external "C" end

	win_gdi_bmpdc (dcid: INTEGER): INTEGER is external "C" end

	win_gdi_loadbmp (name_str: POINTER): INTEGER is external "C" end

	win_gdi_bmpsize (dcid: INTEGER; w, h: INTEGER): INTEGER is external "C" end

	win_gdi_bmpw (dcid, bmpid: INTEGER): INTEGER is external "C" end

	win_gdi_bmph (dcid, bmpid: INTEGER): INTEGER is external "C" end

	win_gdi_bitblt (tid: INTEGER;tx, ty, tw, th: INTEGER;sid: INTEGER;sx, sy: INTEGER) is external "C" end

	win_gdi_strblt (tid: INTEGER;tx, ty, tw, th: INTEGER;sid: INTEGER;sx, sy, sw, sh: INTEGER) is external "C" end

	win_gdi_bmpsave (dcid: INTEGER;bmpid: INTEGER;fname_str: POINTER): BOOLEAN is external "C" end

	win_gdi_bmpload (dcid: INTEGER;fname_str: POINTER): INTEGER is external "C" end

feature -- Graphics device interface: Objects

	win_gdi_select (dcid: INTEGER; gdiobj: INTEGER) is external "C" end

	win_gdi_remove (gdiobj: INTEGER) is external "C" end
	
	win_gdi_stock (code: INTEGER): INTEGER is external "C" end
	
	win_gdi_pen (style, width, color: INTEGER): INTEGER is external "C" end

	win_gdi_expen (type, width, style, endstyle, joinstyle, color, brushstyle, brushpat: INTEGER): INTEGER is external "C" end

	win_gdi_brush (brushstyle, brushpat, color: INTEGER): INTEGER is external "C" end

feature -- GDI: Colour

	win_gdi_palette (logpalette_ptr: POINTER): INTEGER is external "C" end

	win_gdi_selectpal (dcid: INTEGER; palid: INTEGER) is external "C" end

	win_gdi_rgbcolor (red, green, blue: INTEGER): INTEGER is external "C" end

	win_gdi_rgbpalette (red, green, blue: INTEGER): INTEGER is external "C" end

	win_gdi_palcolor (index: INTEGER): INTEGER is external "C" end
	
feature -- GDI: Mode

	win_gdi_mapmode (dcid: INTEGER;mode: INTEGER) is external "C" end

	win_gdi_origin (dcid: INTEGER;x, y: INTEGER) is external "C" end

	win_gdi_winext (dcid: INTEGER;x, y: INTEGER) is external "C" end

	win_gdi_vportext (dcid: INTEGER;x, y: INTEGER) is external "C" end

	win_gdi_dp2lp (dcid: INTEGER;x, y: INTEGER) is external "C" end

	win_gdi_lp2dp (dcid: INTEGER;x, y: INTEGER) is external "C" end

	win_gdi_rop2 (dcid: INTEGER;mode: INTEGER) is external "C" end

	win_gdi_bkmode (dcid: INTEGER; mode: INTEGER) is external "C" end

	win_gdi_bkcolor (dcid: INTEGER; color: INTEGER) is external "C" end

feature -- GDI: Text

	win_gdi_text (dcid: INTEGER;x, y: INTEGER; str: POINTER) is external "C" end

	win_gdi_extext (dcid: INTEGER;posx, posy: INTEGER;rx, ry, rxp, ryp: INTEGER;clipped, filled: BOOLEAN; str: POINTER) is external "C" end
	
	win_gdi_textis (dcid: INTEGER;ispace: INTEGER) is external "C" end

	win_gdi_textclr (dcid: INTEGER;color: INTEGER) is external "C" end

	win_gdi_textalign (dcid: INTEGER;horiz, vert: INTEGER) is external "C" end

	win_gdi_textsize (dcid: INTEGER;str: POINTER) is external "C" end

	win_gdi_textmetr (dcid: INTEGER;code: INTEGER): INTEGER is external "C" end

	win_gdi_textface (dcid: INTEGER) is external "C" end
	
	win_gdi_textjustif (dcid: INTEGER;justif, brk: INTEGER) is external "C" end

	win_gdi_charsize (dcid: INTEGER;ch: INTEGER): INTEGER is external "C" end

	win_gdi_font (logfont_ptr: POINTER): INTEGER is external "C" end

	win_gdi_enumfonts (dcid: INTEGER; facename_str: POINTER; logont_buffer_size: INTEGER; logfont_buffer: POINTER): INTEGER is external "C" end

feature -- GDI: Unicode text

	win_gdi_wtext (charmap_ptr: POINTER; dcid: INTEGER; x, y: INTEGER; text_ptr: POINTER) is external "C" end

	win_gdi_wextext (charmap_ptr: POINTER; dcid: INTEGER;posx, posy: INTEGER;rx, ry, rxp, ryp: INTEGER;clipped, filled: BOOLEAN; str: POINTER) is external "C" end

	win_gdi_wtextsize (charmap_ptr: POINTER; dcid: INTEGER; text_ptr: POINTER) is external "C" end

feature -- GDI: Drawing primitives

	win_gdi_line (dcid: INTEGER;x, y, xp, yp: INTEGER) is external "C" end

	win_gdi_polyline (dcid: INTEGER;dword_ptr: POINTER; dword_ptr_size: INTEGER) is external "C" end

	win_gdi_bezier (dcid: INTEGER; dword_ptr: POINTER; dword_ptr_size: INTEGER) is external "C" end

	win_gdi_polygon (dcid: INTEGER; dword_ptr: POINTER; dword_ptr_size: INTEGER) is external "C" end

	win_gdi_polymode (dcid: INTEGER;alternate: BOOLEAN) is external "C" end

	win_gdi_focusrect (dcid: INTEGER;rx1, ry1, rx2, ry2: INTEGER) is external "C" end
	
	win_gdi_invertrect (dcid: INTEGER;rx1, ry1, rx2, ry2: INTEGER) is external "C" end
			
	win_gdi_rectangle (dcid: INTEGER;rx1, ry1, rx2, ry2: INTEGER) is external "C" end

	win_gdi_roundrect (dcid: INTEGER;rx1, ry1, rx2, ry2, cw, ch: INTEGER) is external "C" end
	
	win_gdi_ellipse (dcid: INTEGER;rx1, ry1, rx2, ry2: INTEGER) is external "C" end

	win_gdi_arc (dcid: INTEGER;rx1, ry1, rx2, ry2, sx, sy, fx, fy: INTEGER) is external "C" end

	win_gdi_chord (dcid: INTEGER;rx1, ry1, rx2, ry2, sx, sy, fx, fy: INTEGER) is external "C" end

	win_gdi_pie (dcid: INTEGER;rx1, ry1, rx2, ry2, sx, sy, fx, fy: INTEGER) is external "C" end

	win_gdi_arcdir (dcid: INTEGER;clockw: BOOLEAN) is external "C" end

	win_gdi_icon (dcid, icon: INTEGER;x, y: INTEGER) is external "C" end

	win_gdi_floodfill (dcid: INTEGER; x, y, color, mode: INTEGER): BOOLEAN is external "C" end

	win_gdi_getpixel (dcid: INTEGER; x, y: INTEGER): INTEGER is external "C" end

	win_gdi_setpixel (dcid: INTEGER; x, y, color: INTEGER) is external "C" end

	win_gdi_ptvisu (dcid: INTEGER;px, py: INTEGER): BOOLEAN is external "C" end

	win_gdi_rectvisu (dcid: INTEGER;rx1, ry1, rx2, ry2: INTEGER): BOOLEAN is external "C" end

	win_gdi_capint (dcid: INTEGER; cap: INTEGER): INTEGER is external "C" end

	win_gdi_capbool (dcid: INTEGER; cap, field: INTEGER): BOOLEAN is external "C" end

feature -- System

	win_ver_above (major, minor: INTEGER): BOOLEAN is external "C" end

feature -- Timer

	win_timer_start (winid: INTEGER; id, delay: INTEGER) is external "C" end

	win_timer_stop (winid: INTEGER; id: INTEGER) is external "C" end

feature -- Registry

	win_reg_open (base: INTEGER; name_str: POINTER): INTEGER is external "C" end

	win_reg_create (base: INTEGER; name_str: POINTER): INTEGER is external "C" end

	win_reg_close (keyid: INTEGER) is external "C" end

	win_reg_type (keyid: INTEGER; label_str: POINTER): INTEGER is external "C" end

	win_reg_delete (keyid: INTEGER; subkey_str: POINTER): BOOLEAN is external "C" end

	win_reg_values (keyid: INTEGER; index: INTEGER): BOOLEAN is external "C" end

	win_reg_subkeys (keyid: INTEGER; index: INTEGER): BOOLEAN is external "C" end

	win_reg_getint (keyid: INTEGER; label_str: POINTER): INTEGER is external "C" end

	win_reg_getstring (keyid: INTEGER; label_str: POINTER) is external "C" end

	win_reg_putint (keyid: INTEGER; label_str: POINTER; value: INTEGER) is external "C" end

	win_reg_putstring (keyid: INTEGER; label_str: POINTER; value_str: POINTER) is external "C" end

feature -- mmap file

	win_mmap_file (file_handle: INTEGER): POINTER is external "C" end

	win_mmap_free (memory: POINTER) is external "C" end

feature -- Synchronisation

	win_synch_wait (obj_handle: INTEGER; delay_ms: INTEGER): BOOLEAN is external "C" end

	win_synch_mwait (obj_handles: POINTER; obj_count: INTEGER; delay: INTEGER): INTEGER is external "C" end

	win_event_create (name_str: POINTER; man_reset: BOOLEAN): INTEGER is external "C" end

	win_event_open (name_str: POINTER): INTEGER is external "C" end

	win_event_set (evt_handle: INTEGER) is external "C" end

	win_event_reset (evt_handle: INTEGER) is external "C" end

	win_mutex_create (name_str: POINTER): INTEGER is external "C" end

	win_mutex_open (name_str: POINTER): INTEGER is external "C" end

	win_mutex_release (mtx_handle: INTEGER) is external "C" end

	win_sema_create (name_str: POINTER;max_count: INTEGER): INTEGER is external "C" end

	win_sema_open (name_str: POINTER): INTEGER is external "C" end

	win_sema_release (handle: INTEGER) is external "C" end

feature -- File services

	win_file_asread (fileid: INTEGER; byteptr: POINTER; byteptr_size: INTEGER; overptr: POINTER): BOOLEAN is external "C" end

	win_file_aswrite (fileid: INTEGER; byteptr: POINTER; byteptr_size: INTEGER; overptr: POINTER): BOOLEAN is external "C" end

	win_file_asok (fileid: INTEGER; overptr: POINTER): INTEGER is external "C" end

	win_file_read (fileid: INTEGER; byteptr: POINTER; byteptr_size: INTEGER): INTEGER is external "C" end

	win_file_write (fileid: INTEGER; byteptr: POINTER; byteptr_size: INTEGER): INTEGER is external "C" end

	win_file_writebyte (fileid: INTEGER; byte: INTEGER): BOOLEAN is external "C" end

	win_file_close (fileid: INTEGER) is external "C" end

	win_file_open (filename_str: POINTER; createm: INTEGER; read, write, sharer, sharew, randomacc: BOOLEAN): INTEGER is external "C" end

	win_file_ptr (fileid: INTEGER): INTEGER is external "C" end

	win_file_setptr (fileid: INTEGER; index: INTEGER): BOOLEAN is external "C" end

	win_file_size (fileid: INTEGER): INTEGER is external "C" end
	
	win_file_dspace (drive_str: POINTER): INTEGER is external "C" end

	win_file_attrib (fname_str: POINTER; flag: INTEGER): BOOLEAN is external "C" end

	win_file_mkdir (pname_str: POINTER): BOOLEAN is external "C" end

	win_file_rmdir (pname_str: POINTER): BOOLEAN is external "C" end

	win_file_curdir is external "C" end 

	win_file_tmpdir is external "C" end

	win_file_setcurdir (newdir_str: POINTER): BOOLEAN is external "C" end

	win_file_ffirst (pattern_str: POINTER) is external "C" end

	win_file_fnext (handle: INTEGER) is external "C" end

	win_file_fclose (handle: INTEGER) is external "C" end

	win_file_exists (fname_str: POINTER): BOOLEAN is external "C" end

	win_file_remove (fname_str: POINTER): BOOLEAN is external "C" end

	win_file_copy (orig_str, dest_str: POINTER): BOOLEAN is external "C" end

	win_file_move (orig_str, dest_str: POINTER): BOOLEAN is external "C" end

	win_file_prop (fname_str: POINTER; finddata_ptr: POINTER): BOOLEAN is external "C" end

	win_file_convtime (filetime_ptr, systime_ptr: POINTER) is external "C" end

	win_last_error: INTEGER is external "C" end

feature -- Sockets

	win_tcp_init is external "C" end

	win_tcp_finish is external "C" end

	win_tcp_connect (ip1, ip2, ip3, ip4: INTEGER; protocol: INTEGER): INTEGER is external "C" end

	win_tcp_byname (dns_name: POINTER): BOOLEAN is external "C" end

feature -- System services

	win_time (zone, id: INTEGER): INTEGER is external "C" end

	win_timezone: INTEGER is external "C" end

	win_tick: INTEGER is external "C" end
	
	win_execute (str: POINTER): BOOLEAN is external "C" end

	win_getpid: INTEGER is external "C" end

	win_sleep (ms: INTEGER) is external "C" end

	win_debugout (out_str: POINTER) is external "C" end

	win_beep (beeptype: INTEGER) is external "C" end

	win_rsrcstring (id: INTEGER) is external "C" end

	win_mci_send (cmd_str: POINTER): INTEGER is external "C" end

	win_mci_error (code: INTEGER) is external "C" end

invariant
	enough_bits: -- PLATFORM.Integer_bits >= 32

end -- class WIN_RUNTIME
