-- Pylon: a foundation library; version 0.82(beta); 1998-01-04

-- 
-- Copyright (c) 1997 Franck Arnaud, England, E.U.
-- 
-- Permission is hereby granted, without written agreement and without
-- licence or royalty fees, to use, copy, modify, or distribute this source
-- code or binary programs using it or part of it, subject to the following
-- condition:
-- 
-- The above copyright notice and the following disclaimer must appear in
-- all copies or substantial portions of this source code. (Binary programs
-- derived from the source code can be distributed without restriction or
-- mention of the origin of the source.)
-- 
-- THIS SOFTWARE IS PROVIDED BY FRANCK ARNAUD AND THE AUTHORS ``AS IS'' AND
-- ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
-- IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
-- ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHORS BE LIABLE TO ANY PARTY FOR 
-- ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
-- DAMAGES ARISING IN ANY WAY OUT OF THE USE OF THIS SOURCE CODE.
-- 

indexing
	cluster: Pylon, Container
	description: "Catalog (several items associated with a key)"
	interface: client

class P_CATALOG[G,K -> HASHABLE]

inherit
	P_TABLE[G,K]
		end

creation
	make

feature {P_CATALOG}
	
	container : P_DICTIONARY[P_LINKED_LIST[G],K]
			
feature -- Creation

	make, reset is
			-- Initialise.
		local
			element: P_LINKED_LIST[G]
		do
			!!container.make
			!!element.make --? SmallEiffel type coverage
		end

feature -- from ANY
	
	copy(other : like Current) is
			-- Copy from other catalog.
		do
			container.copy(other.container)
		end

	is_equal(other : like Current) : BOOLEAN is
			-- Equality.
		do
			Result := container.is_equal(other.container)
		end

feature -- from P_CONTAINER

	count : INTEGER is
			-- Number of keys in container.
		do
			Result := container.count
		end
	
	is_writable : BOOLEAN is true
			-- Catalog is always writable.
	
	item : G is
			-- Current item.
		local
			it : P_ITERATOR[G] 
		do
			it := item_list.iterator
			it.first 
			Result := it.item
			it.stop 
		end

feature -- Catalog

	item_list : P_LIST[G] is
			-- List of items associated with a single key.
		require
			not_empty: not is_empty
		do
			Result := container.item
		ensure
			is_shallow_copy: -- list of real items in the list.
		end

feature -- from P_TABLE

	key_search(x : K) is
			-- Search by key, compared using is_equal.
		do
			container.key_search(x)
			found := container.found
		end

	found : BOOLEAN;
			-- Result of search.

	found_count : INTEGER is
			-- Number of items found corresponding to key.
		do
			Result := item_list.count
		end

	key : K is
			-- Current key.
		do
			Result := container.key
		end

feature
	
	is_key_writable(k : K) : BOOLEAN is
			-- Is it possible to add item with corresponding key?
			-- (always true in catalog)
		do
			Result := true
		ensure then
			catalog: Result = true
		end

	add(x : G; k : K) is
			-- Add item with correspoding key.
		local
			container_item : P_LINKED_LIST[G]
		do
			container.key_search(k)
			if container.found then
				container_item := container.item
			else
				!!container_item.make
				container.add(container_item, k)
			end
			container_item.add(x)
		end

	replace(x : G) is
			-- Replace the first item in item list.
		local
			it : P_ITERATOR[G]
		do
			it := item_list.iterator
			it.first
			it.synchronize_container
			it.stop
			item_list.replace(x)
		end

	remove is
			-- Remove current key and associated item(s).
		do
			container.remove
		end

feature
	
	to_key_list : P_LIST[K] is
			-- List of keys in the container.
		do
			Result := container.to_key_list
		end

	to_item_list : P_LIST[G] is
			-- List of items in the containers (possibly more than keys).
		local
			it : P_ITERATOR[P_LINKED_LIST[G]]
			rlist : P_LINKED_LIST[G]
		do
			!!rlist.make
			Result := rlist
			
			from
				it := container.to_item_list.iterator
				it.first
			variant
				it.forward_variant
			until
				it.outside
			loop
				Result.append(it.item)
				it.forth
			end
		end

end
