-- Pylon: a foundation library; version 0.82(beta); 1998-01-04

-- 
-- Copyright (c) 1997 Franck Arnaud, England, E.U.
-- 
-- Permission is hereby granted, without written agreement and without
-- licence or royalty fees, to use, copy, modify, or distribute this source
-- code or binary programs using it or part of it, subject to the following
-- condition:
-- 
-- The above copyright notice and the following disclaimer must appear in
-- all copies or substantial portions of this source code. (Binary programs
-- derived from the source code can be distributed without restriction or
-- mention of the origin of the source.)
-- 
-- THIS SOFTWARE IS PROVIDED BY FRANCK ARNAUD AND THE AUTHORS ``AS IS'' AND
-- ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
-- IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
-- ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHORS BE LIABLE TO ANY PARTY FOR 
-- ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
-- DAMAGES ARISING IN ANY WAY OUT OF THE USE OF THIS SOURCE CODE.
-- 

indexing
	cluster: Pylon, Time
	description: "24 hours time"
	interface: client

class P_TIME

inherit
	COMPARABLE
		redefine
			out, is_equal
		end
	HASHABLE
		undefine
			out, is_equal
		end
	P_TEXT_OBJECT
		undefine
			out, is_equal
		end

feature -- Comparison

	infix "<" (other : like Current) : BOOLEAN is
		do
			Result := hour < other.hour 
				or else hour = other.hour and (minute < other.minute
					or else (hour = other.minute and second < other.second))

		end

	is_equal(other : like Current) : BOOLEAN is
		do
			Result := hour = other.hour and minute = other.minute and second = other.second
		end

feature -- Hashable

	hash_code : INTEGER is
		do
			Result := 3600 * hour + 60 * minute + second
		end

feature {PUBLIC_NONE} -- Text object

	object_to_string(tm : P_TEXT_OBJECT_MANAGER) is
		do
			tm.put_integer(hour)
			tm.put_integer(minute)
			tm.put_integer(second)
		end

	string_to_object(tm : P_TEXT_OBJECT_MANAGER) is
		do
			tm.get_integer
			hour := tm.last_integer
			tm.get_integer
			minute := tm.last_integer
			tm.get_integer
			second := tm.last_integer

			if not tm.in_success then
				hour := 0 minute := 0 second := 0
			end
		end

feature -- Status
	
	hour : INTEGER
			-- Hour (0-23)

	minute : INTEGER
			-- Minute (0-59)

	second : INTEGER
			-- Second (0-59)

	set_hour(hh : INTEGER) is
			-- Set hour.
		require
			valid: hh >= 0 and hh < 24
		do
			hour := hh
		ensure
			done: hh = hour
		end

	set_minute(mm : INTEGER) is
			-- Set minute.
		require
			valid: mm >= 0 and mm < 60
		do
			minute := mm
		ensure
			done: mm = minute
		end

	set_second(ss : INTEGER) is
			-- Set second.
		require
			valid: ss >= 0 and ss < 60
		do
			second := ss
		ensure
			done: ss = second
		end

	set(hh, mm, ss : INTEGER) is
			-- Set time.
		require
			valid_hour: hh >= 0 and hh < 24
			valid_minute: mm >= 0 and mm < 60
			valid_second: ss >= 0 and ss < 60
		do
			set_hour(hh)
			set_minute(mm)
			set_second(ss)
		ensure
			done: hour = hh and minute = mm and second = ss
		end

feature -- Status

	hour_12 : INTEGER is
			-- Hour in am/pm format.
		do
			if hour <= 12 and hour >= 1 then
				Result := hour
			elseif hour = 0 then
				Result := 12
			else -- > 12
				Result := hour - 12
			end
		ensure
			done: Result >= 1 and Result <= 12
		end

	is_am : BOOLEAN is
			-- Is morning?
		do
			Result := hour <= 12 and hour >= 1
		end

	is_pm : BOOLEAN is
			-- Is evening?
		do
			Result := not is_am
		end
	
feature -- Output

	out : STRING is
			-- Default string representation: ISO format.
		do
			Result := to_iso
		end
	
	to_iso : STRING is
			-- Output time in ISO format.
			-- eg: "233000"
		local
			fmt : P_FORMAT
		do
			!!fmt
			!!Result.make(0)
			Result.copy(fmt.int_to_str_min(hour,2))
			Result.append_string(fmt.int_to_str_min(minute,2))
			Result.append_string(fmt.int_to_str_min(second,2))
		ensure
			is_copy: Result /= Void
		end

	to_rfc : STRING is
			-- Output time in RFC format.
			-- eg: "23:30:00"
		local
			fmt : P_FORMAT
		do
			!!fmt
			!!Result.make(0)
			Result.copy(fmt.int_to_str_min(hour,2))
			Result.append_string(Separator)
			Result.append_string(fmt.int_to_str_min(minute,2))
			Result.append_string(Separator)
			Result.append_string(fmt.int_to_str_min(second,2))
		ensure
			is_copy: Result /= Void
		end

feature {NONE} -- Constant(s)
	
	Separator : STRING is ":"

invariant
	hour: hour >= 0 and hour < 24
	minute: minute >= 0 and minute < 60
	second: second >= 0 and second < 60
	ampm: is_am xor is_pm

end
