/*
 * Fenestra Win32 runtime system: Header file
 * -----------------------------------------------------------------
 * Copyright (c) 1998 Franck Arnaud
 *
 * Defines eiffel types, includes eiffel headers, type conversion 
 * macros and some API macros.
 *
 * This file depends on the Eiffel compiler's own runtime system.
 */

/* General includes (Note: this runtime does not use the C library) */

#include <windows.h>
#include <winsock.h>

/* Eiffel includes */
   
#if defined(EIFFELS)
#define EANSI
#include <eiffel.h>
#include "eansi.h" /* NOTIFIER + external prototype */

#elif defined(TOWER)
#include <tecil.h>
#include "notifier.h"

#elif defined(CECIL)
#include <cecil.h>

#if defined(ISECECIL) /* extra header for ISE Eiffel */
#include <hector.h>
#endif

#if defined(ISSCECIL) /* extra header for ISS Base */
#include <eif_hector.h>
#endif

#ifndef EIF_NO_TYPE
#define EIF_NO_TYPE ((EIF_TYPE_ID) NULL)
#endif
#ifndef EIF_TRUE
#define EIF_TRUE 1
#endif
#ifndef EIF_FALSE
#define EIF_FALSE 0 
#endif

#elif defined(SMALLEIFFEL)
#include "fenestra_se.h"

#endif

/* DLL spec */

#ifdef GENERICDLL
#define FRTAPI __declspec(dllexport)
#define ANY_EIFFEL
#else
#define FRTAPI /* nothing */
#endif

/*
	Eiffel types
	----------------------------------------------------------------
	EPOINTER	Eiffel 32bit pointer
	ECHAR		Eiffel character
	ESTRING		Eiffel string
	EBOOL		Eiffel boolean (value = ETRUE or EFALSE)
	EINT		Eiffel (32bit) integer

	Eiffel type		Win32 type		Eiffel > Win32	Win32 > Eiffel	
	----------------------------------------------------------------
	E_HWND			HWND			E2W_HWND		W2E_HWND
	E_HMENU			HMENU			E2W_HMENU		W2E_HMENU
	E_HGDI			HGDIOBJ			E2W_HGDI		W2E_HGDI
	EINT			DWORD			E2W_INT			W2E_INT
	EBOOL			BOOL			E2W_BOOL		W2E_BOOL
	ESTRING			LPTSTR			E2W_STRING		-
	EBYTEARRAY		BYTE *			E2W_BYTEARRAY	-
	EINTARRAY		DWORD *			E2W_INTARRAY	-
	ELOGFONT		LOGFONT *		E2W_LOGFONT		-
	ELOGPALETTE		LOGPALETTE *	E2W_LOGPALETTE	-
*/

/* -----------------------------------------------------------------------
 * Runtime symbols 
 */

/* Exception */

#define EXCP_OTHER  2001   /* code for runtime API failure */


/* Runtime exception enabled */

#ifdef FRT_NOASSERT /* no debug */

#define WINAPI_EXCEPTION(a) /* expands to nothing */

#define ASSERT_EXCEPTION(a,b)
#define FALSE_EXCEPTION(a,b)
#define NULL_EXCEPTION(a,b)
#define ZERO_EXCEPTION(a,b)
#define NOTZERO_EXCEPTION(a,b)

#else /* do debug */ 

#define WINAPI_EXCEPTION(a) fenestra_exception(TEXT(a))
#define ASSERT_EXCEPTION(a,b) { if(!(a)) fenestra_exception(TEXT(b)); }
#define FALSE_EXCEPTION(a,b) { if(a == FALSE) fenestra_exception(TEXT(b)); }
#define NULL_EXCEPTION(a,b) { if(a == NULL) fenestra_exception(TEXT(b)); }
#define ZERO_EXCEPTION(a,b) { if(a == 0) fenestra_exception(TEXT(b)); }
#define NOTZERO_EXCEPTION(a,b) { if(a) fenestra_exception(TEXT(b)); }

#endif /* assertions */

/* Win32 constant string for some of our objects */

#define PROP_CALLBACK 		TEXT("fenestraOriginalCallback")
#define CLASS_OVERLAPPED	TEXT("fenestraMain")
#define	CLASS_CHILDWINDOW	TEXT("fenestraChild")
#define CLASS_DIALOG		TEXT("fenestraDialog")
#define CLASS_FREECURSOR	TEXT("fenestraChildC")

#define UNIQUEID			1000				/* Not unique, for controls, IDs not used */
#define BAD_COLOR 			0x7F000000 			/* win_std_color, win_std_font, win_gdi_getpixel */

#define OPENFILE_SEPARATOR	'|'	/* file dialog constants */
#define OPENFILE_FILESIZE	2000

#define RSRC_STRING			500
#define MCI_ERROR_STRING	130	/* documented max: 128 */

/* pseudo events */

#define FILO_SCROLL 1280	/* 0x500 (in range 0x400 [WM_USER] - 0x7FFF) */
#define FILO_COMMAND 1281	/* 0x501 */

/* windows API shortcuts */

#define PROGHINST			GetModuleHandle(NULL) /* Our hinstance */

/* Prototypes for callbacks from win32_notifier.c */

LRESULT CALLBACK UniversalWindowCallback(HWND,UINT,WPARAM,LPARAM);
LRESULT CALLBACK ControlSubclass(HWND,UINT,WPARAM,LPARAM);
int CALLBACK EnumFontFamCallback(ENUMLOGFONT *lpelf, NEWTEXTMETRIC FAR *lpntm, int FontType, LPARAM lParam);

/* ----------------------------------------------------------------------- 
 * Eiffel<>Win32 type conversion
 */

/* Win32 handles */

/* NOTE:
 * All handle types are 32bit values under Win32 and are here converted 
 * to Eiffel's INTEGER. This implementation requires Eiffel's INTEGER to 
 * be (at least) 32 bit wide.
 */

/* Eiffel basic types */

#if defined(EIFFELS) /* --------------- Basic types for Eiffel/S */

#define EINT	   			INTEGER				/* INTEGER / int */
#define E2W_INT(a)			((DWORD) a)			/* Convert Eiffel's INTEGER to C's int */
#define	W2E_INT(a)			((INTEGER) a)		/* Convert C's int to Eiffel's INTEGER */

#define EPOINTER			POINTER				/* Eiffel pointer type */
#define ECHAR				CHARACTER			/* Eiffel character type */

#define EBOOL				BOOLEAN				/* Eiffel boolean type */
#define ETRUE				true				/* Eiffel boolean 'true' value */
#define EFALSE				false				/* Eiffel boolean 'false' value */

#define W2E_CHAR(a) 		((CHARACTER)a)

#elif defined(TOWER) /* --------------- Basic types for TowerEiffel */

#define EINT				EIF_INTEGER			/* INTEGER / int */
#define E2W_INT(a)			((DWORD) a)			/* Convert Eiffel's INTEGER to C's int */
#define	W2E_INT(a)			((EIF_INTEGER) a)	/* Convert C's int to Eiffel's INTEGER */

#define EPOINTER			EIF_POINTER			/* Eiffel pointer type */
#define ECHAR				EIF_CHARACTER		/* Eiffel character type */

#define EBOOL				EIF_BOOLEAN			/* Eiffel boolean type */
#define ETRUE				EIF_TRUE			/* Eiffel boolean 'true' value */
#define EFALSE				EIF_FALSE			/* Eiffel boolean 'false' value */

#define W2E_CHAR(a) 		((EIF_CHARACTER)a) 	/* Eiffel character type */

#elif defined(CECIL) /* --------------- Basic CECIL types */

#define EINT				EIF_INTEGER			/* INTEGER / int */
#define E2W_INT(a)			((DWORD) a)			/* Convert Eiffel's INTEGER to C's int */
#define	W2E_INT(a)			((EIF_INTEGER) a)	/* Convert C's int to Eiffel's INTEGER */

#define EPOINTER			EIF_POINTER			/* Eiffel pointer type */
#define ECHAR				EIF_CHARACTER		/* Eiffel character type */

#define EBOOL				EIF_BOOLEAN			/* Eiffel boolean type */
#define ETRUE				EIF_TRUE			/* Eiffel boolean 'true' value */
#define EFALSE				EIF_FALSE			/* Eiffel boolean 'false' value */

#define W2E_CHAR(a) 		((EIF_CHARACTER)a) 	/* Eiffel character type */

#elif defined(SMALLEIFFEL) /* --------- SmallEiffel types */

#define EINT				int					/* INTEGER / int */
#define E2W_INT(a)			((DWORD) a)			/* Convert Eiffel's INTEGER to C's int */
#define W2E_INT(a)			((int) a)			/* Convert C's int to Eiffel's INTEGER */

#define EPOINTER			void *				/* Eiffel pointer type */
#define ECHAR				char				/* Eiffel character type */

#define EBOOL				int					/* Eiffel boolean type */
#define ETRUE				1					/* Eiffel boolean 'true' value */
#define EFALSE				0					/* Eiffel boolean 'false' value */

#define W2E_CHAR(a) 		((char)a) 			/* Eiffel character type */

#elif defined(ANY_EIFFEL) /* ---------- Generic callback/DLL interface */

#define EINT				int					/* INTEGER / int */
#define E2W_INT(a)			((DWORD) a)			/* Convert Eiffel's INTEGER to C's int */
#define W2E_INT(a)			((int) a)			/* Convert C's int to Eiffel's INTEGER */

#define EPOINTER			void *				/* Eiffel pointer type */
#define ECHAR				char				/* Eiffel character type */

#define EBOOL				int					/* Eiffel boolean type */
#define ETRUE				1					/* Eiffel boolean 'true' value */
#define EFALSE				0					/* Eiffel boolean 'false' value */

#define W2E_CHAR(a) 		((char)a) 			/* Eiffel character type */

#endif /* compiler symbols */

#ifdef UNICODE
#error "write unicode support"
#else
#define W2C_STRING(a)		(a) 				/* LPTSTR to char*, don't do unicode */
#endif

#define E2W_BOOL(a)			( (a==ETRUE) ? TRUE : FALSE )	/* Convert Eiffel to Win32 */
#define W2E_BOOL(a)			((EBOOL)( (a==TRUE) ? ETRUE : EFALSE ))	/* Convert Win32 to Eiffel */

#define E2W_POINTER(a)		((void *) a)		/* Convert POINTER */
#define W2E_POINTER(a)		((EPOINTER) a)		/* Convert to POINTER */

#define ESTRING				EPOINTER			/* Eiffel string */
#define E2W_STRING(a) 		((LPTSTR) a)		/* STRING > LPTSTR conversion */ 


#define E_HWND				EINT				/* Window handle type from Eiffel */
#define E2W_HWND(a)			((HWND) a)			/* convert Eiffel to Win32 */
#define W2E_HWND(a) 		((EINT) a)			/* convert Win32 to eiffel */

#define E_HMENU				EINT				/* Menu handle type from Eiffel */
#define E2W_HMENU(a)		((HMENU) a)			/* Convert Eiffel to Win32 */
#define W2E_HMENU			W2E_HWND			/* Convert Win32 to Eiffel */

#define E_HGDI				EINT				/* GDI object handle from Eiffel */
#define E2W_HGDI(a)			((HGDIOBJ) a)		/* Convert Eiffel to Win32 */
#define W2E_HGDI			W2E_HWND			/* Convert Win32 to Eiffel */

#define	E_HDLE				EINT				/* General system handle */
#define	E2W_HDLE(a)			((HANDLE) a)		/* Convert Eiffel to Win32 */
#define W2E_HDLE			W2E_HWND			/* Convert Win32 to Eiffel */


/* the following types are handled with F_MEMORY */

#define EINTARRAY 			EPOINTER			/* INTEGER * in Eiffel */
#define E2W_INTARRAY(a)		((DWORD *) E2W_POINTER(a))

#define EBYTEARRAY 			EPOINTER			/* Eiffel ARRAY[CHARACTER] */
#define E2W_BYTEARRAY(a)	((BYTE *) E2W_POINTER(a))

#define ELOGFONT			EPOINTER			/* LOGFONT in Eiffel */
#define E2W_LOGFONT(a)		((LOGFONT *) E2W_POINTER(a))

#define ELOGPALETTE			EPOINTER			/* LOGPALETTE in Eiffel */
#define E2W_LOGPALETTE(a)	((LOGPALETTE *) E2W_POINTER(a))

#define EFINDDATA			EPOINTER			/* WIN32_FIND_DATA in Eiffel */
#define E2W_FINDDATA(a)		((WIN32_FIND_DATA *) E2W_POINTER(a))

#define EFILETIME			EPOINTER			/* FILETIME in Eiffel */
#define E2W_FILETIME(a)		((FILETIME *) E2W_POINTER(a))

#define ESYSTIME			EPOINTER			/* SYSTEMTIME in Eiffel */
#define E2W_SYSTIME(a)		((SYSTEMTIME *) E2W_POINTER(a))

/* --------------------------------------------------------------------
 * Prototypes for runtime utility functions 
 */

LPVOID fenestra_malloc(DWORD size);
void fenestra_free(LPVOID ptr);

void fenestra_exception(LPTSTR command);

void fenestra_memset(void *,int);
void fenestra_memcpy(void *,void *,int);

void fenestra_void_string();
void fenestra_pass_string(LPTSTR str);
void fenestra_copy_string(LPTSTR str);

void fenestra_set_bool(EBOOL );
void fenestra_set_int(EINT );
void fenestra_set_int2(EINT, EINT);
void fenestra_set_int4(EINT, EINT, EINT, EINT);

/**/