-- Fenestra; version 0.80(beta); 1998-09-27

-- 
-- Copyright (c) 1998 Franck Arnaud, http://www.altsoft.demon.co.uk/
-- 
-- This package is distributed under the Eiffel Forum Freeware License, 
-- version 1 (see file forum.txt for full text). The main points are:
-- 
-- "Permission is hereby granted, without written agreement and without 
--  license or royalty fees, to use, copy, modify and/or distribute this 
--  package, provided that: copyright notices are retained unchanged [...]
-- 
--  Permission is hereby also granted [...] to distribute binary programs 
--  which depend on this package, provided that: if the binary program 
--  depends on a modified version of this package, you must publicly release 
--  the modified version of this package [...].
-- 
--  THIS PACKAGE IS PROVIDED "AS IS" AND WITHOUT WARRANTY [...]"
-- 

indexing
	cluster: Fenestra, Bubble
	description: "Arrow linking two class objects"
	interface: client

class ARROW

inherit
	BUBBLE_OBJECT
		redefine
			is_dependent
		end

creation
	make, make_store

feature {NONE} -- Creation

	make (ctx: DRAWING; sub, super: BUBBLE) is
			-- Create new object from scratch.
		require
			ok_context: ctx /= Void
			ok_class: sub /= Void and super /= Void
			not_same: sub /= super
		do
			context := ctx
			subclass := sub
			superclass := super
		end

	make_store (ctx: DRAWING) is
			-- Create object to be retrieved from store.
		require
			ok: ctx /= Void
		do
			context := ctx
		end

feature -- Properties

	properties (dw: BUBBLE_DRAW): MODELESS_DIALOG is
		local
			op: ARROW_PROPERTIES
		do
			!!op.make (dw.overlapped_parent)
			op.set_object (Current)
			op.set_draw (dw)

			Result := op
		end

feature -- General

	is_equal (other: like Current): BOOLEAN is
		do
			Result := subclass = other.subclass and superclass = other.superclass
		end

	copy (other: like Current) is
		do
			check not_possible: false end
		end

feature -- ID

	type: STRING is
		do
			Result := "arrow"
		end

	is_valid: BOOLEAN is
		do
			Result := (subclass_id = Void) and (superclass_id = Void)
				and (subclass /= Void) and (superclass /= Void)
		end

	is_dependent (other: BUBBLE_OBJECT): BOOLEAN is
		do
			Result := (other = subclass) or (other = superclass)
		end

feature {NONE}
	
	object_to_string (tm: P_TEXT_OBJECT_MANAGER) is
		do
			tm.put_integer (1)
			tm.put_boolean (is_client)
			tm.put_string (subclass.reference_id)
			tm.put_string (superclass.reference_id)
		end

	string_to_object (tm: P_TEXT_OBJECT_MANAGER) is
		do
			tm.get_integer
			tm.get_boolean
			is_client := tm.last_boolean
			tm.get_string
			subclass_id := tm.last_string
			tm.get_string
			superclass_id := tm.last_string
		end

feature
	
	resolve (bubbles: P_SEARCHABLE[BUBBLE,STRING]) is
			-- Resolve retrieved object.
		require
			ok: bubbles /= Void
		do
			subclass := Void
			superclass := Void

			bubbles.search (subclass_id)
			if bubbles.found then
				subclass := bubbles.item
				bubbles.search (superclass_id)
				if bubbles.found then
					superclass := bubbles.item
					subclass_id := Void
					superclass_id := Void
				end
			end
		ensure
			done: -- is_valid set
		end
		
feature {NONE} -- Attributes

	context: DRAWING
	subclass_id, superclass_id: STRING

feature -- More attributes
	
	subclass, superclass: BUBBLE
	is_client: BOOLEAN

feature -- Status

	set_client (iscli: BOOLEAN) is
			-- Change status.
		do
			is_client := iscli
		ensure
			done: is_client = iscli
		end

feature {NONE}
	
	Handle_size: INTEGER is 8

	handle (point: POINT): RECTANGLE is
		do
			!!Result.make
			Result.set_width (Handle_size)
			Result.set_height (Handle_size)
			Result.translate_center (point)
		end
	
	start_point: POINT is
		do
			Result := subclass.edge (superclass.position.center)
		end

	last_point: POINT is
		do
			Result := superclass.edge (subclass.position.center)
		end

feature {NONE} -- Arrow

	Sine_3pi4: DOUBLE is 0.70710578
	Arrow_size: DOUBLE is 8.0
	Selection_distance: DOUBLE is 4.0

	arrow_head: DRAWABLE_POLYGON is
			-- Prepare arrow head.
		local
			xo, yo, xd, yd, x, y, d: DOUBLE
			pt: POINT
			m: P_MATH
			sys: P_SYSTEM
		do
			!!m
			!!sys


			pt := start_point
			xo := pt.x yo := pt.y
			
			pt := last_point
			xd := pt.x yd := pt.y
			

			x := xd-xo
			y := yd-yo

			d := m.square_root (x^2+y^2)
			if d > Arrow_size then -- real arrow then
				
				!!Result.make
				Result.add_point (pt) -- last point

				!!pt.make
				pt.set_x (sys.floor (xd- (Arrow_size* (x+y)*Sine_3pi4)/d))
				pt.set_y (sys.floor (yd+ (Arrow_size* (x-y)*Sine_3pi4)/d))

				Result.add_point (pt)

				pt.set_x (sys.floor (xd+ (Arrow_size* (y-x)*Sine_3pi4)/d))
				pt.set_y (sys.floor (yd- (Arrow_size* (x+y)*Sine_3pi4)/d)) 

				Result.add_point (pt)
			end
		ensure
			may_be_void: -- if no meaningful arrow
		end

feature
	
	is_inside (pt: POINT): BOOLEAN is
			-- Near line? (selection)
		local
			st, fi: POINT
			arrow_x, arrow_y, x0, y0: DOUBLE
			distance, length, xpos: DOUBLE
			m: P_MATH
		do
			!!m
			st := start_point
			fi := last_point
			
			if not fi.is_equal (st) then
				arrow_x := fi.x - st.x
				arrow_y := fi.y - st.y
				
				x0 := pt.x - st.x
				y0 := pt.y - st.y

				-- length of arrow
				length := m.square_root (arrow_x^2 + arrow_y^2)

				-- distance from arrow lineto pt
				distance := ( (arrow_x*y0 - arrow_y*x0)/length).abs

				-- is the point inside the arrow segment?
				xpos := ( (arrow_x*x0 + arrow_y*y0)/length)

				Result := (xpos >= 0) and (xpos <= length) and (distance < Selection_distance)
			end
		end

feature
	
	bounding_box: RECTANGLE is
			-- Bounding box including handles.
		do
			Result := handle (start_point)
			Result.add (handle (last_point))
		end
	
	anchor (pt: POINT): BUBBLE_SHADOW is
		do
			if handle (start_point).has_point (pt) then
				!ARROW_MOVE!Result.make_start (last_point)
			elseif handle (last_point).has_point (pt) then
				!ARROW_MOVE!Result.make_final (start_point)
			end

			-- init anchor
			if Result /= Void then
				Result.set_anchor (pt)
			end
		end

	resize (bs: BUBBLE_SHADOW) is
		local
			am: ARROW_MOVE
			bub: BUBBLE
		do
			am ?= bs
			check ok: am /= Void end
			
			bub ?= context.find_object (am.final)
			if bub /= Void then
				if am.is_start then
					subclass := bub
				else
					superclass := bub
				end
			end
		end

feature -- Drawing

	draw_selection (dev: GRAPHIC_DEVICE) is
		local
			dr: DRAWABLE_RECTANGLE
			gm: GRAPHIC_MODE
			gr: GRAPHICS_ROUTINES
			clr: RGB_COLOR
		do
			!!gr

			-- xor mode
			!!gm.make
			gm.inverse
			dev.set_mode (gm)

			-- frame
			!!clr.black
			dev.set_pen (gr.color_pen (clr))
			dev.set_brush (gr.color_brush (clr))
			
			dev.set_brush (gr.void_brush)
			
			!!dr.make
			dr.set_position (handle (start_point))
			dev.draw (dr)

			dr.set_position (handle (last_point))
			dev.draw (dr)

			gm.pen
			dev.set_mode (gm)
		end

	draw (dev: GRAPHIC_DEVICE) is
			-- Draw arrow.
		local
			dab: DRAWABLE
			line: DRAWABLE_LINE
			fore_clr: RGB_COLOR
			pen: GEOMETRIC_PEN
			brush: COLOR_BRUSH
		do
			!!fore_clr.black

			-- Setup device
			!!pen.make
			pen.set_color (fore_clr)
			if is_client then
				pen.set_line_dash
			end
			dev.set_pen (pen)

			!!brush.make
			brush.set_color (fore_clr)
			dev.set_brush (brush)

			-- Actual line
			!!line.make
			line.set_start (start_point)
			line.set_final (last_point)
			dev.draw (line)
			
			-- Draw arrow head.
			dab := arrow_head
			if dab /= Void then -- actually has one
				dev.draw (dab)
			end
		end

	draw_outline (dev: GRAPHIC_DEVICE; any_shadow: BUBBLE_SHADOW) is
			-- Draw arrow outline.
		local
			line: DRAWABLE_LINE
			box: ARROW_MOVE
			mode: GRAPHIC_MODE
			gr: GRAPHICS_ROUTINES
			outline_clr: RGB_COLOR
		do
			!!gr

			!!outline_clr.normal_blue

			box ?= any_shadow
			check good_box: box /= Void end

			!!mode.make

			mode.inverse
			dev.set_mode (mode)

			dev.set_pen (gr.color_pen (outline_clr))
			dev.set_brush (gr.void_brush)

			!!line.make
			line.set_start (box.start)
			line.set_final (box.final)
			dev.draw (line)

			mode.pen
			dev.set_mode (mode)
		end

end

