-- Fenestra; version 0.80(beta); 1998-09-27

-- 
-- Copyright (c) 1998 Franck Arnaud, http://www.altsoft.demon.co.uk/
-- 
-- This package is distributed under the Eiffel Forum Freeware License, 
-- version 1 (see file forum.txt for full text). The main points are:
-- 
-- "Permission is hereby granted, without written agreement and without 
--  license or royalty fees, to use, copy, modify and/or distribute this 
--  package, provided that: copyright notices are retained unchanged [...]
-- 
--  Permission is hereby also granted [...] to distribute binary programs 
--  which depend on this package, provided that: if the binary program 
--  depends on a modified version of this package, you must publicly release 
--  the modified version of this package [...].
-- 
--  THIS PACKAGE IS PROVIDED "AS IS" AND WITHOUT WARRANTY [...]"
-- 

indexing
	cluster: Fenestra, Bubble
	description: "Mouse handler for BUBBLE_DRAW (arrow mode)"
	interface: client

class ARROW_MOUSE

inherit
	MOUSE_HANDLER
		redefine
			parent, when_drag_started, 
			when_dragged, when_drag_finished
		end

creation
	make

feature -- Parent
	
	parent: BUBBLE_DRAW

feature {NONE}
	
	start_bubble: BUBBLE
	start_point, last_point: POINT

feature {NONE}
	
	logical_point: POINT is
		local
			dev: GRAPHIC_DEVICE
		do
			dev := parent.device
			if dev.is_ready then
				Result := dev.device_to_logical (point)
			else
				dev.prepare
				Result := dev.device_to_logical (point)
				dev.release
			end
		end

feature {NONE}
	
	draw_line is
			-- Draw line when user dragging.
		local
			dev: GRAPHIC_DEVICE
			mode: GRAPHIC_MODE
			line: DRAWABLE_LINE
			outline_clr: RGB_COLOR
			gr: GRAPHICS_ROUTINES
		do
			!!gr
			!!outline_clr.normal_blue

			dev := parent.device
			dev.prepare
			
			!!mode.make
			mode.inverse
			dev.set_mode (mode)
			
			dev.set_pen (gr.color_pen (outline_clr))
			dev.set_brush (gr.void_brush)

			!!line.make
			line.set_start (start_point)
			line.set_final (last_point)
			dev.draw (line)

			mode.pen
			dev.set_mode (mode)

			dev.release
		end

feature {PUBLIC_NONE}
	
	when_drag_started is
		do
			last_point := Void
			start_point := Void

			start_bubble ?= parent.drw.find_object (logical_point)
			if start_bubble /= Void then
				!!start_point.make
				start_point.copy (logical_point)
				last_point := Void
			end
		end
	
	when_dragged is
		do
			if start_bubble /= Void then
				-- previous line
				if last_point = Void then
					!!last_point.make
				else
					draw_line -- erase previous line
				end

				-- new line
				last_point.copy (logical_point)
				draw_line -- draw line
			end
		end

	when_drag_finished is
		local
			last_bubble: BUBBLE
			arrow: ARROW
		do
			if last_point /= Void then
				draw_line -- erase previous line
			end

			last_bubble ?= parent.drw.find_object (logical_point)
			if last_bubble /= Void and then last_bubble /= start_bubble then
				!!arrow.make (parent.drw, start_bubble, last_bubble)
				parent.drw.add_object (arrow)
				parent.paint_logical_rectangle (parent.drw.area_changed)
			end

			start_bubble := Void
			start_point := Void
			last_point := Void
		end

end
