-- Fenestra; version 0.80(beta); 1998-09-27

-- 
-- Copyright (c) 1998 Franck Arnaud, http://www.altsoft.demon.co.uk/
-- 
-- This package is distributed under the Eiffel Forum Freeware License, 
-- version 1 (see file forum.txt for full text). The main points are:
-- 
-- "Permission is hereby granted, without written agreement and without 
--  license or royalty fees, to use, copy, modify and/or distribute this 
--  package, provided that: copyright notices are retained unchanged [...]
-- 
--  Permission is hereby also granted [...] to distribute binary programs 
--  which depend on this package, provided that: if the binary program 
--  depends on a modified version of this package, you must publicly release 
--  the modified version of this package [...].
-- 
--  THIS PACKAGE IS PROVIDED "AS IS" AND WITHOUT WARRANTY [...]"
-- 

class BUBBLE 

inherit 
	BOX
		end

creation
	make

feature {NONE}
	
	make is
			-- Creation.
		do
			!!class_name.make (0)
			!!position.make
			position.set_width (Default_width)
			position.set_height (Default_height)
		end

	Default_width: INTEGER is 80
	Default_height: INTEGER is 40

feature
	
	properties (dw: BUBBLE_DRAW): MODELESS_DIALOG is
			-- Properties.
		local
			bp: BUBBLE_PROPERTIES
		do
			!!bp.make (dw.overlapped_parent)
			bp.set_object (Current)
			bp.set_draw (dw)

			Result := bp
		end

feature
	
	copy (other: like Current) is
		do
			!!position.make
			position.copy (other.position)

			handles := Void

			!!class_name.make (0)
			class_name.copy (other.class_name)
			deferred_class := other.deferred_class
		end

	is_equal (other: like Current): BOOLEAN is
		do
			Result := position.is_equal (other.position)
				and class_name.is_equal (other.class_name)
				and deferred_class = other.deferred_class
		end

feature -- ID

	type: STRING is
			-- Type name.
		do
			Result := "bubble"
		end

	reference_id: STRING is
			-- Reference for stored arrows.
		do
			!!Result.make (0)
			Result.append_string (class_name)
			if deferred_class then Result.append_character ('*') end
			Result.append_string (position.out)
		end

feature {NONE} -- Serialisation

	string_to_object (tm: P_TEXT_OBJECT_MANAGER) is
		local
			version: INTEGER
		do
			make

			tm.get_integer
			version := tm.last_integer
			
			tm.get_string
			class_name.copy (tm.last_string)

			tm.get_boolean
			deferred_class := tm.last_boolean
		
			tm.get_object (position)

			if version /= 1 or not tm.in_success then
				make
			end
		end

	object_to_string (tm: P_TEXT_OBJECT_MANAGER) is
		do
			tm.put_integer (1)
			tm.put_string (class_name)
			tm.put_boolean (deferred_class)
			tm.put_object (position)
		end

feature -- Attributes

	class_name: STRING
	deferred_class: BOOLEAN

feature -- Status

	set_name (cn: STRING) is
			-- Change class name.
		require
			ok: cn /= Void
		do
			class_name.copy (cn)
		ensure
			done: class_name.is_equal (cn)
		end

	set_deferred (ds: BOOLEAN) is
			-- Change deferred status.
		do
			deferred_class := ds
		end

feature {NONE}
	
	cut_name (res: P_LIST[STRING]; cn: STRING) is
			-- Cut class name on _[] boundary.
		require
			ok_list: res /= Void
			ok_name: cn /= Void
		local
			distance, cd, candidate: INTEGER
			i: INTEGER
		do
			from
				i := cn.count
				distance := cn.count + 1
			variant i until i = 0 loop
				if cn.item (i) = '_' or cn.item (i) = '[' or cn.item (i) = ']' then
					cd := (cn.count // 2 - i).abs
					if cd < distance then
						distance := cd
						candidate := i
					end
				end
				i := i - 1
			end

			if candidate > 0 and candidate < cn.count then
				res.add (cn.substring (1, candidate))
				res.add (cn.substring (candidate+1, cn.count))
			else
				res.add (cn)
			end
		end

feature -- Drawing

	draw (dev: GRAPHIC_DEVICE) is
		local
			ellipse: DRAWABLE_ELLIPSE
			dt: DRAWABLE_TEXT
			fore_clr, back_clr: RGB_COLOR
			strlist: P_LINKED_LIST[STRING]
			it: P_ITERATOR[STRING]
			extent: RECTANGLE
			pt: POINT
			gr: GRAPHICS_ROUTINES
		do
			!!gr

			!!fore_clr.black
			!!back_clr.white

			dev.set_pen (gr.color_pen (fore_clr))
			dev.set_brush (gr.color_brush (back_clr))
			
			!!ellipse.make
			ellipse.set_bounding_rectangle (position)
			dev.draw (ellipse)
			
			-- prepare text
			!!dt.make
			dt.align_center
			!!strlist.make

			-- deferred
			if deferred_class then
				strlist.add ("*")
			end

			-- draw text
			if not class_name.empty then
				dt.set_text (class_name)
				extent := dev.text_extent (dt)
				if extent.width > (position.width * 4 // 5) then
					cut_name (strlist, class_name)
				else
					strlist.add (class_name)
				end
			else
				dt.set_text ("*")
				extent := dev.text_extent (dt)
			end
			from
				it := strlist.iterator
				it.first
				pt := position.center
				pt.set_y (pt.y- (extent.height * strlist.count // 2))
			variant it.forward_variant until it.outside loop
				dt.set_position (pt)
				dt.set_text (it.item)
				dev.draw (dt)
				
				pt.set_y (pt.y+extent.height) -- next line

				it.forth
			end
		end


	draw_outline (dev: GRAPHIC_DEVICE; any_shadow: BUBBLE_SHADOW) is
		local
			ellipse: DRAWABLE_ELLIPSE
			box: BOX_SHADOW
			mode: GRAPHIC_MODE
			pen: GEOMETRIC_PEN
			gr: GRAPHICS_ROUTINES
			outline_clr: RGB_COLOR
		do
			!!gr

			!!outline_clr.normal_blue

			box ?= any_shadow
			check good_box: box /= Void end

			!!mode.make

			mode.inverse
			dev.set_mode (mode)
			
			!!pen.make
			pen.set_line_dash
			pen.set_color (outline_clr)

			dev.set_pen (pen)
			dev.set_brush (gr.void_brush)

			!!ellipse.make
			ellipse.set_bounding_rectangle (box.position)
			dev.draw (ellipse)

			mode.pen
			dev.set_mode (mode)
		end

feature {BUBBLE_OBJECT}
	
	edge (remote_point: POINT): POINT is
		require
			ok: remote_point /= Void
		local
			bubble_centre: POINT
			w, h, a, x, y: DOUBLE
			sys: P_SYSTEM
			m: P_MATH
		do
			if position.has_point (remote_point) then -- inside then centre
				Result := position.center
			else
				!!m
				bubble_centre := position.center

				w := position.width / 2
				h := position.height / 2

				if h = 0 or w = 0 then
					x := 0
					y := 0
				elseif remote_point.x = bubble_centre.x then
					x := 0
					if remote_point.y >= bubble_centre.y then
						y := h
					else
						y := - h
					end
				else
					a := (remote_point.y - bubble_centre.y) / (remote_point.x - bubble_centre.x)

					x := (w*h) / m.square_root (h^2 + (w^2 * a^2))
					if bubble_centre.x > remote_point.x then -- get sign back
						x := - x
					end
					y := a*x
				end

				!!sys

				!!Result.make
				Result.set_x (sys.floor (x)+bubble_centre.x)
				Result.set_y (sys.floor (y)+bubble_centre.y)
			end
		ensure
			inside_implies_centre: position.has_point (remote_point) implies Result.is_equal (position.center)
			always_inside: bounding_box.has_point (Result)
		end

end
