-- Fenestra; version 0.80(beta); 1998-09-27

-- 
-- Copyright (c) 1998 Franck Arnaud, http://www.altsoft.demon.co.uk/
-- 
-- This package is distributed under the Eiffel Forum Freeware License, 
-- version 1 (see file forum.txt for full text). The main points are:
-- 
-- "Permission is hereby granted, without written agreement and without 
--  license or royalty fees, to use, copy, modify and/or distribute this 
--  package, provided that: copyright notices are retained unchanged [...]
-- 
--  Permission is hereby also granted [...] to distribute binary programs 
--  which depend on this package, provided that: if the binary program 
--  depends on a modified version of this package, you must publicly release 
--  the modified version of this package [...].
-- 
--  THIS PACKAGE IS PROVIDED "AS IS" AND WITHOUT WARRANTY [...]"
-- 

indexing
	cluster: Fenestra, Bubble
	description: "Toolbar for Bubble"
	interface: client

class BUBBLE_TOOLBAR

inherit
	CHILD_WINDOW_COLOR
		redefine
			when_created, when_mouse_clicked, when_drawn
		end

creation
	make

feature {NONE}
	
	make (ow: like parent) is
			-- Creation.
		local
			background_clr: RGB_COLOR
		do
			!!background_clr.light_grey

			make_child_color (ow, background_clr)
		end

feature {NONE} -- Attribute(s)
	
	tools: P_SEQUENCE_LIST[CMD_TOOL]
	selection: INTEGER

	Icon_size: INTEGER is 32

feature
	
	add_command (cmd: CMD_TOOL) is
			-- Add command.
		do
			tools.add (cmd)
		end

	set_selection (cmd: CMD_TOOL) is
			-- Change selection.
		local
			oldsel: INTEGER
		do
			tools.equality_search (cmd)
			check has_command: tools.found end
			
			oldsel := selection
			selection := tools.index
			
			paint_rectangle (selection_rectangle (oldsel))
			paint_rectangle (selection_rectangle (selection))
		end

feature
	
	when_created is
		local
			rect: RECTANGLE
		do
			set_border (false)

			!!tools.make
			selection := 1
			
			rect := size
			rect.set_width (Icon_size)
			set_size (rect)
		end

	when_mouse_clicked (pt: POINT) is
		local
			cnt: INTEGER
		do
			-- find selection
			cnt := ( (pt.y - start) // interval) + 1

			-- if changed paint
			if (cnt >= 0 and cnt <= tools.count) and then (cnt /= selection) then
				-- change mode
				tools.set_index (cnt)
				tools.item.execute
			end
		end

	when_drawn is
		local
			ico: ICON
			pt: POINT
			it: P_ITERATOR[CMD_TOOL]
		do
			!!pt.make
			pt.set_x (start)
			pt.set_y (pt.x)
			
			draw_selection
			
			from
				it := tools.iterator
				it.first
			variant it.forward_variant until it.outside loop
				-- draw icon
				ico := it.item.icon
				ico.set_position (pt)
				device.draw (ico)

				-- next
				pt.set_y (pt.y + interval)
				it.forth
			end
		end

feature {NONE}
	
	start: INTEGER is
		do
			Result := 0 -- (Icon_size//2)
		end

	interval: INTEGER is
		do
			Result := Icon_size -- *3//2
		end

	selection_rectangle (sel: INTEGER): RECTANGLE is
		local
			pt: POINT
		do
			!!pt.make
			pt.set_x (start)
			pt.set_y (interval * (sel-1) + start)
			
			!!Result.make
			Result.set_width (Icon_size)
			Result.set_height (Icon_size)
			Result.translate_lower (pt)
		end

	draw_selection is
		local
			dr: DRAWABLE_RECTANGLE
			gr: GRAPHICS_ROUTINES
			select_clr: RGB_COLOR
		do
			!!select_clr.dark_grey

			!!gr
			gr.set_pen (device, select_clr)
			gr.set_brush (device, select_clr)

			!!dr.make
			dr.set_position (selection_rectangle (selection))
			device.draw (dr)
		end

end
