-- Fenestra; version 0.80(beta); 1998-09-27

-- 
-- Copyright (c) 1998 Franck Arnaud, http://www.altsoft.demon.co.uk/
-- 
-- This package is distributed under the Eiffel Forum Freeware License, 
-- version 1 (see file forum.txt for full text). The main points are:
-- 
-- "Permission is hereby granted, without written agreement and without 
--  license or royalty fees, to use, copy, modify and/or distribute this 
--  package, provided that: copyright notices are retained unchanged [...]
-- 
--  Permission is hereby also granted [...] to distribute binary programs 
--  which depend on this package, provided that: if the binary program 
--  depends on a modified version of this package, you must publicly release 
--  the modified version of this package [...].
-- 
--  THIS PACKAGE IS PROVIDED "AS IS" AND WITHOUT WARRANTY [...]"
-- 

indexing
	description: "Draw a 5-branch star"
	status: public_domain

class FIVE_BRANCH_STAR

creation
	make

feature -- Creation

	make (c: POINT; r: INTEGER) is
		require
			valid: c /= Void
		do
			!!m
			set_centre (c)
			set_radius (r)
		ensure
			keep_reference: centre=c
		end

feature {NONE}
	
	centre: POINT
	radius: INTEGER

feature {NONE} -- Mixin
	
	m: P_MATH

feature -- Position & size
	
	set_centre (pt: POINT) is
		require
			valid_point: pt /= Void
		do
			centre := pt
		ensure
			keep_reference: centre = pt
		end

	set_radius (r: INTEGER) is
		require
			valid_radius: r >= 0
		do
			radius := r
		end

feature {NONE} -- Maths
	
	kradius: DOUBLE is
		once
			Result := m.sine (m.Pi_value/10.0) / m.cosine (m.Pi_value/5.0)
		end

	tho: DOUBLE is
		once
			Result := m.Pi_value * 2.0 / 5.0
		end

	pi5: DOUBLE is
		once
			Result := m.Pi_value / 5.0
		end

	starpoint (r: DOUBLE;n: INTEGER;angl: DOUBLE): POINT is
		local
			x, y: DOUBLE
			sys: P_SYSTEM
		do
			!!sys
			x := r * m.sine ( angl + (tho * n) ) + centre.x
			y := r * m.cosine ( angl + (tho * n) ) + centre.y
			!!Result.make
			Result.set_xy (sys.floor (x), sys.floor (y))
		end

feature -- Drawing

	polygon: DRAWABLE_POLYGON is
		local
			n: INTEGER
			iradius: DOUBLE 
			pt: POINT
		do
			check valid_coord: centre /= Void and then radius >= 0 end

			!!Result.make
			
			if radius = 0 then
				Result.add_point (centre)
			else
				from
					n := 0
					iradius := kradius * radius
					pt := starpoint (iradius, n, - pi5)
					Result.add_point (pt)
				variant
					5 - n
				until
					n > 4
				loop
					-- draw branch
					pt := starpoint (radius, n, 0)
					Result.add_point (pt)
					pt := starpoint (iradius, n, pi5)
					Result.add_point (pt)

					-- loop
					n := n + 1
				end -- loop
			end -- radius = 0
		end

end -- class FIVE_BRANCH_STAR
