-- Fenestra; version 0.80(beta); 1998-09-27

-- 
-- Copyright (c) 1998 Franck Arnaud, http://www.altsoft.demon.co.uk/
-- 
-- This package is distributed under the Eiffel Forum Freeware License, 
-- version 1 (see file forum.txt for full text). The main points are:
-- 
-- "Permission is hereby granted, without written agreement and without 
--  license or royalty fees, to use, copy, modify and/or distribute this 
--  package, provided that: copyright notices are retained unchanged [...]
-- 
--  Permission is hereby also granted [...] to distribute binary programs 
--  which depend on this package, provided that: if the binary program 
--  depends on a modified version of this package, you must publicly release 
--  the modified version of this package [...].
-- 
--  THIS PACKAGE IS PROVIDED "AS IS" AND WITHOUT WARRANTY [...]"
-- 

indexing
	cluster: Fenestra, System
	description: "System clock"
	interface: client, mixin

class F_CLOCK

inherit
	WIN_API
		end

feature -- Clock

	get_local_time is
			-- Get time in the local time zone.
		do
			get_time (wrt.Filo_local)
			last_time.set_timezone_bias (wrt.win_timezone)
		ensure
			done: last_time /= Void and then last_time.is_valid
			zoned: not last_time.is_local -- has time zone information
		end

	get_utc_time is
			-- Get Universal Time (GMT).
		do
			get_time (wrt.Filo_gmt)
			last_time.set_timezone_bias (0)
		ensure
			done: last_time /= Void and then last_time.is_valid
			utc: last_time.is_utc
		end

	last_time: P_DATE_TIME;
			-- Last result.

feature -- System timer

	get_tick is
			-- Number of milliseconds since system started, wraps 
			-- around zero every so often.
		do
			last_tick := wrt.win_tick
		ensure
			positive: last_tick >= 0
			wraps: -- last_tick > old last_tick or last_tick < old last_tick
		end

	last_tick: INTEGER;
			-- Result from previous get_tick call.
		
feature {NONE}
	
	get_time (zone: INTEGER) is
		do
			!!last_time.make
			last_time.set_date (wrt.win_time (zone, wrt.Filo_year),
					wrt.win_time (zone, wrt.Filo_month),
					wrt.win_time (zone, wrt.Filo_day))
			last_time.set_time (wrt.win_time (zone, wrt.Filo_hour),
					wrt.win_time (zone, wrt.Filo_minute),
					wrt.win_time (zone, wrt.Filo_second))
		end
	
end -- class F_CLOCK
