-- Fenestra; version 0.80(beta); 1998-09-27

-- 
-- Copyright (c) 1998 Franck Arnaud, http://www.altsoft.demon.co.uk/
-- 
-- This package is distributed under the Eiffel Forum Freeware License, 
-- version 1 (see file forum.txt for full text). The main points are:
-- 
-- "Permission is hereby granted, without written agreement and without 
--  license or royalty fees, to use, copy, modify and/or distribute this 
--  package, provided that: copyright notices are retained unchanged [...]
-- 
--  Permission is hereby also granted [...] to distribute binary programs 
--  which depend on this package, provided that: if the binary program 
--  depends on a modified version of this package, you must publicly release 
--  the modified version of this package [...].
-- 
--  THIS PACKAGE IS PROVIDED "AS IS" AND WITHOUT WARRANTY [...]"
-- 

indexing
	cluster: Fenestra, System, Synchronisation
	description: "Synchronisation event"
	interface: inheritance

class F_EVENT

inherit
	F_SYNCH_OBJECT
		end

creation
	make, make_anonymous,
	make_auto, make_existing

feature {CREATION_NONE}
	
	make (name: STRING) is
			-- Create event object or open existing.
			-- Default: new event is not set.
		require
			ok: name /= Void and then is_valid_name (name)
		do
			finalisation_register
			set_handle (wrt.win_event_create (wrt.ew_string (name).to_c, true))
		ensure
			done: -- is_valid set
		end

	make_anonymous is
			-- Create event object with no name.
			-- Default: new event is not set.
		do
			finalisation_register
			set_handle (wrt.win_event_create (wrt.ew_string ("").to_c, true))
		ensure
			done: is_valid
		end

	make_auto (name: STRING) is
			-- Create event object which is automatically reset after 
			-- a single waiting thread has been released.
			-- Default: new event is not set.
		require
			ok: name /= Void and then is_valid_name (name)
		do
			finalisation_register
			set_handle (wrt.win_event_create (wrt.ew_string (name).to_c, false))
		ensure
			done: -- is_valid set
		end

	make_existing (name: STRING) is
			-- Open existing event object.
		require
			ok: name /= Void and then is_valid_name (name)
		do
			finalisation_register
			set_handle (wrt.win_event_open (wrt.ew_string (name).to_c))
		ensure
			done: -- is_valid set
		end

feature -- Change status

	set is
			-- Set event.
		require
			valid: is_valid
		do
			wrt.win_event_set (handle)
		end

	reset is
			-- Reset event.
		require
			valid: is_valid
		do
			wrt.win_event_reset (handle)
		end

end -- class F_EVENT
