-- Fenestra; version 0.80(beta); 1998-09-27

-- 
-- Copyright (c) 1998 Franck Arnaud, http://www.altsoft.demon.co.uk/
-- 
-- This package is distributed under the Eiffel Forum Freeware License, 
-- version 1 (see file forum.txt for full text). The main points are:
-- 
-- "Permission is hereby granted, without written agreement and without 
--  license or royalty fees, to use, copy, modify and/or distribute this 
--  package, provided that: copyright notices are retained unchanged [...]
-- 
--  Permission is hereby also granted [...] to distribute binary programs 
--  which depend on this package, provided that: if the binary program 
--  depends on a modified version of this package, you must publicly release 
--  the modified version of this package [...].
-- 
--  THIS PACKAGE IS PROVIDED "AS IS" AND WITHOUT WARRANTY [...]"
-- 

indexing
	cluster: Fenestra, System
	description: "TCP/IP (v4) Address"
	interface: client

class IP_ADDRESS

inherit
	WIN_API
		redefine
			out, copy, is_equal
		end

creation
	make

feature -- Reset
	
	make, reset is
			-- Create or reset IP address.
		do
			b1 := 0
			b2 := 0
			b3 := 0
			b4 := 0
		ensure
			reset: b1 = 0 and b2 = 0 and b3 = 0 and b4 = 0
		end

feature -- Address
	
	b1, b2, b3, b4: INTEGER
			-- Address components.

	last_result: BOOLEAN
			-- Result of side-effect procedure(s).

feature -- Status

	to_string: STRING is
			-- Standard decimal representation.
			-- ("127.0.0.1")
		do
			!!Result.make (0)
			Result.append_integer (b1)
			Result.append_character ('.')
			Result.append_integer (b2)
			Result.append_character ('.')
			Result.append_integer (b3)
			Result.append_character ('.')
			Result.append_integer (b4)
		ensure
			is_copy: Result /= Void
		end

feature -- Change

	set (v1, v2, v3, v4: INTEGER) is
			-- Set address from components.
		require
			valid_values: -- see invariant
		do
			b1 := v1
			b2 := v2
			b3 := v3
			b4 := v4
		end

	set_string (str: STRING) is
			-- Set from string representation ("127.0.0.1").
		require
			ok: str /= Void
		local
			i, pre_i, cnt, bx: INTEGER
			fmt: P_FORMAT
		do
			!!fmt
			reset
			from
				i := 1
				cnt := 1
				pre_i := 1
			invariant pre_i <= i variant str.count + 1 - i until
				cnt > 4 or i > str.count
			loop
				if str.item (i) = '.' then
					if pre_i < i then
						bx := fmt.str_to_int (str.substring (pre_i, i-1))

						inspect cnt
						when 1 then b1 := bx
						when 2 then b2 := bx
						when 3 then b3 := bx
						when 4 then b4 := bx
						end 
						
						cnt := cnt + 1
						pre_i := i + 1
					end
				end
				i := i + 1
			end -- loop
		end

feature -- Name resolution

	resolve_by_name (nm: STRING) is
			-- Find TCP/IP address from Fully Qualified Domain Name by 
			-- asking the Domain Name Service (DNS). This is a blocking call.
		require
			ok: nm /= Void
			blocking: -- may take several seconds
		do
			reset
			last_result := wrt.win_tcp_byname (wrt.ew_string (nm).to_c)
			if last_result then
				b1 := wrt.win_lastint1
				b2 := wrt.win_lastint2
				b3 := wrt.win_lastint3
				b4 := wrt.win_lastint4
			end
		ensure
			done: -- last_result implies address set
		end

feature -- GENERAL

	out: STRING is
			-- Standard string representation.
		do
			Result := to_string
		end

	is_equal (other: like Current): BOOLEAN is
			-- Equality.
		do
			Result := b1 = other.b1 and b2 = other.b2 and b3 = other.b3 and b4 = other.b4
		end

	copy (other: like Current) is
			-- Copy address.
		do
			set (other.b1, other.b2, other.b3, other.b4)
		end

invariant
	address_ok: (b1 >= 0 and b1 <= 255) and (b2 >= 0 and b2 <= 255) and 
		(b3 >= 0 and b3 <= 255) and (b4 >= 0 and b4 <= 255)

end -- class IP_ADDRESS
