-- Fenestra; version 0.80(beta); 1998-09-27

-- 
-- Copyright (c) 1998 Franck Arnaud, http://www.altsoft.demon.co.uk/
-- 
-- This package is distributed under the Eiffel Forum Freeware License, 
-- version 1 (see file forum.txt for full text). The main points are:
-- 
-- "Permission is hereby granted, without written agreement and without 
--  license or royalty fees, to use, copy, modify and/or distribute this 
--  package, provided that: copyright notices are retained unchanged [...]
-- 
--  Permission is hereby also granted [...] to distribute binary programs 
--  which depend on this package, provided that: if the binary program 
--  depends on a modified version of this package, you must publicly release 
--  the modified version of this package [...].
-- 
--  THIS PACKAGE IS PROVIDED "AS IS" AND WITHOUT WARRANTY [...]"
-- 

indexing
	cluster: Fenestra, System
	description: "Notifier for synchronisation objects"
	interface: client

class SYNCH_NOTIFIER

inherit
	WIN_API
		end

creation {F_SYNCH_OBJECT}
	make

feature {NONE} -- Creation
	
	make is
			-- Create stand alone synchronisation notifier.
		do
			init_event_notifier
			latency := wrt.Infinite
		end

feature {NONE} -- Attributes
	
	events: P_DICTIONARY[F_SYNCH_OBJECT,INTEGER]
			-- Outstanding events.

	has_event: BOOLEAN
			-- Last event result.

	latency: INTEGER
			-- Latency of object wait.

feature {NONE}
	
	Default_latency: INTEGER is 250

feature {NONE} -- Initialisation

	init_event_notifier is
		do
			latency := Default_latency
			!!events.make
		end

feature {F_SYNCH_OBJECT}
	
	register_synch_object (evt: F_SYNCH_OBJECT) is
			-- Register a synchronisation object for event wait.
		require
			ok: evt /= Void
		do
			events.add (evt, evt.handle)
		end

	unregister_synch_object (evt: F_SYNCH_OBJECT) is
			-- Unregister synchronisation object.
		require
			ok: evt /= Void
			has_events: has_events
			has_this_event: -- events.has (evt)
		do
			check has_some: not events.is_empty end
			if events.item /= evt then
				events.key_search (evt.handle)
				check found: events.found end
			end
			
			check really_got_it: events.item = evt end
			events.remove
		end

feature -- Options

	set_latency (nlat_ms: INTEGER) is
			-- Change maximum duration of a single wait operation.
			-- Default: infinite
		require
			ok: nlat_ms >= 0
		do
			latency := nlat_ms
		ensure
			done: latency = nlat_ms
		end

feature -- Wait
	
	has_events: BOOLEAN is
			-- Is there at least one event to wait for?
		do
			Result := not events.is_empty
		end

	wait_once is
			-- Wait for one event.
		require
			has_events: has_events
		local
			evt_array: F_MEMORY
			it: P_ITERATOR[INTEGER]
			i, sig_obj: INTEGER
			item: F_SYNCH_OBJECT
		do
			has_event := false

			-- prepare array
			--? cache it
			!!evt_array.make (events.count*4)
			from
				i := 1
				it := events.to_key_list.iterator
				it.first
			variant it.forward_variant until it.outside loop
				evt_array.set_double_word (i, it.item)
				i := i + 4
				it.forth
			end
			
			-- get signaled object
			sig_obj := wrt.win_synch_mwait (evt_array.to_windows, events.count, latency)
			
			if sig_obj >= 0 then
				-- process event
				--? wrt.win_lastbool1 is set if abandonned mutex
				events.key_search (evt_array.double_word ((sig_obj*4)+1))
				check found: events.found end
				item := events.item
				events.remove -- remove before event processing which can trigger modifications of event list

				item.internal_when_signaled
				has_event := true
			end
		end

	wait is
			-- Process all outstanding events.
		do
			from
				has_event := true
			until
				(not has_event) or (not has_events)
			loop
				wait_once
			end
		end

end -- class SYNCH_NOTIFIER
