-- Fenestra; version 0.80(beta); 1998-09-27

-- 
-- Copyright (c) 1998 Franck Arnaud, http://www.altsoft.demon.co.uk/
-- 
-- This package is distributed under the Eiffel Forum Freeware License, 
-- version 1 (see file forum.txt for full text). The main points are:
-- 
-- "Permission is hereby granted, without written agreement and without 
--  license or royalty fees, to use, copy, modify and/or distribute this 
--  package, provided that: copyright notices are retained unchanged [...]
-- 
--  Permission is hereby also granted [...] to distribute binary programs 
--  which depend on this package, provided that: if the binary program 
--  depends on a modified version of this package, you must publicly release 
--  the modified version of this package [...].
-- 
--  THIS PACKAGE IS PROVIDED "AS IS" AND WITHOUT WARRANTY [...]"
-- 

indexing
	cluster: Fenestra, Windows, Keyboard
	description: "Keyboard accelerator (shortcut)"
	interface: client

class ACCELERATOR

inherit
	WIN_ACCEL_CONST
		end
	WIN_API
		end

creation
	make

feature {NONE} -- Internal attribute(s)
	
	char: CHARACTER
	vkey: VIRTUAL_KEY

feature -- Status

	has_alt, has_control, has_shift: BOOLEAN;
			-- Associated shortcut keys.

feature -- Status

	is_key: BOOLEAN is
			-- Is this accelerator based on a virtual key?
		do
			Result := vkey /= Void
		ensure
			coherent: Result = not is_character
		end

	is_character: BOOLEAN is
			-- Is this accelerator based on a character code?
		do
			Result := not is_key
		ensure
			coherent: Result = not is_key
		end


feature -- Update

	make, reset is
			-- Reset control keys.
		do
			has_alt := false
			has_control := false
			has_shift := false
			char := '%/0/'
			vkey := Void
		end

	set_alt is
			-- Add ALT key to shortcut.
		do
			has_alt := true
		ensure
			ok: has_alt
		end

	set_control is
			-- Add CONTROL key to shortcut.
		do
			has_control := true
		ensure
			ok: has_control
		end

	set_shift is
			-- Add SHIFT key to shortcut.
		do
			has_shift := true
		ensure
			ok: has_shift
		end

	set_character (ch: CHARACTER) is
			-- Associate character with keyboard shortcut.
			-- (this cancels the associated vkey if any).
		require
			case_sensitive: -- character based accelerator is case sensitive
		do
			char := ch
			vkey := Void
		ensure
			registered: char = ch
			is_char_accel: is_key
		end

	set_virtual_key (vk: VIRTUAL_KEY) is
			-- Associate virtual key with keyboard shortcut.
			-- (this cancels the associated character if any).
		require
			ok: vk /= Void
		do
			vkey := vk
		ensure
			keep_reference: vkey = vk
			is_vk_accel: is_key
		end

feature {WIN_API} -- OS interface
	
	to_windows (cmd_id: INTEGER): F_MEMORY is
			-- Make low-level structure.
		local
			flags: INTEGER
		do
			!!Result.make (Accel_size)
			
			-- associated key/char
			if is_key then
				flags := wrt.Fvirtkey
				Result.set_word (Key_offset, vkey.value)
			else
				flags := 0
				Result.set_word (Key_offset, char.code)
			end

			-- finish flags for control keys
			if has_control then flags := Result.bit_or (flags, wrt.Fcontrol) end
			if has_alt then flags := Result.bit_or (flags, wrt.Falt) end
			if has_shift then flags := Result.bit_or (flags, wrt.Fshift) end
			Result.set_byte (Flags_offset, flags)
			
			-- associated command
			Result.set_word (Cmd_offset, cmd_id)
		end

end -- class ACCELERATOR

