-- Fenestra; version 0.80(beta); 1998-09-27

-- 
-- Copyright (c) 1998 Franck Arnaud, http://www.altsoft.demon.co.uk/
-- 
-- This package is distributed under the Eiffel Forum Freeware License, 
-- version 1 (see file forum.txt for full text). The main points are:
-- 
-- "Permission is hereby granted, without written agreement and without 
--  license or royalty fees, to use, copy, modify and/or distribute this 
--  package, provided that: copyright notices are retained unchanged [...]
-- 
--  Permission is hereby also granted [...] to distribute binary programs 
--  which depend on this package, provided that: if the binary program 
--  depends on a modified version of this package, you must publicly release 
--  the modified version of this package [...].
-- 
--  THIS PACKAGE IS PROVIDED "AS IS" AND WITHOUT WARRANTY [...]"
-- 

indexing
	cluster: Fenestra, Windows
	description: "Application's main window"
	interface: inheritance

class APPLICATION

inherit
	OVERLAPPED_WINDOW
		redefine
			when_destroyed,
			internal_when_drop
		end

creation
	-- effectively deferred
	-- (actual creator should be 'make' in heir)

feature {CREATION_NONE} -- Creation
	
	make is
			-- Make application main window.
		do
			wrt.win_initwin -- register classes
			!!wo.make_handle (wrt.win_create) -- create windows
			
			application_register
		ensure
			no_return: -- warning: no return before app end
		end

feature {NONE}
	
	application_register is
			-- Initialise application window.
		do
			dragdrop := false
			notifier.set_application (Current) -- store handle & get screen dpi for font
		
			register -- register with notifier
			notifier.start -- start the event loop
		end

feature -- Application wide settings

	set_background_color (bcol: BRUSH) is
			-- Set background window's colour brush 
			-- (for ALL windows in this application, it is not 
			-- processed immediately on opened windows).
		require
			valid_brush: bcol /= Void
			valid_window: is_valid
		do
			if wbcol /= Void then
				wbcol.force_dispose
			end
			wbcol := bcol.to_windows
			wrt.win_back_color (handle, wbcol.handle)
		end

	accept_dropped_files is
			-- Accept files dropped onto our application window.
		require
			valid_window: is_valid
		do
			dragdrop := true
			wrt.win_drag_accept (handle, true)
		end

	set_icon (ico: ICON) is
			-- Set window's icon (for ALL windows in this application).
		require
			valid_icon: ico /= Void
			valid_window: is_valid
		do
			wrt.win_icon_set (handle, ico.handle)
		end

feature {NONE}
	
	wbcol: WIN_BRUSH

feature {PUBLIC_NONE}
	
	when_files_dropped (list: P_LIST[STRING]) is
			-- Files have been dropped on our application window.
		require
			valid_list: list /= Void
			is_copy: -- `list' can be modified.
		do
		end

feature {NONE} -- Private attributes
	
	dragdrop: BOOLEAN

feature {NONE}
	
	when_destroyed is
		do
			if dragdrop then
				wrt.win_drag_accept (handle, false)
			end
			standard_when_destroyed
			wrt.win_quit
		end

	internal_when_drop (drophandle: INTEGER) is
			-- Internal drop files handling.
		require else
			effective: true
		local
			i, files: INTEGER
			filelist: P_LINKED_LIST[STRING]
		do
			check drop_on: dragdrop = true end
			
			!!filelist.make

			from
				i := 0
				files := wrt.win_drag_files (drophandle)
			variant
				files - i
			until
				i >= files
			loop
				wrt.win_drag_getfile (drophandle, i)
				filelist.add (wrt.win_laststring)
				i := i + 1
			end

			wrt.win_drag_finish (drophandle)

			when_files_dropped (filelist)
		end

invariant
	is_unique: -- only one instance of APPLICATION should exist per application

end
