-- Fenestra; version 0.80(beta); 1998-09-27

-- 
-- Copyright (c) 1998 Franck Arnaud, http://www.altsoft.demon.co.uk/
-- 
-- This package is distributed under the Eiffel Forum Freeware License, 
-- version 1 (see file forum.txt for full text). The main points are:
-- 
-- "Permission is hereby granted, without written agreement and without 
--  license or royalty fees, to use, copy, modify and/or distribute this 
--  package, provided that: copyright notices are retained unchanged [...]
-- 
--  Permission is hereby also granted [...] to distribute binary programs 
--  which depend on this package, provided that: if the binary program 
--  depends on a modified version of this package, you must publicly release 
--  the modified version of this package [...].
-- 
--  THIS PACKAGE IS PROVIDED "AS IS" AND WITHOUT WARRANTY [...]"
-- 

indexing
	cluster: Fenestra, Windows
	description: "Carets (text entry-point pointers)"
	interface: client

class CARET

inherit
	WIN_API
		end

creation
	make

feature {CREATION_NONE} -- Creation
	
	make (window: like parent; width, height: INTEGER) is
			-- Create a caret.
		require
			valid_parent: window /= Void
			good_size: width > 0 and height > 0
			one_per_window: not has_caret (window)
		do
			!!pos.make
			parent := window
			parent.set_caret (Current)
			wrt.win_caret_create (parent.handle, width, height)
			is_valid := true
		ensure
			created: is_valid = true
			keep_reference: parent = window
		end

feature -- Preconditions

	has_caret (win: WINDOW): BOOLEAN is
			-- Is there only one caret associated with the window?
		do
			Result := (win.caret /= Void)
		end
	
	is_valid: BOOLEAN
			-- Is this caret valid?

feature {NONE} -- parent handle
	
	parent: WINDOW
	pos: POINT
	
feature -- Actions

	hide is
			-- Hide the caret.
		require
			valid: is_valid = true
		do
			wrt.win_caret_hide (parent.handle)
		end

	show is
			-- Show caret.
		require
			valid: is_valid = true
		do
			wrt.win_caret_show (parent.handle)
		end
	
	destroy is
			-- Destroy caret, must be done before losing keyboard focus.
		require
			valid: is_valid = true
		do
			wrt.win_caret_destroy
			parent.set_caret (Void)
			is_valid := false
		ensure
			invalid: is_valid = false
		end

feature -- Position

	position: POINT is
			-- Caret's current position.
		do
			!!Result.make
			Result.copy (pos)
		ensure
			is_copy: -- may be modified.
		end

	set_position (pt: POINT) is
			-- Set caret's position in its parent window.
		require
			good_point: pt /= Void
			valid: is_valid = true
		do
			pos.copy (pt)
			wrt.win_caret_position (pt.x, pt.y) 
			-- Caution: coordinate system used depends on CS_OWNDC status.
		end

invariant
	has_parent: parent /= Void
	used_with_parent: is_valid implies parent.is_valid
	--used_when_focus: is_valid implies parent.has_focus (false when destroying)

end -- class CARET
