-- Fenestra; version 0.80(beta); 1998-09-27

-- 
-- Copyright (c) 1998 Franck Arnaud, http://www.altsoft.demon.co.uk/
-- 
-- This package is distributed under the Eiffel Forum Freeware License, 
-- version 1 (see file forum.txt for full text). The main points are:
-- 
-- "Permission is hereby granted, without written agreement and without 
--  license or royalty fees, to use, copy, modify and/or distribute this 
--  package, provided that: copyright notices are retained unchanged [...]
-- 
--  Permission is hereby also granted [...] to distribute binary programs 
--  which depend on this package, provided that: if the binary program 
--  depends on a modified version of this package, you must publicly release 
--  the modified version of this package [...].
-- 
--  THIS PACKAGE IS PROVIDED "AS IS" AND WITHOUT WARRANTY [...]"
-- 

indexing
	cluster: Fenestra, Windows
	description: "Base file selection dialog box"
	interface: client

deferred class FILE_DIALOG

inherit
	STANDARD_DIALOG
		end

feature {CREATION_NONE}
	
	make (p: OVERLAPPED_WINDOW) is
			-- Create dialog.
		do
			parent := p
			has_selection := false
			!!title.make (0)
			!!initdir.make (0)
			!!initfile.make (0)
			reset_filters
		end

feature {NONE}
	
	dialog_type: BOOLEAN is
			-- File selection type: true = open, false = save.
		deferred
		end

feature {NONE}
	
	parent: WINDOW

	title, initdir, initfile: STRING
	filters: STRING
	lastfile: STRING

feature {NONE} -- Clean string of separators

	cleanstr (str: STRING): STRING is
		require
			valid: str /= Void
		local
			i: INTEGER
		do
			!!Result.make (0)
			from
				i := 1
			variant
				str.count + 1 - i
			until
				i > str.count
			loop
				if str.item (i) = wrt.Separator_filter then
					Result.append_character (' ')
				else
					Result.append_character (str.item (i))
				end
				i := i + 1
			end
		end
		
feature -- Setup

	set_title (str: STRING) is
			-- Set dialog title.
			-- Default: provided by operating system.
		require
			valid: str /= Void
		do
			title.copy (str)
		end

	set_directory (str: STRING) is
			-- Set inital directory.
			-- Default: current directory.
		require
			valid: str /= Void
		do
			initdir.copy (str)
		end

	set_file (str: STRING) is
			-- Set inital filename.
			-- Default: none.
		require
			valid: str /= Void
		do
			initfile.copy (str)
		end

	add_filter (name, pattern: STRING) is
			-- Add a file type filter with descriptive `name' and 
			-- file type `pattern'. (eg: "Text files", "*.txt;*.doc").
			-- The first added filter is the default one.
		require
			valid: name /= Void and pattern /= Void
		do
			filters.append_string (cleanstr (name))
			filters.append_character (wrt.Separator_filter)
			filters.append_string (cleanstr (pattern))
			filters.append_character (wrt.Separator_filter)
		end

	reset_filters is
			-- Reset file type filters.
		do
			!!filters.make (0)
		end

feature -- Action
	
	has_selection: BOOLEAN;
			-- Has last dialog produced a valid file name.
			
	execute is
			-- Display the file open dialog.
		do
			wrt.win_std_file (dialog_type, parent.handle,
							wrt.ew_string (title).to_c, wrt.ew_string (initdir).to_c,
							wrt.ew_string (initfile).to_c, wrt.ew_string (filters).to_c)
			has_selection := wrt.win_lastbool1
			lastfile := wrt.win_laststring
		end

feature -- Result
	
	last_file_name: STRING is
			-- Last selected file name.
		require
			has_select: has_selection
		do
			Result := lastfile
		ensure
			valid: Result /= Void
		end
	
end -- class FILE_DIALOG
