-- Fenestra; version 0.80(beta); 1998-09-27

-- 
-- Copyright (c) 1998 Franck Arnaud, http://www.altsoft.demon.co.uk/
-- 
-- This package is distributed under the Eiffel Forum Freeware License, 
-- version 1 (see file forum.txt for full text). The main points are:
-- 
-- "Permission is hereby granted, without written agreement and without 
--  license or royalty fees, to use, copy, modify and/or distribute this 
--  package, provided that: copyright notices are retained unchanged [...]
-- 
--  Permission is hereby also granted [...] to distribute binary programs 
--  which depend on this package, provided that: if the binary program 
--  depends on a modified version of this package, you must publicly release 
--  the modified version of this package [...].
-- 
--  THIS PACKAGE IS PROVIDED "AS IS" AND WITHOUT WARRANTY [...]"
-- 

indexing
	cluster: Fenestra, Windows
	description: "Menu bar or popup menu"
	interface: client

deferred class MENU

inherit
	WIN_OBJECT

	WIN_API

feature {MENU_ITEM} -- Parent

	parent: OVERLAPPED_WINDOW
			-- Parent window

	items: P_LIST[MENU_ITEM]
			-- Menu items

feature {NONE}
	
	initialize is
		do
			finalisation_register
			!P_LINKED_LIST[MENU_ITEM]!items.make
		ensure
			has_items: items /= Void
		end

feature -- Menu items management

	add_popup (item: POPUP_MENU) is
			-- Add a popup menu at the end of the current menu.
			-- It cannot be removed.
		require
			valid_menu: is_valid
			valid_item: item /= Void
		do
			wrt.win_menu_addpopup (handle, item.handle, wrt.ew_string (item.name).to_c)
		end

	add_separator is
			-- Add a separator at the end of the current menu.
			-- It cannot be removed.
		require
			valid: is_valid
		do
			wrt.win_menu_addsep (handle)
		end
	
	add_item (item: MENU_ITEM) is
			-- Add an item at the end of the current menu.
			-- Use item.detach to remove the item.
		require
			valid_menu: is_valid
			existing_item: item /= Void
			unattached_item: not item.is_valid
		do
			check has_items: items /= Void end

			-- attach
			item.set_parent (Current)

			-- install
			parent.register_menu_item (item) -- makes ID
			wrt.win_menu_additem (handle, item.id, wrt.ew_string (item.name).to_c)
		
			-- add item
			items.add (item)
		ensure
			attached: item.is_valid
		end

feature {MENU_ITEM}

	remove_item (item: MENU_ITEM) is
			-- Remove a menu item attached to this menu.
		require
			valid_menu: is_valid
			item_ok: item /= Void
			attached: item.is_valid
		do
			check
				has_items: items /= Void
				has_the_item: -- items.has (item)
			end

			-- unregister from parent
			parent.unregister_menu_item (item)

			-- remove from own list
			items.equality_search (item)
			check has_item: items.found end
			items.remove

			-- remove this from parent
			if items.is_empty then
				parent.unregister_menu (Current)
			end
		end

feature {OVERLAPPED_WINDOW}
	
	when_prepared is
			-- Prepare.
		local
			it: P_ITERATOR[MENU_ITEM]
		do
			from
				it := items.iterator
				it.first
			variant it.forward_variant until it.outside loop
				it.item.internal_when_prepared
				it.forth
			end
		end

feature
	
	is_valid: BOOLEAN is
		do
			Result := is_valid_handle
		end

invariant
	valid_parent: parent /= Void
	items_ok: items /= Void

end -- class MENU
