-- Fenestra; version 0.80(beta); 1998-09-27

-- 
-- Copyright (c) 1998 Franck Arnaud, http://www.altsoft.demon.co.uk/
-- 
-- This package is distributed under the Eiffel Forum Freeware License, 
-- version 1 (see file forum.txt for full text). The main points are:
-- 
-- "Permission is hereby granted, without written agreement and without 
--  license or royalty fees, to use, copy, modify and/or distribute this 
--  package, provided that: copyright notices are retained unchanged [...]
-- 
--  Permission is hereby also granted [...] to distribute binary programs 
--  which depend on this package, provided that: if the binary program 
--  depends on a modified version of this package, you must publicly release 
--  the modified version of this package [...].
-- 
--  THIS PACKAGE IS PROVIDED "AS IS" AND WITHOUT WARRANTY [...]"
-- 

indexing
	cluster: Fenestra, Windows
	description: "Standard message box"
	interface: client

class MESSAGE_BOX

inherit
	STANDARD_DIALOG
		end

creation
	make

feature {CREATION_NONE}
	
	make (par: OVERLAPPED_WINDOW) is
			-- Create.
		do
			parent := par
			!!title.make (0)
			!!text.make (0)
			set_title ("Error")
			set_text ("Void text")
			reset_icon
			reset_buttons
			has_selection := false
		end

feature {NONE}
	
	parent: WINDOW
	title, text: STRING
	icon, buttons: INTEGER

	last_answer: INTEGER

feature -- Icon

	set_icon_information, reset_icon is
			-- Dialog's icon is the letter i in a circle.
		do
			icon := wrt.Mb_iconinformation
		end

	set_icon_exclamation is
			-- Dialog's icon is an exclamation point.
		do
			icon := wrt.Mb_iconexclamation
		end

	set_icon_question is
			-- Dialog's icon is a question mark.
		do
			icon := wrt.Mb_iconquestion
		end

	set_icon_stop is
			-- Dialog's icon is an icon stop.
		do
			icon := wrt.Mb_iconstop
		end

feature -- Text

	set_title (str: STRING) is
			-- Set dialog's title.
		require
			valid: str /= Void
		do
			title.copy (str)
		ensure
			done: title.is_equal (str)
		end

	set_text (str: STRING) is
			-- Set dialog's text.
		require
			valid: str /= Void
		do
			text.copy (str)
		ensure
			done: text.is_equal (str)
		end

feature -- Buttons (labels use national language).
	
	set_buttons_yes_no is
			-- Dialog has 'Yes' and 'No' buttons.
		do
			buttons := wrt.Mb_yesno
		end

	set_buttons_yes_no_cancel is
			-- Dialog has 'Yes' and 'No' and 'Cancel' buttons.
		do
			buttons := wrt.Mb_yesnocancel
		end

	set_buttons_ok, reset_buttons is
			-- Dialog has 'OK' button.
		do
			buttons := wrt.Mb_ok
		end

	set_buttons_ok_cancel is
			-- Dialog has 'OK' and 'Cancel' buttons.
		do
			buttons := wrt.Mb_okcancel
		end

	set_buttons_retry_cancel is
			-- Dialog has 'Retry' and 'Cancel' buttons.
		do
			buttons := wrt.Mb_retrycancel
		end

	set_buttons_abort_retry_ignore is
			-- Dialog has 'Abort' and 'Retry' and 'Ignore' buttons.
		do
			buttons := wrt.Mb_abortretryignore
		end

feature -- Excecution

	has_selection: BOOLEAN
			-- Has the dialog been executed?

	execute is
			-- Display the message box.
		do
			last_answer := wrt.win_message (parent.handle,
					wrt.ew_string (title).to_c, wrt.ew_string (text).to_c,
					icon, buttons)
			has_selection := true
		end

feature -- User answer

	is_yes: BOOLEAN is
			-- User answered 'Yes' or 'OK' or 'Retry'.
		require
			done: has_selection
		do
			Result := (last_answer = wrt.Idok) or else (last_answer = wrt.Idyes) or else (last_answer = wrt.Idretry)
		end

	is_no: BOOLEAN is
			-- User answered 'No' or 'Ignore'.
		require
			done: has_selection
		do
			Result := (last_answer = wrt.Idno) or else (last_answer = wrt.Idignore)
		end

	is_cancel: BOOLEAN is
			-- User answered 'Cancel' or 'Abort'.
		require
			done: has_selection
		do
			Result := (last_answer = wrt.Idcancel) or else (last_answer = wrt.Idabort)
		end
		
end -- class MESSAGE_BOX
