-- Fenestra; version 0.80(beta); 1998-09-27

-- 
-- Copyright (c) 1998 Franck Arnaud, http://www.altsoft.demon.co.uk/
-- 
-- This package is distributed under the Eiffel Forum Freeware License, 
-- version 1 (see file forum.txt for full text). The main points are:
-- 
-- "Permission is hereby granted, without written agreement and without 
--  license or royalty fees, to use, copy, modify and/or distribute this 
--  package, provided that: copyright notices are retained unchanged [...]
-- 
--  Permission is hereby also granted [...] to distribute binary programs 
--  which depend on this package, provided that: if the binary program 
--  depends on a modified version of this package, you must publicly release 
--  the modified version of this package [...].
-- 
--  THIS PACKAGE IS PROVIDED "AS IS" AND WITHOUT WARRANTY [...]"
-- 

indexing
	cluster: Fenestra, Windows
	description: "Notifier - dispatches events"
	interface: client, internal

class NOTIFIER

inherit
	WIN_API
		end

creation {COMMON_NOTIFIER}
	make

feature {NONE} -- Creation

	make is
			-- Create.
		local
			spec_initialisations: WIN_SPEC
		do
			!! spec_initialisations.make -- compiler specific initialisation

			init_windows
			init_controls
			init_timers
		
			wrt.win_init_notifier (Current) -- register self with C runtime.
			wrt.win_init_notifier_ex (Current, $message, $msg_control, $msg_control_key) -- alternative registration with callbacks
		end
		
feature {APPLICATION}
	
	start is
			-- Starts the notifier, called by the APPLICATION window's 
			-- creator.
		do
			-- loop until no events left (end of application)
			from
			until
				wrt.win_event = false
			loop
				--? could call GC here
			end
		end

	set_application (app: APPLICATION) is
		local
			dev_handle: INTEGER
		do
			-- register application handle for timers
			check one: application_handle = 0 end
			application_handle := app.handle

			-- get screen size
			dev_handle := wrt.win_gdi_get (application_handle) -- prepare
			screen_dpi := wrt.win_gdi_capint (dev_handle, wrt.Logpixelsy) -- get DPI
			wrt.win_gdi_release (dev_handle, application_handle) -- release
		end

feature {LOGICAL_FONT}
	
	screen_dpi: INTEGER;
			-- Screen DPI for some font point conversions.

feature {NONE}
	
	application_handle: INTEGER

feature {WINDOW} -- send objects to other window

	send_object (win: WINDOW; object: ANY) is
			-- Send an object to the specified window.
		require
			valid: win /= Void
			valid_object: object /= Void
		do
			win.internal_when_object (object)
		end

feature {NONE} ---------------------------------- Timers
	
	timers: P_DICTIONARY[F_TIMER,INTEGER]

	init_timers is
		do
			!!timers.make
		end

feature {NONE} -- Timers: ID

	Maximum_timer_id: INTEGER is 10000

	set_timer_id (tim: F_TIMER) is
		require
			valid: tim /= Void
		local
			candidate_id: INTEGER
			done: BOOLEAN
			i: INTEGER
			clock: F_CLOCK
			random: P_RANDOM
		do
			!!clock
			clock.get_tick
			!!random.make (clock.last_tick)

			from
				i := Maximum_timer_id
			variant
				i
			until
				done or else i = 0
			loop
				random.get_number (1, Maximum_timer_id)
				candidate_id := random.last_number

				timers.key_search (candidate_id)
				if not timers.found then
					tim.set_id (candidate_id)
					done := true
				end
				
				i := i - 1
			end

			check done: i > 0 end -- should not happen...
		end

feature {F_TIMER} -- Timers
	
	register_timer (tim: F_TIMER) is
		require
			valid: tim /= Void and then not tim.is_active
		do
			check valid_app: application_handle /= 0 end

			set_timer_id (tim)
			timers.add (tim, tim.id)
			wrt.win_timer_start (application_handle, tim.id, tim.delay)
		end

	unregister_timer (tim: F_TIMER) is
		require
			valid: tim /= Void and then tim.is_active
		do
			wrt.win_timer_stop (application_handle, tim.id)
			timers.key_search (tim.id)
			check got_it: timers.found end
			timers.remove
		end

feature {NONE} ---------------------------------- Controls
	
	controls: P_DICTIONARY[STD_CONTROL,INTEGER]

	init_controls is
		do
			!!controls.make
		end

feature {STD_CONTROL} -- Controls: registration
	
	register_std_control (ctl: STD_CONTROL) is
			-- Register a built-in (standard) control. 
		require
			valid: ctl /= Void
		do
			controls.add (ctl, ctl.self.handle)
		end

	register_combobox_edit (win: STD_COMBOBOX[LISTBOX_ITEM]; editctl: INTEGER) is
			-- Register edit control of a combo box.
		require
			valid: win /= Void and editctl /= 0
		do
			controls.add (win, editctl)
		end
	
feature {NONE} -- Subclassed control callbacks

	msg_control_key (winid: INTEGER; c: CHARACTER): BOOLEAN is
			-- Subclassed callback, character entered. (Result: event processed?)
		do
			controls.key_search (winid)
			check got_it: controls.found end
			Result := controls.item.std_control_when_key (c)
		end

	msg_control (winid: INTEGER ; code: INTEGER; char: CHARACTER) is
			-- General subclassed callback.
		do
			-- find control
			controls.key_search (winid)
			check got_it: controls.found end

			if code = wrt.Wm_destroy then -- Subclassed window destroy callback.
				controls.remove
			elseif code = wrt.Wm_setfocus then-- Subclassed callback, window got focus.
				controls.item.std_control_when_focus
			elseif code = wrt.Wm_killfocus then -- lost focus
				controls.item.std_control_when_focus_lost
			elseif code = wrt.Wm_syschar then
				controls.item.std_control_when_altkey (char)
			-- elseif code = wrt.Wm_keydown or code = wrt.Wm_char then
			else
				check not_supported: false end
			end
		end
	
feature {NONE}
	
	msg_command (winid: INTEGER; not_code: INTEGER) is
			-- from 'message'.
		do
			controls.key_search (winid)
			if controls.found then
				-- check got_it: controls.found end
				controls.item.std_control_when_command (not_code)
			-- else unprocessed message
			end
		end

	msg_scroll (winid: INTEGER; code, pos: INTEGER) is
			-- from 'message'.
		do
			controls.key_search (winid)
			if controls.found then
				controls.item.std_control_when_scroll (code, pos)
			-- else unprocessed message
			end
		end

feature {NONE} ---------------------------------- Windows

	windows: P_DICTIONARY[WINDOW,INTEGER]
			-- table of windows

	init_windows is
		do
			!!windows.make
			cacheid := 0
		end

feature {WINDOW} -- Window management
	
	register_window (win: WINDOW) is
			-- Register a window with the notifier, actually creates it 
			-- and start the message loop.
		require
			valid: win /= Void
		do
			add_window (win)
			win.internal_when_created
		end

	unregister_window (win: WINDOW) is
			-- Remove the window from the notifier (when destroyed).
		require
			valid: win /= Void
		do
			-- Remove from list
			remove_window (win)
		end

feature {NONE} -- Window cache

	cacheid: INTEGER -- current window ID (for cacheing)
	cache: WINDOW -- current cached window

feature {NONE} -- Window management

	add_window (win: WINDOW) is
		require
			valid: win /= Void
		do
			windows.add (win, win.handle)
		end

	remove_window (win: WINDOW) is
		require
			valid: win /= Void
		do
			windows.key_search (win.handle)
			check gotit: windows.found end
			if windows.found then
				windows.remove
			end

			cacheid := 0
		end

	is_valid_window_id (id: INTEGER): BOOLEAN is
			-- Check if the given low level IO id is valid.
		do
			if id = 0 then
				Result := false
			elseif id = cacheid then
				Result := true
			else
				windows.key_search (id)
				Result := windows.found
			end
		end

	get_window (id: INTEGER): WINDOW is
			-- Return the Window described by 'id'.
		require
			valid_id: is_valid_window_id (id)
		do
			if id /= cacheid then
				cacheid := 0
				cache := Void
				
				windows.key_search (id)
				if windows.found then 
					cache := windows.item
					check cache.handle = id end
					cacheid := id
				end
			end
			Result := cache
		ensure
			good_result: Result /= Void
			cache_ok: (cacheid = 0) = (cache = Void)
			cache_coherent: cache /= Void implies cache.handle = cacheid
			window_ok: Result.handle = id 
		end

feature {NONE} -- Standard callback
	
	message (winid: INTEGER; code: INTEGER; 
				char: CHARACTER ; int1, int2: INTEGER; queueable: BOOLEAN) is
			-- Callback from the Windows API. 
		local
			event: WIN_EVENT
			win: WINDOW
		do
			if winid = application_handle and then code = wrt.Wm_timer then
				-- get timer
				timers.key_search (int1)
				check found: timers.found end
				if timers.found then
					timers.item.internal_when_executed
				end
			elseif code = wrt.Filo_command then
				msg_command (winid, int1)
			elseif code = wrt.Filo_scroll then
				msg_scroll (winid, int1, int2)
			elseif is_valid_window_id (winid) then
				-- prepare event
				win := get_window (winid)
				!!event.make (win, code, char, int1, int2)

				-- execute
				event.execute
			else -- else not registered yet
				check size_unprocessed: code = wrt.Wm_size end
				-- other messages before registration?
			end
		end

feature {NONE} -- Invariant support

	Max_objects: INTEGER is 100 --? debug!!

	no_leaks: BOOLEAN is
		do
			Result := windows.count < Max_objects and 
				controls.count < Max_objects and 
				timers.count < Max_objects
		end

invariant
	no_win_leaks: no_leaks --? debug only

end -- class NOTIFIER
