-- Fenestra; version 0.80(beta); 1998-09-27

-- 
-- Copyright (c) 1998 Franck Arnaud, http://www.altsoft.demon.co.uk/
-- 
-- This package is distributed under the Eiffel Forum Freeware License, 
-- version 1 (see file forum.txt for full text). The main points are:
-- 
-- "Permission is hereby granted, without written agreement and without 
--  license or royalty fees, to use, copy, modify and/or distribute this 
--  package, provided that: copyright notices are retained unchanged [...]
-- 
--  Permission is hereby also granted [...] to distribute binary programs 
--  which depend on this package, provided that: if the binary program 
--  depends on a modified version of this package, you must publicly release 
--  the modified version of this package [...].
-- 
--  THIS PACKAGE IS PROVIDED "AS IS" AND WITHOUT WARRANTY [...]"
-- 

indexing
	cluster: Fenestra, Xcontrols
	description: "Progress bar"
	interface: client

class PROGRESS_BAR

inherit
	CONTROL
		rename
			register as ctl_register,
			set_text as ctl_set_text
		redefine
			tabstop
		end -- inherit DIALOG_ELEMENT indirectly
	CONTROL_WINDOW
		rename
			make_child as make
		redefine
			when_drawn, when_created
		end

creation
	make
	
feature {NONE} -- Attributes

	position: DOUBLE

feature -- Public interface

	set_position (p: DOUBLE) is
			-- Set position of bar.
		require
			ratio_ok: p >= 0 and p <= 1
		local
			cs, r: RECTANGLE
			pt: POINT
		do
			cs := client_size

			if has_bar (cs) then -- redraw if big enough
				r := bar (cs, position)
				!!pt.make
				pt.set_xy (r.upper.x, r.lower.y)
				
				position := p

				r.set_points (pt, bar (cs, position).upper)
				paint_rectangle (r)
			else
				position := p
			end
		end

feature {PUBLIC_NONE}
	
	bar_color: COLOR is
			-- Colour of the progress bar.
		do
			!RGB_COLOR!Result.normal_red
		end
	
	background_color: COLOR is
			-- Colour of the background of the progess bar.
		do
			!RGB_COLOR!Result.light_gray
		end

	tabstop: BOOLEAN is
			-- No keyboard interface.
		do
			Result := false
		end

feature {PUBLIC_NONE} -- Events

	when_created is
			-- Initialise
		do
			-- no border
			set_border (false)

			-- CONTROL_WINDOW init
			set_control (Current)

			-- CONTROL is CHILD_WINDOW
			self := Current
			ctl_register
		end

	when_drawn is
			-- Redrawn.
		local
			rect: RECTANGLE
			dr: DRAWABLE_RECTANGLE
			gr: GRAPHICS_ROUTINES
			d3: DRAW_3D_CONTROLS
		do
			rect := client_size
			if has_bar (rect) then
				!!gr
				!!d3
			
				d3.frame_down (device, rect)
				
				!!dr.make
				dr.set_position (bar (rect, position))
				gr.set_pen_and_brush (device, bar_color)
				device.draw (dr)
			end
		end

feature {NONE}
	
	has_bar (r: RECTANGLE): BOOLEAN is
			-- Is window large enough.
		require
			ok: r /= Void
		do
			Result := r.width > 2 and r.height > 2
		end
	
	bar (cs: RECTANGLE; p: DOUBLE): RECTANGLE is
			-- Progress bar size from window size.
		require
			ok_rect: cs /= Void
			ok_real: p >= 0 and p <= 1
			ok_bar: has_bar (cs)
		local
			pt1, pt2: POINT
			sys: P_SYSTEM
		do 
			!!sys

			pt1 := cs.lower
			pt1.set_xy (pt1.x+1, pt1.y+1)

			pt2 := cs.upper
			pt2.set_xy (pt2.x-1, pt2.y-1)

			!!Result.make
			Result.set_points (pt1, pt2)
			Result.set_width ((sys.floor (p * Result.width)).max (1)) 
		end

end -- class PROGRESS_BAR
