-- Fenestra; version 0.80(beta); 1998-09-27

-- 
-- Copyright (c) 1998 Franck Arnaud, http://www.altsoft.demon.co.uk/
-- 
-- This package is distributed under the Eiffel Forum Freeware License, 
-- version 1 (see file forum.txt for full text). The main points are:
-- 
-- "Permission is hereby granted, without written agreement and without 
--  license or royalty fees, to use, copy, modify and/or distribute this 
--  package, provided that: copyright notices are retained unchanged [...]
-- 
--  Permission is hereby also granted [...] to distribute binary programs 
--  which depend on this package, provided that: if the binary program 
--  depends on a modified version of this package, you must publicly release 
--  the modified version of this package [...].
-- 
--  THIS PACKAGE IS PROVIDED "AS IS" AND WITHOUT WARRANTY [...]"
-- 

indexing
	cluster: Fenestra, Xcontrols
	description: "Split bar separating two child windows inside a frame"
	interface: client

class SPLIT_WINDOW

inherit
	CHILD_WINDOW_CURSOR
		redefine
			when_resized, when_drawn
		end
	P_TEXT_OBJECT
		end

creation
	make_horizontal, make_vertical

feature {CREATION_NONE} -- Creation

	make_horizontal (p: like parent) is
			-- Create horizontal split window.
		local
			mcursor: MOUSE_CURSOR
		do
			is_horizontal := true
			init (p)

			!! mcursor.make_size_west_east
			set_cursor (mcursor)
		end

	make_vertical (p: like parent) is
			-- Create vertical split window.
		local
			mcursor: MOUSE_CURSOR
		do
			is_horizontal := false
			init (p)

			!! mcursor.make_size_north_south
			set_cursor (mcursor)
		end

feature {NONE} -- Effect P_TEXT_OBJECT

	object_to_string (ptm: P_TEXT_OBJECT_MANAGER) is
		do
			ptm.put_integer (1) -- version placeholder
			ptm.put_double (split)
		end

	string_to_object (ptm: P_TEXT_OBJECT_MANAGER) is
		do
			ptm.get_integer
			if ptm.last_integer = 1 then
				ptm.get_double
				if ptm.in_success then
					split := ptm.last_double
				end
			end
		end
	
feature {NONE} -- Attributes
	
	first, second: CHILD_WINDOW
	split: DOUBLE
	
feature {SPLIT_WINDOW_MOUSE}
	
	Split_size: INTEGER is 6
	is_horizontal: BOOLEAN
	mouse: SPLIT_WINDOW_MOUSE --? go local

feature {NONE} -- Initialise
	
	init (p: like parent) is 
		do
			make_child (p)
			!!mouse.make (Current)
			set_left_mouse_manager (mouse)
			set_split (0.5)
			set_border (false)
		end

feature -- Setup
	
	set_first (win: CHILD_WINDOW) is
			-- Set right or top window.
		require
			ok: win /= Void
			child: win.parent = Current
		do
			first := win
			when_resized
		ensure
			keep_reference: first = win
		end

	set_second (win: CHILD_WINDOW) is
			-- Set left or bottom window.
		require
			ok: win /= Void
			child: win.parent = Current
		do
			second := win
			when_resized
		ensure
			keep_reference: second = win
		end

	set_split (newsplit: DOUBLE) is
			-- Set split bar position.
		require
			ok: newsplit >= 0 and newsplit <= 1
		do
			split := newsplit
			when_resized
		end

feature {PUBLIC_NONE}
	
	when_drawn is
		local
			clr: COLOR
			dr: DRAWABLE_RECTANGLE
			gr: GRAPHICS_ROUTINES
		do
			!RGB_COLOR!clr.light_gray
			!!gr

			gr.set_pen_and_brush (device, clr)

			!!dr.make
			dr.set_position (client_size)
			device.draw (dr)
		end

	when_resized is
		local
			cs: RECTANGLE
			low, up: POINT
			middle, actual: INTEGER
			sys: P_SYSTEM
		do
			!!sys
			cs := client_size
			
			if first /= Void and second /= Void and not cs.is_thin then
				!!low.make
				!!up.make

				check 
					size: (is_horizontal implies cs.width > Split_size)
						or else cs.height > Split_size
				end

				if is_horizontal then
					middle := sys.floor (split * cs.width)
					
					-- resize first
					actual := middle - (Split_size//2)
					if actual > 0 then
						up.set_x (actual.min (cs.width-1-Split_size))
						up.set_y (cs.height-1)
						set_and_show (first, low, up) -- low is (0, 0)
					else
						actual := -1
						first.hide
					end

					-- resize second
					actual := actual + Split_size + 1
					if actual < cs.width - 1 then
						low.set_x (actual)
						set_and_show (second, low, cs.upper)
					else
						second.hide
					end
				else -- not is_horizontal
					middle := sys.floor (split * cs.height)

					-- resize first
					actual := middle - (Split_size//2)
					if actual > 0 then
						up.set_x (cs.width-1)
						up.set_y (actual.min (cs.height-1-Split_size))
						set_and_show (first, low, up)
					else
						actual := -1
						first.hide
					end

					-- resize second
					actual := actual + Split_size + 1
					if actual < cs.height - 1 then
						low.set_y (actual)
						set_and_show (second, low, cs.upper)
					else
						second.hide
					end
				end
			end -- has first & second
		end

feature {NONE}
	
	set_and_show (cw: CHILD_WINDOW; one, two: POINT) is
			-- Set window size and make it visible if necessary.
		require
			ok: cw /= Void and one /= Void and two /= Void
		local
			r: RECTANGLE
		do
			!!r.make
			r.set_points (one, two)

			cw.set_size (r)
			
			if not cw.is_visible then
				cw.show
			end
		end

invariant
	split: split >= 0
	
end -- class SPLIT_WINDOW
