-- Fenestra; version 0.80(beta); 1998-09-27

-- 
-- Copyright (c) 1998 Franck Arnaud, http://www.altsoft.demon.co.uk/
-- 
-- This package is distributed under the Eiffel Forum Freeware License, 
-- version 1 (see file forum.txt for full text). The main points are:
-- 
-- "Permission is hereby granted, without written agreement and without 
--  license or royalty fees, to use, copy, modify and/or distribute this 
--  package, provided that: copyright notices are retained unchanged [...]
-- 
--  Permission is hereby also granted [...] to distribute binary programs 
--  which depend on this package, provided that: if the binary program 
--  depends on a modified version of this package, you must publicly release 
--  the modified version of this package [...].
-- 
--  THIS PACKAGE IS PROVIDED "AS IS" AND WITHOUT WARRANTY [...]"
-- 

indexing
	cluster: Fenestra, Xcontrols
	description: "Implementation of mouse button handler for SPLIT_WINDOW"
	interface: internal, client

class SPLIT_WINDOW_MOUSE 

inherit 
	MOUSE_HANDLER
		rename
			make as old_make 
		redefine
			when_dragged, when_drag_finished
		end

creation {SPLIT_WINDOW}
	make

feature {NONE}
	
	splitw: SPLIT_WINDOW
	hot_rectangle: RECTANGLE
	is_horizontal: BOOLEAN
	
feature {NONE}
	
	make (win: SPLIT_WINDOW) is
		do
			splitw := win
			is_horizontal := splitw.is_horizontal
			old_make (win)
		end

feature {NONE}
	
	draw is
		require
			has_hot: hot_rectangle /= Void
		local
			dr: DRAWABLE_GUI_RECTANGLE
			dev: GRAPHIC_DEVICE
		do
			!!dr.make
			dr.set_position (hot_rectangle)
			dr.set_type_invert

			dev := splitw.device
			dev.prepare
			dev.draw (dr)
			dev.release
		end

feature {PUBLIC_NONE}

	when_dragged is
			-- Mouse moved with button down.
		local
			pt: POINT
			cs: RECTANGLE
		do
			cs := splitw.client_size
			!!pt.make

			-- Centre of hot rectangle
			if is_horizontal then
				pt.set_x (point.x)
				pt.set_y (cs.height // 2)
			else
				pt.set_x (cs.width // 2)
				pt.set_y (point.y)
			end

			-- Undraw or create hot rectangle
			if hot_rectangle /= Void then
				draw -- XOR operation, actually undraw
			else
				!!hot_rectangle.make
				if is_horizontal then
					hot_rectangle.set_width (splitw.Split_size)
					hot_rectangle.set_height (cs.height)
				else
					hot_rectangle.set_width (cs.width)
					hot_rectangle.set_height (splitw.Split_size)
				end
			end
			check rectok: hot_rectangle /= Void end

			-- Draw new rectangle
			hot_rectangle.translate_center (pt)
			draw
		end

	when_drag_finished is
			-- Mouse drag finished.
		do
			draw -- undraw
			hot_rectangle := Void

			if is_horizontal then
				set_split (point.x, splitw.client_size.width)
			else
				set_split (point.y, splitw.client_size.height)
			end
		end

feature {NONE}
	
	set_split (s, max: INTEGER) is
			-- Synch split window.
		require
			max_pos: max > 0
		local
			new_split: DOUBLE
		do
			if s > max then
				new_split := 1.0
			elseif s > 0 then
				new_split := s / max
			end -- else s <= 0 and new_split = 0
		
			splitw.set_split (new_split)
		end

end -- class SPLIT_WINDOW_MOUSE
